package net.minecraft.client.gui;

import java.io.IOException;
import java.io.DataInput;
import com.nbt.NBTBase;
import java.io.DataInputStream;
import java.util.zip.GZIPInputStream;
import com.nbt.NBTTagCompound;
import java.io.InputStream;
import org.lwjgl.opengl.Display;
import net.minecraft.client.renderer.Tessellator;
import org.lwjgl.opengl.GL11;
import net.minecraft.client.MinecraftError;
import net.minecraft.client.Minecraft;

public class GuiLevelLoading implements GuiLoading {
    private String text;
    private Minecraft mc;
    private String title;
    private long start;
    
    public GuiLevelLoading(final Minecraft d) {
        this.text = "";
        this.title = "";
        this.start = System.currentTimeMillis();
        this.mc = d;
    }
    
    public final void setTitle(final String title) {
        if (!this.mc.running) {
            throw new MinecraftError();
        }
        this.title = title;
        final ScaledResolution scaledResolution;
        final int scaledWidth = (scaledResolution = new ScaledResolution(this.mc.displayWidth, this.mc.displayHeight)).getScaledWidth();
        final int scaledHeight = scaledResolution.getScaledHeight();
        GL11.glClear(256);
        GL11.glMatrixMode(5889);
        GL11.glLoadIdentity();
        GL11.glOrtho(0.0, (double)scaledWidth, (double)scaledHeight, 0.0, 100.0, 300.0);
        GL11.glMatrixMode(5888);
        GL11.glLoadIdentity();
        GL11.glTranslated(0.0f, 0.0f, -200.0f);
    }
    
    public final void setText(final String text) {
        if (!this.mc.running) {
            throw new MinecraftError();
        }
        this.start = 0L;
        this.text = text;
        this.setProgress(-1);
        this.start = 0L;
    }
    
    public final void setProgress(final int progress) {
        if (!this.mc.running) {
            throw new MinecraftError();
        }
        final long currentTimeMillis;
        if ((currentTimeMillis = System.currentTimeMillis()) - this.start < 20L) {
            return;
        }
        this.start = currentTimeMillis;
        final ScaledResolution scaledResolution;
        final int scaledWidth = (scaledResolution = new ScaledResolution(this.mc.displayWidth, this.mc.displayHeight)).getScaledWidth();
        final int scaledHeight = scaledResolution.getScaledHeight();
        GL11.glClear(256);
        GL11.glMatrixMode(5889);
        GL11.glLoadIdentity();
        GL11.glOrtho(0.0, (double)scaledWidth, (double)scaledHeight, 0.0, 100.0, 300.0);
        GL11.glMatrixMode(5888);
        GL11.glLoadIdentity();
        GL11.glTranslated(0.0f, 0.0f, -200.0f);
        GL11.glClear(16640);
        final Tessellator instance = Tessellator.instance;
        int texture = this.mc.renderEngine.getTexture("/dirt.png");
        GL11.glBindTexture(3553, texture);
        instance.startDrawingQuads();
        instance.setColorOpaque_I(4210752);
        instance.addVertexWithUV(0.0f, (double)scaledHeight, 0.0f, 0.0f, scaledHeight / 32.0f);
        instance.addVertexWithUV((double)scaledWidth, (double)scaledHeight, 0.0f, scaledWidth / 32.0f, scaledHeight / 32.0f);
        instance.addVertexWithUV((double)scaledWidth, 0.0f, 0.0f, scaledWidth / 32.0f, 0.0f);
        instance.addVertexWithUV(0.0f, 0.0f, 0.0f, 0.0f, 0.0f);
        instance.draw();
        if (progress >= 0) {
            texture = scaledWidth / 2 - 50;
            final int n = scaledHeight / 2 + 16;
            GL11.glDisable(3553);
            instance.startDrawingQuads();
            instance.setColorOpaque_I(8421504);
            instance.addVertex((double)texture, (double)n, 0.0f);
            instance.addVertex((double)texture, (double)(n + 2), 0.0f);
            instance.addVertex((double)(texture + 100), (double)(n + 2), 0.0f);
            instance.addVertex((double)(texture + 100), (double)n, 0.0f);
            instance.setColorOpaque_I(8454016);
            instance.addVertex((double)texture, (double)n, 0.0f);
            instance.addVertex((double)texture, (double)(n + 2), 0.0f);
            instance.addVertex((double)(texture + progress), (double)(n + 2), 0.0f);
            instance.addVertex((double)(texture + progress), (double)n, 0.0f);
            instance.draw();
            GL11.glEnable(3553);
        }
        this.mc.fontRenderer.drawStringWithShadow(this.title, (scaledWidth - this.mc.fontRenderer.getStringWidth(this.title)) / 2, scaledHeight / 2 - 4 - 16, 16777215);
        this.mc.fontRenderer.drawStringWithShadow(this.text, (scaledWidth - this.mc.fontRenderer.getStringWidth(this.text)) / 2, scaledHeight / 2 - 4 + 8, 16777215);
        Display.update();
        try {
            Thread.yield();
        }
        catch (Exception ex) {}
    }
    
    public GuiLevelLoading() {
    }
    
    public static NBTTagCompound writeLevelTags(InputStream var_0_0F) throws IOException {
        var_0_0F = (InputStream)new DataInputStream((InputStream)new GZIPInputStream(var_0_0F));
        try {
            final NBTBase tag;
            if ((tag = NBTBase.readTag((DataInput)var_0_0F)) instanceof NBTTagCompound) {
                return (NBTTagCompound)tag;
            }
            throw new IOException("Root tag must be a named compound tag");
        }
        finally {
            ((DataInputStream)var_0_0F).close();
        }
    }
}
