package net.minecraft.client.particle;

import net.minecraft.game.block.Block;
import net.minecraft.client.renderer.Tessellator;
import org.lwjgl.opengl.GL11;
import net.minecraft.game.MathHelper;
import net.minecraft.game.entity.Entity;
import java.util.ArrayList;
import java.util.Random;
import net.minecraft.client.renderer.RenderEngine;
import java.util.List;
import net.minecraft.game.level.World;

public final class EffectRenderer {
    private World worldObj;
    private List[] fxLayers;
    private RenderEngine renderer;
    private Random rand;
    
    public EffectRenderer(final World world, final RenderEngine renderEngine) {
        this.fxLayers = new List[3];
        this.rand = new Random();
        if (world != null) {
            this.worldObj = world;
        }
        this.renderer = renderEngine;
        for (int i = 0; i < 3; ++i) {
            this.fxLayers[i] = (List)new ArrayList();
        }
    }
    
    public final void addEffect(final EntityFX entityFX) {
        this.fxLayers[entityFX.getFXLayer()].add(entityFX);
    }
    
    public final void updateEffects() {
        for (int i = 0; i < 3; ++i) {
            for (int j = 0; j < this.fxLayers[i].size(); ++j) {
                final EntityFX entityFX;
                (entityFX = (EntityFX)this.fxLayers[i].get(j)).onEntityUpdate();
                if (entityFX.isDead) {
                    this.fxLayers[i].remove(j--);
                }
            }
        }
    }
    
    public final void renderParticles(final Entity particle, final double double2) {
        final double cos = MathHelper.cos(particle.rotationYaw * 3.1415927f / 180.0f);
        final double sin;
        final double double3 = -(sin = MathHelper.sin(particle.rotationYaw * 3.1415927f / 180.0f)) * MathHelper.sin(particle.rotationPitch * 3.1415927f / 180.0f);
        final double double4 = cos * MathHelper.sin(particle.rotationPitch * 3.1415927f / 180.0f);
        final double cos2 = MathHelper.cos(particle.rotationPitch * 3.1415927f / 180.0f);
        for (int i = 0; i < 2; ++i) {
            if (this.fxLayers[i].size() != 0) {
                int n = 0;
                if (i == 0) {
                    n = this.renderer.getTexture("/particles.png");
                }
                if (i == 1) {
                    n = this.renderer.getTexture("/terrain.png");
                }
                GL11.glBindTexture(3553, n);
                final Tessellator instance;
                (instance = Tessellator.instance).startDrawingQuads();
                for (int j = 0; j < this.fxLayers[i].size(); ++j) {
                    ((EntityFX)this.fxLayers[i].get(j)).renderParticle(instance, double2, cos, cos2, sin, double3, double4);
                }
                instance.draw();
            }
        }
    }
    
    public final void renderLitParticles(final double double1) {
        if (this.fxLayers[2].size() == 0) {
            return;
        }
        final Tessellator instance = Tessellator.instance;
        for (int i = 0; i < this.fxLayers[2].size(); ++i) {
            ((EntityFX)this.fxLayers[2].get(i)).renderParticle(instance, double1, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f);
        }
    }
    
    public final void clearEffects(final World world) {
        this.worldObj = world;
        for (int i = 0; i < 3; ++i) {
            this.fxLayers[i].clear();
        }
    }
    
    public final void addBlockDestroyEffects(final long integer1, final long integer2, final long integer3) {
        final int blockId;
        if ((blockId = this.worldObj.getBlockId(integer1, integer2, integer3)) == 0) {
            return;
        }
        final Block x = Block.blocksList[blockId];
        for (int i = 0; i < 4; ++i) {
            for (int j = 0; j < 4; ++j) {
                for (int k = 0; k < 4; ++k) {
                    final double double2 = integer1 + (i + 0.5f) / 4.0f;
                    final double double3 = integer2 + (j + 0.5f) / 4.0f;
                    final double double4 = integer3 + (k + 0.5f) / 4.0f;
                    this.addEffect(new EntityDiggingFX(this.worldObj, double2, double3, double4, double2 - integer1 - 0.5f, double3 - integer2 - 0.5f, double4 - integer3 - 0.5f, x));
                }
            }
        }
    }
    
    public final void addBlockHitEffects(final long blockX, final long blockY, final long blockZ, final int integer4) {
        final int blockId;
        if ((blockId = this.worldObj.getBlockId(blockX, blockY, blockZ)) == 0) {
            return;
        }
        final Block x = Block.blocksList[blockId];
        double double2 = blockX + this.rand.nextDouble() * (x.maxX - x.minX - 0.2f) + 0.1f + x.minX;
        double double3 = blockY + this.rand.nextDouble() * (x.maxY - x.minY - 0.2f) + 0.1f + x.minY;
        double double4 = blockZ + this.rand.nextDouble() * (x.maxZ - x.minZ - 0.2f) + 0.1f + x.minZ;
        if (integer4 == 0) {
            double3 = blockY + x.minY - 0.1f;
        }
        if (integer4 == 1) {
            double3 = blockY + x.maxY + 0.1f;
        }
        if (integer4 == 2) {
            double4 = blockZ + x.minZ - 0.1f;
        }
        if (integer4 == 3) {
            double4 = blockZ + x.maxZ + 0.1f;
        }
        if (integer4 == 4) {
            double2 = blockX + x.minX - 0.1f;
        }
        if (integer4 == 5) {
            double2 = blockX + x.maxX + 0.1f;
        }
        this.addEffect(new EntityDiggingFX(this.worldObj, double2, double3, double4, 0.0f, 0.0f, 0.0f, x).multiplyVelocity(0.2f).multipleParticleScaleBy(0.6f));
    }
    
    public final String getStatistics() {
        return new StringBuilder().append("").append(this.fxLayers[0].size() + this.fxLayers[1].size() + this.fxLayers[2].size()).toString();
    }
}
