package net.minecraft.client.particle;

import net.minecraft.client.renderer.Tessellator;
import net.minecraft.game.level.World;

public final class EntitySmokeFX extends EntityFX {
    private double smokeParticleScale;
    
    public EntitySmokeFX(final World world, final double double2, final double double3, final double double4) {
        this(world, double2, double3, double4, 1.0f);
    }
    
    public EntitySmokeFX(final World world, final double double2, final double double3, final double double4, final double double5) {
        super(world, double2, double3, double4, 0.0f, 0.0f, 0.0f);
        this.motionX1 *= 0.1f;
        this.motionY1 *= 0.1f;
        this.motionZ1 *= 0.1f;
        final double particleRed = (double)(Math.random() * 0.30000001192092896);
        this.particleBlue = particleRed;
        this.particleGreen = particleRed;
        this.particleRed = particleRed;
        this.particleScale *= 0.75f;
        this.particleScale *= double5;
        this.smokeParticleScale = this.particleScale;
        this.particleMaxAge = (int)(8.0 / (Math.random() * 0.8 + 0.2));
        this.particleMaxAge *= (int)double5;
        this.noClip = false;
    }
    
    @Override
    public final void renderParticle(final Tessellator tessellator, final double double2, final double double3, final double double4, final double double5, final double double6, final double double7) {
        double n;
        if ((n = (this.particleAge + double2) / this.particleMaxAge * 32.0f) < 0.0f) {
            n = 0.0f;
        }
        if (n > 1.0f) {
            n = 1.0f;
        }
        this.particleScale = this.smokeParticleScale * n;
        super.renderParticle(tessellator, double2, double3, double4, double5, double6, double7);
    }
    
    @Override
    public final void onEntityUpdate() {
        this.prevPosX = this.posX;
        this.prevPosY = this.posY;
        this.prevPosZ = this.posZ;
        if (this.particleAge++ >= this.particleMaxAge) {
            this.setEntityDead();
        }
        this.particleTextureIndex = 7 - (this.particleAge << 3) / this.particleMaxAge;
        this.motionY1 += (double)0.004;
        this.moveEntity(this.motionX1, this.motionY1, this.motionZ1);
        if (this.posY == this.prevPosY) {
            this.motionX1 *= (double)1.1;
            this.motionZ1 *= (double)1.1;
        }
        this.motionX1 *= 0.96f;
        this.motionY1 *= 0.96f;
        this.motionZ1 *= 0.96f;
        if (this.onGround) {
            this.motionX1 *= 0.7f;
            this.motionZ1 *= 0.7f;
        }
    }
}
