package net.minecraft.client.physics;

import net.minecraft.client.renderer.Vec3D;

public final class AxisAlignedBB {
    private double epsilon;
    public double minX;
    public double minY;
    public double minZ;
    public double maxX;
    public double maxY;
    public double maxZ;
    
    public AxisAlignedBB(final double minX, final double minY, final double minZ, final double maxX, final double maxY, final double maxZ) {
        this.epsilon = 0.0f;
        this.minX = minX;
        this.minY = minY;
        this.minZ = minZ;
        this.maxX = maxX;
        this.maxY = maxY;
        this.maxZ = maxZ;
    }
    
    public final AxisAlignedBB addCoord(final double double1, final double double2, final double double3) {
        double minX = this.minX;
        double minY = this.minY;
        double minZ = this.minZ;
        double maxX = this.maxX;
        double maxY = this.maxY;
        double maxZ = this.maxZ;
        if (double1 < 0.0f) {
            minX += double1;
        }
        if (double1 > 0.0f) {
            maxX += double1;
        }
        if (double2 < 0.0f) {
            minY += double2;
        }
        if (double2 > 0.0f) {
            maxY += double2;
        }
        if (double3 < 0.0f) {
            minZ += double3;
        }
        if (double3 > 0.0f) {
            maxZ += double3;
        }
        return new AxisAlignedBB(minX, minY, minZ, maxX, maxY, maxZ);
    }
    
    public final AxisAlignedBB expand(double double1, double double2, double double3) {
        if (this.minY > this.maxY) {
            throw new IllegalArgumentException("NOOOOOO!");
        }
        final double minX = this.minX - double1;
        final double minY = this.minY - double2;
        final double minZ = this.minZ - double3;
        double1 += this.maxX;
        double2 += this.maxY;
        double3 += this.maxZ;
        return new AxisAlignedBB(minX, minY, minZ, double1, double2, double3);
    }
    
    public final double calculateXOffset(final AxisAlignedBB aabb, double double2) {
        if (aabb.maxY <= this.minY || aabb.minY >= this.maxY) {
            return double2;
        }
        if (aabb.maxZ <= this.minZ || aabb.minZ >= this.maxZ) {
            return double2;
        }
        double n;
        if (double2 > 0.0f && aabb.maxX <= this.minX && (n = this.minX - aabb.maxX) < double2) {
            double2 = n;
        }
        if (double2 < 0.0f && aabb.minX >= this.maxX && (n = this.maxX - aabb.minX) > double2) {
            double2 = n;
        }
        return double2;
    }
    
    public final double calculateYOffset(final AxisAlignedBB aabb, double double2) {
        if (aabb.maxX <= this.minX || aabb.minX >= this.maxX) {
            return double2;
        }
        if (aabb.maxZ <= this.minZ || aabb.minZ >= this.maxZ) {
            return double2;
        }
        double n;
        if (double2 > 0.0f && aabb.maxY <= this.minY && (n = this.minY - aabb.maxY) < double2) {
            double2 = n;
        }
        if (double2 < 0.0f && aabb.minY >= this.maxY && (n = this.maxY - aabb.minY) > double2) {
            double2 = n;
        }
        return double2;
    }
    
    public final double calculateZOffset(final AxisAlignedBB aabb, double double2) {
        if (aabb.maxX <= this.minX || aabb.minX >= this.maxX) {
            return double2;
        }
        if (aabb.maxY <= this.minY || aabb.minY >= this.maxY) {
            return double2;
        }
        double n;
        if (double2 > 0.0f && aabb.maxZ <= this.minZ && (n = this.minZ - aabb.maxZ) < double2) {
            double2 = n;
        }
        if (double2 < 0.0f && aabb.minZ >= this.maxZ && (n = this.maxZ - aabb.minZ) > double2) {
            double2 = n;
        }
        return double2;
    }
    
    public final boolean intersectsWith(final AxisAlignedBB aabb) {
        return aabb.maxX >= this.minX && aabb.minX <= this.maxX && aabb.maxY >= this.minY && aabb.minY <= this.maxY && aabb.maxZ >= this.minZ && aabb.minZ <= this.maxZ;
    }
    
    public final void offset(final double double1, final double double2, final double double3) {
        this.minX += double1;
        this.minY += double2;
        this.minZ += double3;
        this.maxX += double1;
        this.maxY += double2;
        this.maxZ += double3;
    }
    
    public final AxisAlignedBB copy() {
        return new AxisAlignedBB(this.minX, this.minY, this.minZ, this.maxX, this.maxY, this.maxZ);
    }
    
    public final MovingObjectPosition calculateIntercept(final Vec3D vec3D, Vec3D vec3D2) {
        Vec3D intermediateWithXValue = vec3D.getIntermediateWithXValue(vec3D2, this.minX);
        Vec3D intermediateWithXValue2 = vec3D.getIntermediateWithXValue(vec3D2, this.maxX);
        Vec3D intermediateWithYValue = vec3D.getIntermediateWithYValue(vec3D2, this.minY);
        Vec3D intermediateWithYValue2 = vec3D.getIntermediateWithYValue(vec3D2, this.maxY);
        Vec3D intermediateWithZValue = vec3D.getIntermediateWithZValue(vec3D2, this.minZ);
        vec3D2 = vec3D.getIntermediateWithZValue(vec3D2, this.maxZ);
        if (!this.isVecInYZ(intermediateWithXValue)) {
            intermediateWithXValue = null;
        }
        if (!this.isVecInYZ(intermediateWithXValue2)) {
            intermediateWithXValue2 = null;
        }
        if (!this.isVecInXZ(intermediateWithYValue)) {
            intermediateWithYValue = null;
        }
        if (!this.isVecInXZ(intermediateWithYValue2)) {
            intermediateWithYValue2 = null;
        }
        if (!this.isVecInXY(intermediateWithZValue)) {
            intermediateWithZValue = null;
        }
        if (!this.isVecInXY(vec3D2)) {
            vec3D2 = null;
        }
        Vec3D vec3D3 = null;
        if (intermediateWithXValue != null) {
            vec3D3 = intermediateWithXValue;
        }
        if (intermediateWithXValue2 != null && (vec3D3 == null || vec3D.squareDistanceTo(intermediateWithXValue2) < vec3D.squareDistanceTo(vec3D3))) {
            vec3D3 = intermediateWithXValue2;
        }
        if (intermediateWithYValue != null && (vec3D3 == null || vec3D.squareDistanceTo(intermediateWithYValue) < vec3D.squareDistanceTo(vec3D3))) {
            vec3D3 = intermediateWithYValue;
        }
        if (intermediateWithYValue2 != null && (vec3D3 == null || vec3D.squareDistanceTo(intermediateWithYValue2) < vec3D.squareDistanceTo(vec3D3))) {
            vec3D3 = intermediateWithYValue2;
        }
        if (intermediateWithZValue != null && (vec3D3 == null || vec3D.squareDistanceTo(intermediateWithZValue) < vec3D.squareDistanceTo(vec3D3))) {
            vec3D3 = intermediateWithZValue;
        }
        if (vec3D2 != null && (vec3D3 == null || vec3D.squareDistanceTo(vec3D2) < vec3D.squareDistanceTo(vec3D3))) {
            vec3D3 = vec3D2;
        }
        if (vec3D3 == null) {
            return null;
        }
        int sideHit = -1;
        if (vec3D3 == intermediateWithXValue) {
            sideHit = 4;
        }
        if (vec3D3 == intermediateWithXValue2) {
            sideHit = 5;
        }
        if (vec3D3 == intermediateWithYValue) {
            sideHit = 0;
        }
        if (vec3D3 == intermediateWithYValue2) {
            sideHit = 1;
        }
        if (vec3D3 == intermediateWithZValue) {
            sideHit = 2;
        }
        if (vec3D3 == vec3D2) {
            sideHit = 3;
        }
        return new MovingObjectPosition(0, 0, 0, sideHit, vec3D3);
    }
    
    private boolean isVecInYZ(final Vec3D vev3D) {
        return vev3D != null && (vev3D.yCoord >= this.minY && vev3D.yCoord <= this.maxY && vev3D.zCoord >= this.minZ && vev3D.zCoord <= this.maxZ);
    }
    
    private boolean isVecInXZ(final Vec3D vec3D) {
        return vec3D != null && (vec3D.xCoord >= this.minX && vec3D.xCoord <= this.maxX && vec3D.zCoord >= this.minZ && vec3D.zCoord <= this.maxZ);
    }
    
    private boolean isVecInXY(final Vec3D vec3D) {
        return vec3D != null && (vec3D.xCoord >= this.minX && vec3D.xCoord <= this.maxX && vec3D.yCoord >= this.minY && vec3D.yCoord <= this.maxY);
    }
}
