package net.minecraft.client.renderer;

import net.minecraft.game.block.Material;
import net.minecraft.client.particle.EffectRenderer;
import java.util.List;
import net.minecraft.game.block.Block;
import org.lwjgl.util.glu.GLU;
import net.minecraft.client.physics.MovingObjectPosition;
import net.minecraft.client.player.PlayerControllerCreative;
import java.awt.image.DataBufferInt;
import net.minecraft.game.level.World;
import java.awt.Graphics;
import java.io.OutputStream;
import java.awt.image.RenderedImage;
import javax.imageio.ImageIO;
import java.io.FileOutputStream;
import java.awt.image.ImageObserver;
import java.awt.Image;
import net.minecraft.game.entity.player.EntityPlayer;
import java.awt.image.BufferedImage;
import java.io.File;
import net.minecraft.client.gui.ScaledResolution;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.Display;
import net.minecraft.game.MathHelper;
import org.lwjgl.opengl.GL11;
import net.minecraft.client.player.EntityPlayerSP;
import org.lwjgl.BufferUtils;
import java.util.Random;
import java.nio.DoubleBuffer;
import java.nio.FloatBuffer;
import java.nio.ByteBuffer;
import java.text.DecimalFormat;
import net.minecraft.game.entity.Entity;
import net.minecraft.client.Minecraft;

public final class EntityRenderer {
    private Minecraft mc;
    private boolean anaglyphEnable;
    private double farPlaneDistance;
    public ItemRenderer itemRenderer;
    private int rendererUpdateCount;
    private Entity pointedEntity;
    private int entityRendererInt1;
    private int entityRendererInt2;
    private DecimalFormat entityDecimalFormat;
    private ByteBuffer entityByteBuffer;
    private DoubleBuffer entityDoubleBuffer;
    private Random random;
    private volatile int unusedVolatile0;
    private volatile int unusedVolatile1;
    private FloatBuffer fogColorBuffer;
    private double fogColorRed;
    private double fogColorGreen;
    private double fogColorBlue;
    private double fogColor2;
    private double fogColor1;
    
    public EntityRenderer(final Minecraft minecraft) {
        this.anaglyphEnable = false;
        this.farPlaneDistance = 0.0f;
        this.pointedEntity = null;
        this.entityDecimalFormat = new DecimalFormat("0000");
        this.entityDoubleBuffer = BufferUtils.createDoubleBuffer(16);
        this.random = new Random();
        this.unusedVolatile0 = 0;
        this.unusedVolatile1 = 0;
        this.fogColorBuffer = BufferUtils.createFloatBuffer(16);
        this.mc = minecraft;
        this.itemRenderer = new ItemRenderer(minecraft);
    }
    
    public final void updateRenderer() {
        this.fogColor2 = this.fogColor1;
        double lightBrightness = this.mc.theWorld.getLightBrightness((int)this.mc.thePlayer.posX, (int)this.mc.thePlayer.posY, (int)this.mc.thePlayer.posZ);
        final double n = (3 - this.mc.gameSettings.renderDistance) / 3.0f;
        lightBrightness = lightBrightness * (1.0f - n) + n;
        this.fogColor1 += (lightBrightness - this.fogColor1) * 0.1f;
        ++this.rendererUpdateCount;
        this.itemRenderer.updateEquippedItem();
    }
    
    private Vec3D orientCamera(double double1) {
        final EntityPlayerSP thePlayer;
        final double double2 = (thePlayer = this.mc.thePlayer).prevPosX + (thePlayer.posX - thePlayer.prevPosX) * double1;
        final double double3 = thePlayer.prevPosY + (thePlayer.posY - thePlayer.prevPosY) * double1;
        double1 = thePlayer.prevPosZ + (thePlayer.posZ - thePlayer.prevPosZ) * double1;
        return new Vec3D(double2, double3, double1);
    }
    
    private void hurtCameraEffect(double double1) {
        final EntityPlayerSP thePlayer;
        double sin = (thePlayer = this.mc.thePlayer).hurtTime - double1;
        if (thePlayer.health <= 0) {
            double1 += thePlayer.deathTime;
            GL11.glRotated(40.0f - 8000.0f / (double1 + 200.0f), 0.0f, 0.0f, 1.0f);
        }
        if (sin < 0.0f) {
            return;
        }
        sin = MathHelper.sin((sin /= thePlayer.maxHurtTime) * sin * sin * sin * 3.1415927f);
        GL11.glRotated(-(double1 = thePlayer.attackedAtYaw), 0.0f, 1.0f, 0.0f);
        GL11.glRotated(-sin * 14.0f, 0.0f, 0.0f, 1.0f);
        GL11.glRotated(double1, 0.0f, 1.0f, 0.0f);
    }
    
    private void setupViewBobbing(double double1) {
        if (this.mc.gameSettings.thirdPersonView) {
            return;
        }
        final EntityPlayerSP thePlayer;
        double n = (thePlayer = this.mc.thePlayer).distanceWalkedModified - thePlayer.prevDistanceWalkedModified;
        n = thePlayer.distanceWalkedModified + n * double1;
        final double n2 = thePlayer.prevCameraYaw + (thePlayer.cameraYaw - thePlayer.prevCameraYaw) * double1;
        double1 = thePlayer.prevCameraPitch + (thePlayer.cameraPitch - thePlayer.prevCameraPitch) * double1;
        GL11.glTranslated(MathHelper.sin(n * 3.1415927f) * n2 * 0.5f, -Math.abs(MathHelper.cos(n * 3.1415927f) * n2), 0.0f);
        GL11.glRotated(MathHelper.sin(n * 3.1415927f) * n2 * 3.0f, 0.0f, 0.0f, 1.0f);
        GL11.glRotated(Math.abs(MathHelper.cos(n * 3.1415927f + 0.2f) * n2) * 5.0f, 1.0f, 0.0f, 0.0f);
        GL11.glRotated(double1, 1.0f, 0.0f, 0.0f);
    }
    
    public final void updateCameraAndRender(double double1) {
    	double1 = 1;
        if (this.anaglyphEnable && !Display.isActive()) {
            this.mc.displayInGameMenu();
        }
        this.anaglyphEnable = Display.isActive();
        if (this.mc.inventoryScreen) {
            Mouse.getDX();
            final int entityRendererInt1 = 0;
            Mouse.getDY();
            final int entityRendererInt2 = 0;
            this.mc.mouseHelper.ungrabMouseCursor();
            int scaledHeight = 1;
            if (this.mc.gameSettings.invertMouse) {
                scaledHeight = -1;
            }
            final int xSize_lo = entityRendererInt1 + this.mc.mouseHelper.deltaX;
            final int ySize_lo = entityRendererInt2 - this.mc.mouseHelper.deltaY;
            if (entityRendererInt1 != 0 || this.entityRendererInt1 != 0) {
                System.out.println(new StringBuilder().append("xxo: ").append(entityRendererInt1).append(", ").append(this.entityRendererInt1).append(": ").append(this.entityRendererInt1).append(", xo: ").append(xSize_lo).toString());
            }
            if (this.entityRendererInt1 != 0) {
                this.entityRendererInt1 = 0;
            }
            if (this.entityRendererInt2 != 0) {
                this.entityRendererInt2 = 0;
            }
            if (entityRendererInt1 != 0) {
                this.entityRendererInt1 = entityRendererInt1;
            }
            if (entityRendererInt2 != 0) {
                this.entityRendererInt2 = entityRendererInt2;
            }
            final EntityPlayerSP thePlayer = this.mc.thePlayer;
            final double n = (double)xSize_lo;
            final double n2 = (double)(ySize_lo * scaledHeight);
            final double n3 = n;
            final EntityPlayerSP entityPlayerSP = thePlayer;
            final double rotationPitch = thePlayer.rotationPitch;
            final double rotationYaw = entityPlayerSP.rotationYaw;
            final EntityPlayerSP entityPlayerSP2 = entityPlayerSP;
            entityPlayerSP2.rotationYaw += (double)(n3 * 0.15);
            final EntityPlayerSP entityPlayerSP3 = entityPlayerSP;
            entityPlayerSP3.rotationPitch -= (double)(n2 * 0.15);
            if (entityPlayerSP.rotationPitch < -90.0f) {
                entityPlayerSP.rotationPitch = -90.0f;
            }
            if (entityPlayerSP.rotationPitch > 90.0f) {
                entityPlayerSP.rotationPitch = 90.0f;
            }
            final EntityPlayerSP entityPlayerSP4 = entityPlayerSP;
            entityPlayerSP4.prevRotationPitch += entityPlayerSP.rotationPitch - rotationPitch;
            final EntityPlayerSP entityPlayerSP5 = entityPlayerSP;
            entityPlayerSP5.prevRotationYaw += entityPlayerSP.rotationYaw - rotationYaw;
        }
        final ScaledResolution scaledResolution;
        final int scaledWidth = (scaledResolution = new ScaledResolution(this.mc.displayWidth, this.mc.displayHeight)).getScaledWidth();
        int scaledHeight = scaledResolution.getScaledHeight();
        final int xSize_lo = Mouse.getX() * scaledWidth / this.mc.displayWidth;
        final int ySize_lo = scaledHeight - Mouse.getY() * scaledHeight / this.mc.displayHeight - 1;
        if (this.mc.theWorld != null) {
            this.getMouseOver(double1);
            this.mc.ingameGUI.renderGameOverlay(double1);
        }
        else {
            GL11.glViewport(0, 0, this.mc.displayWidth, this.mc.displayHeight);
            GL11.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
            GL11.glClear(16640);
            GL11.glMatrixMode(5889);
            GL11.glLoadIdentity();
            GL11.glMatrixMode(5888);
            GL11.glLoadIdentity();
            this.setupOverlayRendering();
        }
        if (this.mc.currentScreen != null) {
            GL11.glClear(256);
            this.mc.currentScreen.drawScreen(xSize_lo, ySize_lo, double1);
        }
        Thread.yield();
        Display.update();
    }
    
    public final void grabLargeScreenshot() {
        this.mc.loadingScreen.setTitle("Grabbing large screenshot");
        final File file = new File(".");
        File file2;
        for (int n = 0; (file2 = new File(file, "mc_map_" + this.entityDecimalFormat.format((long)n) + ".png")).exists(); ++n) {}
        final File absoluteFile = file2.getAbsoluteFile();
        this.mc.loadingScreen.setText("Rendering");
        this.mc.loadingScreen.setProgress(0);
        try {
            final int n2 = (this.mc.theWorld.width << 4) + (this.mc.theWorld.length << 4);
            final int n = (this.mc.theWorld.height << 4) + n2 / 2;
            final BufferedImage bufferedImage;
            final Graphics graphics = (bufferedImage = new BufferedImage(n2, n, 1)).getGraphics();
            final int displayWidth = this.mc.displayWidth;
            final int displayHeight = this.mc.displayHeight;
            final int n3 = (n2 / displayWidth + 1) * (n / displayHeight + 1);
            int n4 = 0;
            for (int i = 0; i < n2; i += displayWidth) {
                for (int j = 0; j < n; j += displayHeight) {
                    ++n4;
                    this.mc.loadingScreen.setProgress(n4 * 100 / n3);
                    final int n5 = i - n2;
                    final int n6 = j - n / 2;
                    int sortAndRender = n5;
                    if (this.entityByteBuffer == null) {
                        this.entityByteBuffer = BufferUtils.createByteBuffer(this.mc.displayWidth * this.mc.displayHeight << 2);
                    }
                    final EntityPlayerSP thePlayer = this.mc.thePlayer;
                    final World theWorld = this.mc.theWorld;
                    final RenderGlobal renderGlobal = this.mc.renderGlobal;
                    GL11.glViewport(0, 0, this.mc.displayWidth, this.mc.displayHeight);
                    this.updateFogColor(0.0f);
                    GL11.glClear(16640);
                    GL11.glEnable(2884);
                    this.farPlaneDistance = (double)(512 >> (this.mc.gameSettings.renderDistance << 1));
                    GL11.glMatrixMode(5889);
                    GL11.glLoadIdentity();
                    GL11.glOrtho(0.0, (double)this.mc.displayWidth, 0.0, (double)this.mc.displayHeight, 10.0, 10000.0);
                    GL11.glMatrixMode(5888);
                    GL11.glLoadIdentity();
                    GL11.glTranslated((double)(-sortAndRender), (double)(-n6), -5000.0f);
                    GL11.glScaled(16.0f, -16.0f, -16.0f);
                    this.entityDoubleBuffer.clear();
                    this.entityDoubleBuffer.put(1.0f).put(-0.5f).put(0.0f).put(0.0f);
                    this.entityDoubleBuffer.put(0.0f).put(1.0f).put(-1.0f).put(0.0f);
                    this.entityDoubleBuffer.put(1.0f).put(0.5f).put(0.0f).put(0.0f);
                    this.entityDoubleBuffer.put(0.0f).put(0.0f).put(0.0f).put(1.0f);
                    this.entityDoubleBuffer.flip();
                    GL11.glMultMatrix(this.entityDoubleBuffer);
                    GL11.glRotated(0.0f, 0.0f, 1.0f, 0.0f);
                    GL11.glTranslated(-theWorld.width / 2.0f, -theWorld.height / 2.0f, -theWorld.length / 2.0f);
                    final Frustrum2 frustrum2 = new Frustrum2();
                    this.mc.renderGlobal.clipRenderersByFrustrum(frustrum2);
                    this.mc.renderGlobal.updateRenderers(thePlayer);
                    this.setupFog();
                    GL11.glEnable(2912);
                    GL11.glFogi(2917, 9729);
                    final double n7 = theWorld.height * 8.0f;
                    GL11.glFogf(2915, 5000.0f - (float)n7);
                    GL11.glFogf(2916, 5000.0f + (float)n7 * 8.0f);
                    RenderHelper.enableStandardItemLighting();
                    renderGlobal.renderEntities(this.orientCamera(0.0f), frustrum2, 0.0f);
                    RenderHelper.disableStandardItemLighting();
                    GL11.glBindTexture(3553, this.mc.renderEngine.getTexture("/terrain.png"));
                    renderGlobal.sortAndRender(thePlayer, 0);
                    renderGlobal.oobGroundRenderer();
                    if (theWorld.cloudHeight < theWorld.height) {
                        renderGlobal.renderSky(0.0f);
                    }
                    GL11.glEnable(3042);
                    GL11.glBlendFunc(770, 771);
                    GL11.glColorMask(false, false, false, false);
                    renderGlobal.sortAndRender(thePlayer, 1);
                    GL11.glColorMask(true, true, true, true);
                    if (sortAndRender > 0) {
                        renderGlobal.renderAllRenderLists();
                    }
                    if (theWorld.getGroundLevel() >= 0) {
                        renderGlobal.oobWaterRenderer();
                    }
                    GL11.glDepthMask(true);
                    GL11.glDisable(3042);
                    GL11.glDisable(2912);
                    this.entityByteBuffer.clear();
                    GL11.glPixelStorei(3333, 1);
                    GL11.glReadPixels(0, 0, this.mc.displayWidth, this.mc.displayHeight, 6407, 5121, this.entityByteBuffer);
                    graphics.drawImage((Image)screenshotBuffer(this.entityByteBuffer, displayWidth, displayHeight), i, j, (ImageObserver)null);
                }
            }
            graphics.dispose();
//            this.mc.loadingScreen.setText("Saving as " + absoluteFile.toString());
            this.mc.loadingScreen.setProgress(100);
            final FileOutputStream fileOutputStream = new FileOutputStream(absoluteFile);
            ImageIO.write((RenderedImage)bufferedImage, "png", (OutputStream)fileOutputStream);
            fileOutputStream.close();
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
    }
    
    private static BufferedImage screenshotBuffer(final ByteBuffer byteBuffer, final int integer2, final int integer3) {
        byteBuffer.position(0).limit(integer2 * integer3 << 2);
        final BufferedImage bufferedImage;
        final int[] data = ((DataBufferInt)(bufferedImage = new BufferedImage(integer2, integer3, 1)).getRaster().getDataBuffer()).getData();
        for (int i = 0; i < integer2 * integer3; ++i) {
            data[i] = ((byteBuffer.get(i * 3) & 0xFF) << 16 | (byteBuffer.get(i * 3 + 1) & 0xFF) << 8 | (byteBuffer.get(i * 3 + 2) & 0xFF));
        }
        return bufferedImage;
    }
    
    private void getMouseOver(final double double1) {
        double n = double1;
        final EntityPlayerSP thePlayer;
        double distance = (thePlayer = this.mc.thePlayer).prevRotationPitch + (thePlayer.rotationPitch - thePlayer.prevRotationPitch) * n;
        final double n2 = thePlayer.prevRotationYaw + (thePlayer.rotationYaw - thePlayer.prevRotationYaw) * n;
        final Vec3D orientCamera = this.orientCamera(n);
        final double cos = MathHelper.cos(-n2 * 0.017453292f - 3.1415927f);
        double sin = MathHelper.sin(-n2 * 0.017453292f - 3.1415927f);
        double double2 = -MathHelper.cos(-distance * 0.017453292f);
        double sin2 = MathHelper.sin(-distance * 0.017453292f);
        double n3 = sin * double2;
        double n4 = cos * double2;
        double blockReachDistance = this.mc.playerController.getBlockReachDistance();
        this.mc.objectMouseOver = this.mc.theWorld.rayTraceBlocks(orientCamera, orientCamera.addVector(n3 * blockReachDistance, sin2 * blockReachDistance, n4 * blockReachDistance));
        double distance2 = blockReachDistance;
        final Vec3D orientCamera2 = this.orientCamera(n);
        if (this.mc.objectMouseOver != null) {
            distance2 = this.mc.objectMouseOver.hitVec.distance(orientCamera2);
        }
        if (this.mc.playerController instanceof PlayerControllerCreative) {
            blockReachDistance = 32.0f;
        }
        else {
            if (distance2 > 3.0f) {
                distance2 = 3.0f;
            }
            blockReachDistance = distance2;
        }
        final Vec3D addVector = orientCamera2.addVector(n3 * blockReachDistance, sin2 * blockReachDistance, n4 * blockReachDistance);
        this.pointedEntity = null;
        final List entitiesWithinAABBExcludingEntity = this.mc.theWorld.blockMap.getEntitiesWithinAABBExcludingEntity(thePlayer, thePlayer.boundingBox.addCoord(n3 * blockReachDistance, sin2 * blockReachDistance, n4 * blockReachDistance));
        double n5 = 0.0f;
        for (int i = 0; i < entitiesWithinAABBExcludingEntity.size(); ++i) {
            final Entity pointedEntity;
            final MovingObjectPosition calculateIntercept;
            if ((pointedEntity = (Entity)entitiesWithinAABBExcludingEntity.get(i)).canBeCollidedWith() && (calculateIntercept = pointedEntity.boundingBox.expand(0.1f, 0.1f, 0.1f).calculateIntercept(orientCamera2, addVector)) != null && ((distance = orientCamera2.distance(calculateIntercept.hitVec)) < n5 || n5 == 0.0f)) {
                this.pointedEntity = pointedEntity;
                n5 = distance;
            }
        }
        if (this.pointedEntity != null && !(this.mc.playerController instanceof PlayerControllerCreative)) {
            this.mc.objectMouseOver = new MovingObjectPosition(this.pointedEntity);
        }
        for (int j = 0; j < 2; ++j) {
            if (this.mc.gameSettings.anaglyph) {
                if (j == 0) {
                    GL11.glColorMask(false, true, true, false);
                }
                else {
                    GL11.glColorMask(true, false, false, false);
                }
            }
            final EntityPlayerSP thePlayer2 = this.mc.thePlayer;
            final World theWorld = this.mc.theWorld;
            final RenderGlobal renderGlobal = this.mc.renderGlobal;
            final EffectRenderer effectRenderer = this.mc.effectRenderer;
            GL11.glViewport(0, 0, this.mc.displayWidth, this.mc.displayHeight);
            this.updateFogColor(double1);
            GL11.glClear(16640);
            GL11.glEnable(2884);
            int n6 = j;
            n = double1;
            this.farPlaneDistance = (double)(512 >> (this.mc.gameSettings.renderDistance << 1));
            GL11.glMatrixMode(5889);
            GL11.glLoadIdentity();
            if (this.mc.gameSettings.anaglyph) {
                GL11.glTranslated(-((n6 << 1) - 1) * 0.07f, 0.0f, 0.0f);
            }
            double n7 = n;
            final EntityPlayerSP thePlayer3 = this.mc.thePlayer;
            sin = 70.0f;
            if (thePlayer3.isInsideOfMaterial()) {
                sin = 60.0f;
            }
            if (thePlayer3.health <= 0) {
                double2 = thePlayer3.deathTime + n7;
                sin /= (1.0f - 500.0f / (double2 + 500.0f)) * 2.0f + 1.0f;
            }
            GLU.gluPerspective((float)sin, this.mc.displayWidth / (float)this.mc.displayHeight, 0.05f, (float)this.farPlaneDistance);
            GL11.glMatrixMode(5888);
            GL11.glLoadIdentity();
            if (this.mc.gameSettings.anaglyph) {
                GL11.glTranslated(((n6 << 1) - 1) * 0.1f, 0.0f, 0.0f);
            }
            this.hurtCameraEffect(n);
            if (this.mc.gameSettings.fancyGraphics) {
                this.setupViewBobbing(n);
            }
            final EntityPlayerSP thePlayer4;
            sin = (thePlayer4 = this.mc.thePlayer).prevPosX + (thePlayer4.posX - thePlayer4.prevPosX) * n7;
            double2 = thePlayer4.prevPosY + (thePlayer4.posY - thePlayer4.prevPosY) * n7;
            sin2 = thePlayer4.prevPosZ + (thePlayer4.posZ - thePlayer4.prevPosZ) * n7;
            if (this.mc.gameSettings.thirdPersonView) {
                n3 = 4.0f;
                final double n8 = -MathHelper.sin(thePlayer4.rotationYaw / 180.0f * 3.1415927f) * MathHelper.cos(thePlayer4.rotationPitch / 180.0f * 3.1415927f) * 4.0f;
                n4 = MathHelper.cos(thePlayer4.rotationYaw / 180.0f * 3.1415927f) * MathHelper.cos(thePlayer4.rotationPitch / 180.0f * 3.1415927f) * 4.0f;
                blockReachDistance = -MathHelper.sin(thePlayer4.rotationPitch / 180.0f * 3.1415927f) * 4.0f;
                for (int k = 0; k < 8; ++k) {
                    distance2 = (double)(((k & 0x1) << 1) - 1);
                    n = (double)(((k >> 1 & 0x1) << 1) - 1);
                    n5 = (double)(((k >> 2 & 0x1) << 1) - 1);
                    distance2 *= 0.1f;
                    n *= 0.1f;
                    n5 *= 0.1f;
                    final MovingObjectPosition rayTraceBlocks;
                    final double distance3;
                    if ((rayTraceBlocks = this.mc.theWorld.rayTraceBlocks(new Vec3D(sin + distance2, double2 + n, sin2 + n5), new Vec3D(sin - n8 + distance2 + n5, double2 - blockReachDistance + n, sin2 - n4 + n5))) != null && (distance3 = rayTraceBlocks.hitVec.distance(new Vec3D(sin, double2, sin2))) < n3) {
                        n3 = distance3;
                    }
                }
                GL11.glTranslated(0.0f, 0.0f, -n3);
            }
            else {
                GL11.glTranslated(0.0f, 0.0f, -0.1f);
            }
            GL11.glRotated(thePlayer4.prevRotationPitch + (thePlayer4.rotationPitch - thePlayer4.prevRotationPitch) * n7, 1.0f, 0.0f, 0.0f);
            GL11.glRotated(thePlayer4.prevRotationYaw + (thePlayer4.rotationYaw - thePlayer4.prevRotationYaw) * n7 + 180.0f, 0.0f, 1.0f, 0.0f);
            final Frustrum frustrum = new Frustrum();
            Vec3D camera = this.orientCamera(double1);
            frustrum.setTranslation(-sin, -double2, -sin2);
            this.mc.renderGlobal.clipRenderersByFrustrum(frustrum);
            this.mc.renderGlobal.updateRenderers(thePlayer2);
            GL11.glEnable(2912);
            GL11.glBindTexture(3553, this.mc.renderEngine.getTexture("/terrain.png"));
            RenderHelper.disableStandardItemLighting();
            renderGlobal.sortAndRender(thePlayer2, 0);
//            if (theWorld.isSolid(thePlayer2.posX, thePlayer2.posY, thePlayer2.posZ, 0.1f)) {
//            	Tessellator.instance.startDrawingQuads();
//            	final int sortAndRender = (int)thePlayer2.posX;
//                n6 = (int)thePlayer2.posY;
//                final int n9 = (int)thePlayer2.posZ;
//                final RenderBlocks renderBlocks = new RenderBlocks(theWorld);
//                for (int l = sortAndRender - 1; l <= sortAndRender + 1; ++l) {
//                    for (int n10 = n6 - 1; n10 <= n6 + 1; ++n10) {
//                        for (int n11 = n9 - 1; n11 <= n9 + 1; ++n11) {
//                            final int blockId;
//                            if ((blockId = theWorld.getBlockId(l, n10, n11)) > 0) {
//                                renderBlocks.renderBlockAllFaces(Block.blocksList[blockId], l, n10, n11);
//                            }
//                        }
//                    }
//                }
//                Tessellator.instance.draw();
//            }
            RenderHelper.enableStandardItemLighting();
            renderGlobal.renderEntities(camera, frustrum, double1);
            effectRenderer.renderLitParticles(double1);
            RenderHelper.disableStandardItemLighting();
            this.setupFog();
            Tessellator.instance.setTranslation(-camera.xCoord, -camera.yCoord, -camera.zCoord);
            effectRenderer.renderParticles(thePlayer2, double1);
            Tessellator.instance.setTranslation(0, 0, 0);
            renderGlobal.oobGroundRenderer();
            if (this.mc.objectMouseOver != null && !thePlayer2.isInsideOfMaterial()) {
                GL11.glDisable(3008);
                renderGlobal.drawBlockBreaking(this.mc.objectMouseOver, 0, thePlayer2.inventory.getCurrentItem());
                renderGlobal.drawSelectionBox(this.mc.objectMouseOver, 0);
                GL11.glEnable(3008);
            }
            GL11.glBlendFunc(770, 771);
            this.setupFog();
            renderGlobal.oobWaterRenderer();
            GL11.glEnable(3042);
            GL11.glDisable(2884);
            GL11.glBindTexture(3553, this.mc.renderEngine.getTexture("/terrain.png"));
            renderGlobal.sortAndRender(thePlayer2, 1);
            GL11.glColorMask(false, false, false, false);
            GL11.glColorMask(true, true, true, true);
            if (this.mc.gameSettings.anaglyph) {
                if (j == 0) {
                    GL11.glColorMask(false, true, true, false);
                }
                else {
                    GL11.glColorMask(true, false, false, false);
                }
            }
            GL11.glDepthMask(true);
            GL11.glEnable(2884);
            GL11.glDisable(3042);
            n7 = n;
//            GL11.glCallList(this.mc.renderGlobal.chunkBorderlist);
            GL11.glTranslated(-sin, -double2, -sin2);
//            frustrum.setTranslation(0, 0, 0);
            this.setupFog();
            GL11.glEnable(2912);
//            GL11.glDisable(GL11.GL_DEPTH_TEST);
            renderGlobal.renderSky(double1);
//            if (this.mc.objectMouseOver != null && !thePlayer2.isInsideOfMaterial()) {
//                GL11.glDisable(3008);
//                renderGlobal.drawBlockBreaking(this.mc.objectMouseOver, 0, thePlayer2.inventory.getCurrentItem());
//                renderGlobal.drawSelectionBox(this.mc.objectMouseOver, 0);
//                GL11.glEnable(3008);
//            }
            GL11.glBlendFunc(770, 771);
            GL11.glDisable(2912);
            GL11.glClear(256);
            n6 = j;
            n = double1;
            GL11.glLoadIdentity();
            if (this.mc.gameSettings.anaglyph) {
                GL11.glTranslated(((n6 << 1) - 1) * 0.1f, 0.0f, 0.0f);
            }
            GL11.glPushMatrix();
            this.hurtCameraEffect(n);
            if (this.mc.gameSettings.fancyGraphics) {
                this.setupViewBobbing(n);
            }
            if (!this.mc.gameSettings.thirdPersonView) {
                this.itemRenderer.renderItemInFirstPerson(n);
            }
            GL11.glPopMatrix();
            if (!this.mc.gameSettings.thirdPersonView) {
                this.itemRenderer.renderOverlays(n);
                this.hurtCameraEffect(n);
            }
            if (this.mc.gameSettings.fancyGraphics) {
                this.setupViewBobbing(n);
            }
            if (!this.mc.gameSettings.anaglyph) {
                return;
            }
        }
        GL11.glColorMask(true, true, true, false);
    }
    
    public final void setupOverlayRendering() {
        final ScaledResolution scaledResolution;
        final int scaledWidth = (scaledResolution = new ScaledResolution(this.mc.displayWidth, this.mc.displayHeight)).getScaledWidth();
        final int scaledHeight = scaledResolution.getScaledHeight();
        GL11.glClear(256);
        GL11.glMatrixMode(5889);
        GL11.glLoadIdentity();
        GL11.glOrtho(0.0, (double)scaledWidth, (double)scaledHeight, 0.0, 1000.0, 3000.0);
        GL11.glMatrixMode(5888);
        GL11.glLoadIdentity();
        GL11.glTranslated(0.0f, 0.0f, -2000.0f);
    }
    
    private void updateFogColor(double double1) {
        final World theWorld = this.mc.theWorld;
        final EntityPlayerSP thePlayer = this.mc.thePlayer;
        double n = 1.0f / (4 - this.mc.gameSettings.renderDistance);
        n = 1.0f - (double)Math.pow((double)n, 0.25);
        final Vec3D skyColor;
        final double xCoord = (skyColor = theWorld.getSkyColor(double1)).xCoord;
        final double yCoord = skyColor.yCoord;
        final double zCoord = skyColor.zCoord;
        final Vec3D fogColor = theWorld.getFogColor(double1);
        this.fogColorRed = fogColor.xCoord;
        this.fogColorGreen = fogColor.yCoord;
        this.fogColorBlue = fogColor.zCoord;
        this.fogColorRed += (xCoord - this.fogColorRed) * n;
        this.fogColorGreen += (yCoord - this.fogColorGreen) * n;
        this.fogColorBlue += (zCoord - this.fogColorBlue) * n;
        final Block block;
        if ((block = Block.blocksList[theWorld.getBlockId((int)thePlayer.posX, (int)(thePlayer.posY + 0.12f), (int)thePlayer.posZ)]) != null && block.blockMaterial != Material.air) {
            final Material blockMaterial;
            if ((blockMaterial = block.blockMaterial) == Material.water) {
                this.fogColorRed = 0.02f;
                this.fogColorGreen = 0.02f;
                this.fogColorBlue = 0.2f;
            }
            else if (blockMaterial == Material.lava) {
                this.fogColorRed = 0.6f;
                this.fogColorGreen = 0.1f;
                this.fogColorBlue = 0.0f;
            }
        }
        double fogColorGreen = this.fogColor2 + (this.fogColor1 - this.fogColor2) * double1;
        this.fogColorRed *= fogColorGreen;
        this.fogColorGreen *= fogColorGreen;
        this.fogColorBlue *= fogColorGreen;
        if (this.mc.gameSettings.anaglyph) {
            double1 = (this.fogColorRed * 30.0f + this.fogColorGreen * 59.0f + this.fogColorBlue * 11.0f) / 100.0f;
            fogColorGreen = (this.fogColorRed * 30.0f + this.fogColorGreen * 70.0f) / 100.0f;
            final double fogColorBlue = (this.fogColorRed * 30.0f + this.fogColorBlue * 70.0f) / 100.0f;
            this.fogColorRed = double1;
            this.fogColorGreen = fogColorGreen;
            this.fogColorBlue = fogColorBlue;
        }
        GL11.glClearColor((float)this.fogColorRed, (float)this.fogColorGreen, (float)this.fogColorBlue, 0.0f);
    }
    
    private void setupFog() {
        final World theWorld = this.mc.theWorld;
        final EntityPlayerSP thePlayer = this.mc.thePlayer;
        final int n = 2918;
        final double fogColorRed = this.fogColorRed;
        final double fogColorGreen = this.fogColorGreen;
        final double fogColorBlue = this.fogColorBlue;
        final double n2 = fogColorGreen;
        final double n3 = fogColorRed;
        this.fogColorBuffer.clear();
        this.fogColorBuffer.put((float)n3).put((float)n2).put((float)fogColorBlue).put(1.0f);
        this.fogColorBuffer.flip();
        GL11.glFog(n, this.fogColorBuffer);
        GL11.glNormal3f(0.0f, -1.0f, 0.0f);
        GL11.glColor4d(1.0f, 1.0f, 1.0f, 1.0f);
        final Block block;
        if ((block = Block.blocksList[theWorld.getBlockId((int)thePlayer.posX, (int)(thePlayer.posY + 0.12f), (int)thePlayer.posZ)]) != null && block.blockMaterial.getIsLiquid()) {
            final Material blockMaterial = block.blockMaterial;
            GL11.glFogi(2917, 2048);
            if (blockMaterial == Material.water) {
                GL11.glFogf(2914, 0.1f);
            }
            else if (blockMaterial == Material.lava) {
                GL11.glFogf(2914, 2.0f);
            }
        }
        else {
            GL11.glFogi(2917, 9729);
            GL11.glFogf(2915, (float)this.farPlaneDistance / 4.0f);
            GL11.glFogf(2916, (float)this.farPlaneDistance);
        }
        GL11.glEnable(2903);
        GL11.glColorMaterial(1028, 4608);
    }
}
