package net.minecraft.client.renderer;

import net.minecraft.client.particle.EntitySplashFX;
import net.minecraft.client.particle.EntityLavaFX;
import net.minecraft.client.particle.EntityFlameFX;
import net.minecraft.client.particle.EntityExplodeFX;
import net.minecraft.client.particle.EntitySmokeFX;
import net.minecraft.client.particle.EntityFX;
import net.minecraft.client.particle.EntityBubbleFX;
import net.minecraft.game.item.ItemStack;
import net.minecraft.client.physics.MovingObjectPosition;
import java.util.Collections;
import net.minecraft.game.block.Block;
import net.minecraft.game.MathHelper;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Arrays;
import net.minecraft.client.physics.AxisAlignedBB;
import net.minecraft.game.entity.EntityLiving;
import net.minecraft.game.level.BlockMap;
import net.minecraft.game.level.Chunk;
import net.minecraft.game.level.ChunkCoord;
import net.minecraft.game.entity.Entity;
import net.minecraft.game.entity.player.EntityPlayer;
import net.minecraft.client.renderer.entity.RenderManager;
import java.util.Random;
import org.lwjgl.opengl.ARBOcclusionQuery;
import org.lwjgl.opengl.GLContext;
import org.lwjgl.opengl.GL11;
import java.util.ArrayList;
import org.lwjgl.BufferUtils;
import org.lwjgl.input.Keyboard;

import net.minecraft.client.Minecraft;
import java.util.List;
import java.nio.IntBuffer;
import net.minecraft.game.level.World;
import net.minecraft.game.level.IWorldAccess;

public final class RenderGlobal implements IWorldAccess {
    private World worldObj;
    private RenderEngine renderEngine;
    private int glGenList;
    private IntBuffer renderIntBuffer;
    private HashMap<ChunkCoord,WorldRenderer> chunks;
    private int renderChunksWide;
    private int renderChunksTall;
    private int renderChunksDeep;
    private int glRenderListBase;
    private Minecraft mc;
    private RenderBlocks globalRenderBlocks;
    private IntBuffer glOcclusionQueryBase;
    private boolean occlusionEnabled;
    private int cloudOffsetX;
    private int glSkyList;
    private int countEntitiesTotal;
    private int countEntitiesRendered;
    private int countEntitiesHidden;
    private IntBuffer occlusionResult;
    private int renderersLoaded;
    private int renderersBeingClipped;
    private int renderersBeingOccluded;
    private int renderersBeingRendered;
    private double prevSortX;
    private double prevSortY;
    private double prevSortZ;
    public double damagePartialTime;
	private WorldRenderer[] sortedWorldRenderers;
	protected int chunkBorderlist;
    
    public RenderGlobal(final Minecraft minecraft, final RenderEngine renderEngine) {
        this.renderIntBuffer = BufferUtils.createIntBuffer(65536);
        this.occlusionEnabled = false;
        this.cloudOffsetX = 0;
        this.occlusionResult = BufferUtils.createIntBuffer(64);
        this.prevSortX = -9999.0f;
        this.prevSortY = -9999.0f;
        this.prevSortZ = -9999.0f;
        this.mc = minecraft;
        this.renderEngine = renderEngine;
        this.glGenList = GL11.glGenLists(2);
        //this.glRenderListBase = GL11.glGenLists(786432);
        this.occlusionEnabled = GLContext.getCapabilities().GL_ARB_occlusion_query;
        if (this.occlusionEnabled) {
            this.occlusionResult.clear();
            GL11.glGetInteger(34916, this.occlusionResult);
            if (this.occlusionResult.get(0) == 0) {
                this.occlusionEnabled = false;
            }
        }
        GL11.glNewList(this.glSkyList = GL11.glGenLists(1), 4864);
        final Random random = new Random(10842L);
        for (int i = 0; i < 500; ++i) {
            GL11.glRotated(random.nextDouble() * 360.0f, 1.0f, 0.0f, 0.0f);
            GL11.glRotated(random.nextDouble() * 360.0f, 0.0f, 1.0f, 0.0f);
            GL11.glRotated(random.nextDouble() * 360.0f, 0.0f, 0.0f, 1.0f);
            final Tessellator instance = Tessellator.instance;
            final double double1 = 0.25f + random.nextDouble() * 0.25f;
            instance.startDrawingQuads();
            instance.addVertexWithUV(-double1, -100.0f, double1, 1.0f, 1.0f);
            instance.addVertexWithUV(double1, -100.0f, double1, 0.0f, 1.0f);
            instance.addVertexWithUV(double1, -100.0f, -double1, 0.0f, 0.0f);
            instance.addVertexWithUV(-double1, -100.0f, -double1, 1.0f, 0.0f);
            instance.draw();
        }
        GL11.glEndList();
        chunkBorderlist = GL11.glGenLists(1);
        GL11.glNewList(chunkBorderlist, GL11.GL_COMPILE);
        GL11.glBegin(GL11.GL_LINES);
        GL11.glColor4d(0, 0.4, 1.0, 0.5);
        for(int x=0;x<16;x+=4) {
        	GL11.glVertex3i(x, 0, 0);
        	GL11.glVertex3i(x, 16, 0);
        	GL11.glVertex3i(x, 0, 16);
        	GL11.glVertex3i(x, 16, 16);
        	GL11.glVertex3i(0, 0, x);
        	GL11.glVertex3i(0, 16, x);
        	GL11.glVertex3i(16, 0, x);
        	GL11.glVertex3i(16, 16, x);
        }
        GL11.glColor4d(1, 1, 1, 0.5);
        GL11.glVertex3i(0, 0, 0);
        GL11.glVertex3i(0, 0, 16);
        GL11.glVertex3i(16, 0, 0);
        GL11.glVertex3i(16, 0, 16);
        GL11.glVertex3i(0, 0, 16);
        GL11.glVertex3i(16, 0, 16);
        GL11.glVertex3i(0, 0, 0);
        GL11.glVertex3i(16, 0, 0);
        GL11.glEnd();
        GL11.glEndList();
    }
    
    public final void changeWorld(final World world) {
        if (this.worldObj != null) {
            this.worldObj.removeWorldAccess(this);
        }
        this.prevSortX = -9999.0f;
        this.prevSortY = -9999.0f;
        this.prevSortZ = -9999.0f;
        RenderManager.instance.set(world);
        this.worldObj = world;
        this.globalRenderBlocks = new RenderBlocks(world);
        if (world != null) {
            world.addWorldAccess(this);
            this.loadRenderers();
        }
    }
    
    public final void loadRenderers() {
    	chunks = new HashMap<>();
    	this.reload();
    }
    
    public final void renderEntities(final Vec3D vec3D, final ICamera iCamera, final double double3) {
        final BlockMap blockMap = this.worldObj.blockMap;
        RenderManager.instance.cacheActiveRenderInfo(this.worldObj, this.renderEngine, this.mc.thePlayer, double3);
        this.countEntitiesTotal = 0;
        this.countEntitiesRendered = 0;
        this.countEntitiesHidden = 0;
        /*for (int i = 0; i < blockMap.width; ++i) {
            for (int j = 0; j < blockMap.depth; ++j) {
                for (int k = 0; k < blockMap.height; ++k) {
                    final List list;
                    if ((list = blockMap.entityGrid[(k * blockMap.depth + j) * blockMap.width + i]).size() != 0) {
                        int l = (i << 3) + 4;
                        final int n = (j << 3) + 4;
                        final int n2 = (k << 3) + 4;
                        this.countEntitiesTotal += list.size();
                        final double n3 = (double)l;
                        final double n4 = (double)n;
                        double n5 = (double)n2;
                        double n6 = n4;
                        double n7 = n3;
                        boolean b;
                        if (n7 < 0.0f || n6 < 0.0f || n5 < 0.0f || n7 >= this.worldObj.width || n6 >= this.worldObj.height || n5 >= this.worldObj.length) {
                            b = true;
                        }
                        else {
                            final int n8 = (int)(n7 / 16.0f);
                            final int n9 = (int)(n6 / 16.0f);
                            final int n10 = (int)(n5 / 16.0f);
                            b = (this.worldRenderers[(n10 * this.renderChunksTall + n9) * this.renderChunksWide + n8].isInFrustrum && this.worldRenderers[(n10 * this.renderChunksTall + n9) * this.renderChunksWide + n8].isVisible);
                        }
                        if (b) {
                            for (l = 0; l < list.size(); ++l) {
                                final Entity entity2 = (Entity) list.get(l);;
                                final Entity entity = entity2;
                                n6 = entity.posX - vec3D.xCoord;
                                n5 = entity.posY - vec3D.yCoord;
                                n7 = entity.posZ - vec3D.zCoord;
                                n6 = n6 * n6 + n5 * n5 + n7 * n7;
                                final Entity entity3 = entity;
                                final double n11 = n6;
                                final AxisAlignedBB boundingBox;
                                n6 = (boundingBox = entity3.boundingBox).maxX - boundingBox.minX;
                                n5 = boundingBox.maxY - boundingBox.minY;
                                n7 = boundingBox.maxZ - boundingBox.minZ;
                                n7 = (n7 = (n6 + n5 + n7) / 3.0f) * 64.0f;
                                if (n11 < n7 * n7 && iCamera.isBoundingBoxInFrustrum(entity2.boundingBox) && (entity2 != this.worldObj.player || this.mc.gameSettings.thirdPersonView)) {
                                    ++this.countEntitiesRendered;
                                    RenderManager.instance.renderEntity(entity2, double3);
                                }
                            }
                        }
                        else {
                            this.countEntitiesHidden += list.size();
                        }
                    }
                }
            }
        }*/
        blockMap.render(vec3D, iCamera, renderEngine, double3);
    }
    
    public final String getDebugInfoRenders() {
        return new StringBuilder().append("C: ").append(this.renderersBeingRendered).append("/").append(this.renderersLoaded).append(". F: ").append(this.renderersBeingClipped).append(", O: ").append(this.renderersBeingOccluded).toString();
    }
    
    public final String getDebugInfoEntities() {
        return new StringBuilder().append("E: ").append(this.countEntitiesRendered).append("/").append(this.countEntitiesTotal).append(". B: ").append(this.countEntitiesHidden).append(", I: ").append(this.countEntitiesTotal - this.countEntitiesHidden - this.countEntitiesRendered).toString();
    }
    
    public final int sortAndRender(final EntityPlayer entityPlayer, final int renderPass) {
    	if (renderPass == 0) {
            this.renderersLoaded = 0;
            this.renderersBeingClipped = 0;
            this.renderersBeingOccluded = 0;
            this.renderersBeingRendered = 0;
        }
    	
    	final double n = entityPlayer.posX - this.prevSortX;
        final double n2 = entityPlayer.posY - this.prevSortY;
        final double n3 = entityPlayer.posZ - this.prevSortZ;
        if(sortedWorldRenderers == null || sortedWorldRenderers.length == 0 || n * n + n2 * n2 + n3 * n3 > 16.0f) {
        	sortedWorldRenderers = this.chunks.values().toArray(new WorldRenderer[0]);
        	Arrays.sort(sortedWorldRenderers, new RenderSorter(entityPlayer));
        	this.prevSortX = entityPlayer.posX;
            this.prevSortY = entityPlayer.posY;
            this.prevSortZ = entityPlayer.posZ;
        }
        
//        GL11.glDisable(3553);
//        GL11.glDisable(2896);
//        GL11.glDisable(3008);
//        GL11.glColorMask(false, false, false, false);
//        GL11.glDepthMask(false);
//        for (int i = 0; i < sortedWorldRenderers.length; i++) {
//			WorldRenderer worldRenderer = sortedWorldRenderers[i];
//			if(!worldRenderer.isInFrustrum) worldRenderer.isVisible = true;
//            if (worldRenderer.isInFrustrum && !worldRenderer.isWaitingOnOcclusionQuery) {
//                final int n6 = (int)(1.0f + MathHelper.sqrt_double(worldRenderer.distanceToEntitySquared((EntityLiving)entityPlayer)) / 64.0f);
//                if (this.cloudOffsetX % n6 == i % n6) {
//                    ARBOcclusionQuery.glBeginQueryARB(35092, worldRenderer.glOcclusionQuery);
//                    worldRenderer.callOcclusionQueryList();
//                    ARBOcclusionQuery.glEndQueryARB(35092);
//                    worldRenderer.isWaitingOnOcclusionQuery = true;
//                }
//            }
//		}
//        checkOcclusionQueryResult(0, sortedWorldRenderers.length);
//        GL11.glColorMask(true, true, true, true);
//        GL11.glDepthMask(true);
//        GL11.glEnable(3553);
//        GL11.glEnable(3008);
        
        this.renderSortedRenderers(0, sortedWorldRenderers.length, renderPass, entityPlayer);
        
//        System.out.println(renderersBeingRendered);
        
        return renderersBeingRendered;
    }
    
    private void renderSortedRenderers(int start, final int end, final int renderPass, EntityPlayer player) {
        for (; start < end; ++start) {
        	WorldRenderer worldRenderer = this.sortedWorldRenderers[start];
            if (renderPass == 0) {
                ++this.renderersLoaded;
                if (!worldRenderer.isInFrustrum) {
                    ++this.renderersBeingClipped;
                }
                if (worldRenderer.isInFrustrum && !worldRenderer.isVisible) {
                    ++this.renderersBeingOccluded;
                }
                if (worldRenderer.isInFrustrum && worldRenderer.isVisible) {
                    ++this.renderersBeingRendered;
                }
            }
            if (worldRenderer.isInFrustrum && worldRenderer.isVisible) {
            	
                double offX = worldRenderer.posX - (long) player.posX,
                	   offY = worldRenderer.posY - player.posY,
                	   offZ = worldRenderer.posZ - (long) player.posZ;
                GL11.glPushMatrix();
                GL11.glTranslated(offX, offY, offZ);
                GL11.glTranslated((long)player.posX - player.posX, 0,(long)player.posZ - player.posZ );
//                GL11.glTranslated(worldRenderer.posX / 128D, worldRenderer.posY / 128D, worldRenderer.posZ / 128D);
                worldRenderer.render(renderPass);
//                offX += 16;
//                offZ += 32;
                if(renderPass == 0 && Keyboard.isKeyDown(Keyboard.KEY_F3) && Math.abs(offX) < 48 && Math.abs(offZ) < 48) {
                	GL11.glColor4f(1, 1, 1, 1);
                	GL11.glCallList(chunkBorderlist);
                }
                GL11.glPopMatrix();
            }
        }
    }

	private void checkOcclusionQueryResult(int integer1, int integer2) {
    	for (integer1 = integer1; integer1 < integer2; ++integer1) {
            if (this.sortedWorldRenderers[integer1].isWaitingOnOcclusionQuery) {
                this.occlusionResult.clear();
                ARBOcclusionQuery.glGetQueryObjectuARB(this.sortedWorldRenderers[integer1].glOcclusionQuery, 34919, this.occlusionResult);
                if (this.occlusionResult.get(0) != 0) {
                    this.sortedWorldRenderers[integer1].isWaitingOnOcclusionQuery = false;
                    this.occlusionResult.clear();
                    ARBOcclusionQuery.glGetQueryObjectuARB(this.sortedWorldRenderers[integer1].glOcclusionQuery, 34918, this.occlusionResult);
                    System.out.println(this.occlusionResult.get(0));
                    this.sortedWorldRenderers[integer1].isVisible = (this.occlusionResult.get(0) != 0);
                }
            }
        }
	}

	public final void renderAllRenderLists() {
        GL11.glBindTexture(3553, this.renderEngine.getTexture("/terrain.png"));
        GL11.glCallLists(this.renderIntBuffer);
    }
    
    public final void updateClouds() {
        ++this.cloudOffsetX;
    }
    
    public final void renderSky(final double double1) {
        GL11.glDisable(3553);
        final Vec3D skyColor;
        double n = (skyColor = this.worldObj.getSkyColor(double1)).xCoord;
        double n2 = skyColor.yCoord;
        double double2 = skyColor.zCoord;
        if (this.mc.gameSettings.anaglyph) {
            final double n3 = (n * 30.0f + n2 * 59.0f + double2 * 11.0f) / 100.0f;
            n2 = (n * 30.0f + n2 * 70.0f) / 100.0f;
            double2 = (n * 30.0f + double2 * 70.0f) / 100.0f;
            n = n3;
            n2 = n2;
            double2 = double2;
        }
        GL11.glDepthMask(false);
        final Tessellator instance;
        (instance = Tessellator.instance).startDrawingQuads();
        instance.setColorOpaque_F(n, n2, double2);
        double2 = (double)(this.worldObj.height + 10);
        for (int i = -2048; i < this.worldObj.width + 2048; i += 512) {
            for (int j = -2048; j < this.worldObj.length + 2048; j += 512) {
                instance.addVertex((double)i, double2, (double)j);
                instance.addVertex((double)(i + 512), double2, (double)j);
                instance.addVertex((double)(i + 512), double2, (double)(j + 512));
                instance.addVertex((double)i, double2, (double)(j + 512));
            }
        }
        instance.draw();
        GL11.glEnable(3553);
        GL11.glDisable(2912);
        GL11.glDisable(3008);
        GL11.glEnable(3042);
        GL11.glBlendFunc(1, 1);
        GL11.glPushMatrix();
        n = this.worldObj.player.lastTickPosX + (this.worldObj.player.posX - this.worldObj.player.lastTickPosX) * double1;
        n2 = this.worldObj.player.lastTickPosY + (this.worldObj.player.posY - this.worldObj.player.lastTickPosY) * double1;
        double yCoord = this.worldObj.player.lastTickPosZ + (this.worldObj.player.posZ - this.worldObj.player.lastTickPosZ) * double1;
        GL11.glColor4d(1.0f, 1.0f, 1.0f, 1.0f);
        GL11.glTranslated(n, n2, yCoord);
        GL11.glRotated(0.0f, 0.0f, 0.0f, 1.0f);
        GL11.glRotated(this.worldObj.getCelestialAngle(double1) * 360.0d, 1.0f, 0.0f, 0.0f);
        GL11.glBindTexture(3553, this.renderEngine.getTexture("/terrain/sun.png"));
        instance.startDrawingQuads();
        instance.addVertexWithUV(-30.0f, 100.0f, -30.0f, 0.0f, 0.0f);
        instance.addVertexWithUV(30.0f, 100.0f, -30.0f, 1.0f, 0.0f);
        instance.addVertexWithUV(30.0f, 100.0f, 30.0f, 1.0f, 1.0f);
        instance.addVertexWithUV(-30.0f, 100.0f, 30.0f, 0.0f, 1.0f);
        instance.draw();
        GL11.glBindTexture(3553, this.renderEngine.getTexture("/terrain/moon.png"));
        instance.startDrawingQuads();
        instance.addVertexWithUV(-20.0f, -100.0f, 20.0f, 1.0f, 1.0f);
        instance.addVertexWithUV(20.0f, -100.0f, 20.0f, 0.0f, 1.0f);
        instance.addVertexWithUV(20.0f, -100.0f, -20.0f, 0.0f, 0.0f);
        instance.addVertexWithUV(-20.0f, -100.0f, -20.0f, 1.0f, 0.0f);
        instance.draw();
        GL11.glDisable(3553);
        GL11.glColor4d(double2 = this.worldObj.getStarBrightness(double1), double2, double2, double2);
        GL11.glCallList(this.glSkyList);
        GL11.glColor4d(1.0f, 1.0f, 1.0f, 1.0f);
        GL11.glEnable(3553);
        GL11.glDisable(3042);
        GL11.glEnable(3008);
        GL11.glEnable(2912);
        GL11.glPopMatrix();
        GL11.glDepthMask(true);
        GL11.glBindTexture(3553, this.renderEngine.getTexture("/clouds.png"));
        GL11.glColor4d(1.0f, 1.0f, 1.0f, 1.0f);
        final Vec3D cloudColor;
        n2 = (cloudColor = this.worldObj.getCloudColor(double1)).xCoord;
        yCoord = cloudColor.yCoord;
        n = cloudColor.zCoord;
        if (this.mc.gameSettings.anaglyph) {
            double2 = (n2 * 30.0f + yCoord * 59.0f + n * 11.0f) / 100.0f;
            final double n4 = (n2 * 30.0f + yCoord * 70.0f) / 100.0f;
            n = (n2 * 30.0f + n * 70.0f) / 100.0f;
            n2 = double2;
            yCoord = n4;
            n = n;
        }
        double2 = (double)this.worldObj.cloudHeight;
        final double n4 = (this.cloudOffsetX + double1) * 4.8828125E-4f * 0.03f;
        instance.startDrawingQuads();
        instance.setColorOpaque_F(n2, yCoord, n);
        for (int k = -2048; k < this.worldObj.width + 2048; k += 512) {
            for (int i = -2048; i < this.worldObj.length + 2048; i += 512) {
                instance.addVertexWithUV((double)k, double2, (double)(i + 512), k * 4.8828125E-4f + n4, (i + 512) * 4.8828125E-4f);
                instance.addVertexWithUV((double)(k + 512), double2, (double)(i + 512), (k + 512) * 4.8828125E-4f + n4, (i + 512) * 4.8828125E-4f);
                instance.addVertexWithUV((double)(k + 512), double2, (double)i, (k + 512) * 4.8828125E-4f + n4, i * 4.8828125E-4f);
                instance.addVertexWithUV((double)k, double2, (double)i, k * 4.8828125E-4f + n4, i * 4.8828125E-4f);
                instance.addVertexWithUV((double)k, double2, (double)i, k * 4.8828125E-4f + n4, i * 4.8828125E-4f);
                instance.addVertexWithUV((double)(k + 512), double2, (double)i, (k + 512) * 4.8828125E-4f + n4, i * 4.8828125E-4f);
                instance.addVertexWithUV((double)(k + 512), double2, (double)(i + 512), (k + 512) * 4.8828125E-4f + n4, (i + 512) * 4.8828125E-4f);
                instance.addVertexWithUV((double)k, double2, (double)(i + 512), k * 4.8828125E-4f + n4, (i + 512) * 4.8828125E-4f);
            }
        }
        instance.draw();
    }
    
    public final void oobGroundRenderer() {
        final double lightBrightness = this.worldObj.getLightBrightness(0, this.worldObj.getGroundLevel(), 0);
        GL11.glBindTexture(3553, this.renderEngine.getTexture("/dirt.png"));
        if (this.worldObj.getGroundLevel() > this.worldObj.getWaterLevel() && this.worldObj.defaultFluid == Block.waterMoving.blockID) {
            GL11.glBindTexture(3553, this.renderEngine.getTexture("/grass.png"));
        }
        GL11.glColor4d(lightBrightness, lightBrightness, lightBrightness, 1.0f);
        GL11.glEnable(3553);
        GL11.glCallList(this.glGenList);
    }
    
    private void oobGroundRenderHeight() {
        final Tessellator instance = Tessellator.instance;
        final double n = (double)this.worldObj.getGroundLevel();
        int n2 = 128;
        if (128 > this.worldObj.width) {
            n2 = this.worldObj.width;
        }
        if (n2 > this.worldObj.length) {
            n2 = this.worldObj.length;
        }
        final int n3 = 2048 / n2;
        instance.startDrawingQuads();
        for (int i = -n2 * n3; i < this.worldObj.width + n2 * n3; i += n2) {
            for (int j = -n2 * n3; j < this.worldObj.length + n2 * n3; j += n2) {
                if (n < 0.0f || i < 0 || j < 0 || i >= this.worldObj.width || j >= this.worldObj.length) {
                    instance.addVertexWithUV((double)i, n, (double)(j + n2), 0.0f, (double)n2);
                    instance.addVertexWithUV((double)(i + n2), n, (double)(j + n2), (double)n2, (double)n2);
                    instance.addVertexWithUV((double)(i + n2), n, (double)j, (double)n2, 0.0f);
                    instance.addVertexWithUV((double)i, n, (double)j, 0.0f, 0.0f);
                }
            }
        }
        instance.draw();
    }
    
    public final void oobWaterRenderer() {
        GL11.glEnable(3553);
        GL11.glEnable(3042);
        GL11.glBindTexture(3553, this.renderEngine.getTexture("/water.png"));
        final double lightBrightness;
        GL11.glColor4d(lightBrightness = this.worldObj.getLightBrightness(0, this.worldObj.getWaterLevel(), 0), lightBrightness, lightBrightness, 1.0f);
        GL11.glCallList(this.glGenList + 1);
        GL11.glColor4d(1.0f, 1.0f, 1.0f, 1.0f);
        GL11.glDisable(3042);
    }
    
    private void oobWaterRenderHeight() {
        final double n = (double)this.worldObj.getWaterLevel();
        GL11.glBlendFunc(770, 771);
        final Tessellator instance = Tessellator.instance;
        int n2 = 128;
        if (128 > this.worldObj.width) {
            n2 = this.worldObj.width;
        }
        if (n2 > this.worldObj.length) {
            n2 = this.worldObj.length;
        }
        final int n3 = 2048 / n2;
        instance.startDrawingQuads();
        final double minX = Block.waterMoving.minX;
        final double minZ = Block.waterMoving.minZ;
        for (int i = -n2 * n3; i < this.worldObj.width + n2 * n3; i += n2) {
            for (int j = -n2 * n3; j < this.worldObj.length + n2 * n3; j += n2) {
                final double n4 = n + Block.waterMoving.minY;
                if (n < 0.0f || i < 0 || j < 0 || i >= this.worldObj.width || j >= this.worldObj.length) {
                    instance.addVertexWithUV(i + minX, n4, j + n2 + minZ, 0.0f, (double)n2);
                    instance.addVertexWithUV(i + n2 + minX, n4, j + n2 + minZ, (double)n2, (double)n2);
                    instance.addVertexWithUV(i + n2 + minX, n4, j + minZ, (double)n2, 0.0f);
                    instance.addVertexWithUV(i + minX, n4, j + minZ, 0.0f, 0.0f);
                    instance.addVertexWithUV(i + minX, n4, j + minZ, 0.0f, 0.0f);
                    instance.addVertexWithUV(i + n2 + minX, n4, j + minZ, (double)n2, 0.0f);
                    instance.addVertexWithUV(i + n2 + minX, n4, j + n2 + minZ, (double)n2, (double)n2);
                    instance.addVertexWithUV(i + minX, n4, j + n2 + minZ, 0.0f, (double)n2);
                }
            }
        }
        instance.draw();
        GL11.glDisable(3042);
    }
    
    public final void updateRenderers(final EntityPlayer entityPlayer) {
//        this.chunks.sort(new RenderSorter(entityPlayer));
    	long x = MathHelper.floor_double_long(entityPlayer.posX / 16D);
    	long y = MathHelper.floor_double_long(entityPlayer.posY / 16D);
    	long z = MathHelper.floor_double_long(entityPlayer.posZ / 16D);
    	
    	int renderDistance = (5-mc.gameSettings.renderDistance);
    	
    	for(long X = x - renderDistance;X<x+renderDistance;X++)
    		for(long Y = y - renderDistance;Y<y+renderDistance;Y++)
    			for(long Z = z - renderDistance;Z<z+renderDistance;Z++) {
    				ChunkCoord cc = new ChunkCoord(X,Y,Z);
    				if(!chunks.containsKey(cc)) {
    					WorldRenderer chunk = new WorldRenderer(worldObj, X*16, Y*16, Z*16);
    					chunk.needsUpdate = true;
    					if(occlusionEnabled) {
    						chunk.glOcclusionQuery = ARBOcclusionQuery.glGenQueriesARB();
    					}
    					chunks.put(cc, chunk);
    				}
    			}
    	Iterator<ChunkCoord> iter = chunks.keySet().iterator();
    	while(iter.hasNext()) {
    		ChunkCoord coord = iter.next();
    		if(Math.abs(coord.x - x) > renderDistance || Math.abs(coord.y - y) > renderDistance || Math.abs(coord.z - z) > renderDistance) {
    			iter.remove();
    		}
    	}
    	WorldRenderer[] chunkz = this.chunks.values().toArray(new WorldRenderer[0]);
    	Arrays.sort(chunkz, new RenderSorter(entityPlayer));
    	long time = System.currentTimeMillis();
    	for(WorldRenderer chunk : chunkz) {
    		if(chunk.isInFrustrum) {
    			chunk.updateRenderer();
    			if(System.currentTimeMillis()-time>10)
    				break;
    		}
    	}
    }
    
    public final void drawBlockBreaking(final MovingObjectPosition movingObjectPosition, final int integer, final ItemStack itemStack) {
        final Tessellator instance = Tessellator.instance;
        GL11.glEnable(3042);
        GL11.glEnable(3008);
        GL11.glBlendFunc(770, 1);
        GL11.glColor4d(1.0f, 1.0f, 1.0f, (MathHelper.sin(System.currentTimeMillis() / 100.0f) * 0.2f + 0.4f) * 0.5f);
        if (this.damagePartialTime > 0.0f) {
            GL11.glBlendFunc(774, 768);
            int n = this.renderEngine.getTexture("/terrain.png");
            GL11.glBindTexture(3553, n);
            GL11.glColor4d(1.0f, 1.0f, 1.0f, 0.5f);
            GL11.glPushMatrix();
            Block stone = ((n = this.worldObj.getBlockId(movingObjectPosition.blockX, movingObjectPosition.blockY, movingObjectPosition.blockZ)) > 0) ? Block.blocksList[n] : null;
            GL11.glDisable(3008);
            instance.startDrawingQuads();
            instance.disableColor();
            if (stone == null) {
                stone = Block.stone;
            }
            this.globalRenderBlocks.renderBlockUsingTexture(stone, movingObjectPosition.blockX, movingObjectPosition.blockY, movingObjectPosition.blockZ, 240 + (int)(this.damagePartialTime * 10.0f));
            instance.draw();
            GL11.glEnable(3008);
            GL11.glDepthMask(true);
            GL11.glPopMatrix();
        }
        GL11.glDisable(3042);
        GL11.glDisable(3008);
    }
    
    public final void drawSelectionBox(final MovingObjectPosition movingObjectPosition, int integer) {
        GL11.glEnable(3042);
        GL11.glBlendFunc(770, 771);
        GL11.glColor4d(0.0f, 0.0f, 0.0f, 0.4f);
        GL11.glLineWidth(2.0f);
        GL11.glDisable(3553);
        GL11.glDepthMask(false);
        if ((integer = this.worldObj.getBlockId(movingObjectPosition.blockX, movingObjectPosition.blockY, movingObjectPosition.blockZ)) > 0) {
            final AxisAlignedBB expand = Block.blocksList[integer].getSelectedBoundingBoxFromPool(movingObjectPosition.blockX, movingObjectPosition.blockY, movingObjectPosition.blockZ).expand(0.002f, 0.002f, 0.002f);
            final Tessellator instance;
            expand.offset(-Minecraft.getInstance().thePlayer.posX, -Minecraft.getInstance().thePlayer.posY, -Minecraft.getInstance().thePlayer.posZ);
            (instance = Tessellator.instance).startDrawing(3);
            instance.addVertex(expand.minX, expand.minY, expand.minZ);
            instance.addVertex(expand.maxX, expand.minY, expand.minZ);
            instance.addVertex(expand.maxX, expand.minY, expand.maxZ);
            instance.addVertex(expand.minX, expand.minY, expand.maxZ);
            instance.addVertex(expand.minX, expand.minY, expand.minZ);
            instance.draw();
            instance.startDrawing(3);
            instance.addVertex(expand.minX, expand.maxY, expand.minZ);
            instance.addVertex(expand.maxX, expand.maxY, expand.minZ);
            instance.addVertex(expand.maxX, expand.maxY, expand.maxZ);
            instance.addVertex(expand.minX, expand.maxY, expand.maxZ);
            instance.addVertex(expand.minX, expand.maxY, expand.minZ);
            instance.draw();
            instance.startDrawing(1);
            instance.addVertex(expand.minX, expand.minY, expand.minZ);
            instance.addVertex(expand.minX, expand.maxY, expand.minZ);
            instance.addVertex(expand.maxX, expand.minY, expand.minZ);
            instance.addVertex(expand.maxX, expand.maxY, expand.minZ);
            instance.addVertex(expand.maxX, expand.minY, expand.maxZ);
            instance.addVertex(expand.maxX, expand.maxY, expand.maxZ);
            instance.addVertex(expand.minX, expand.minY, expand.maxZ);
            instance.addVertex(expand.minX, expand.maxY, expand.maxZ);
            instance.draw();
        }
        GL11.glDepthMask(true);
        GL11.glEnable(3553);
        GL11.glDisable(3042);
    }
    
    public void reload() {
    	if(chunks!=null)
    		chunks.clear();
    	this.sortedWorldRenderers = null;
    }
    
    private void markBlocksForUpdate(long integer1, long integer2, long integer3, long integer4, long integer5, long integer6) {
    	integer1 = MathHelper.floor_double_long(integer1/(double)Chunk.WIDTH);
    	integer2 = MathHelper.floor_double_long(integer2/(double)Chunk.HEIGHT);
    	integer3 = MathHelper.floor_double_long(integer3/(double)Chunk.WIDTH);
    	integer4 = MathHelper.floor_double_long(integer4/(double)Chunk.WIDTH);
    	integer5 = MathHelper.floor_double_long(integer5/(double)Chunk.HEIGHT);
    	integer6 = MathHelper.floor_double_long(integer6/(double)Chunk.WIDTH);
    	for(long x=integer1;x<=integer4;x++)
    		for(long y=integer2;y<=integer5;y++)
    			for(long z=integer3;z<=integer6;z++) {
    				WorldRenderer chunk = chunks.get(new ChunkCoord(x,y,z));
    				if(chunk!=null) {
    					chunk.needsUpdate = true;
    				}
    			}
    }
    
    public final void markBlockAndNeighborsNeedsUpdate(long integer1, long integer2, long integer3) {
        this.markBlocksForUpdate(integer1 - 1, integer2 - 1, integer3 - 1, integer1 + 1, integer2 + 1, integer3 + 1);
    }
    
    public final void markBlockRangeNeedsUpdate(long integer1, long integer2, long integer3, long integer4, long integer5, long integer6) {
        this.markBlocksForUpdate(integer1 - 1, integer2 - 1, integer3 - 1, integer4 + 1, integer5 + 1, integer6 + 1);
    }
    
    public final void clipRenderersByFrustrum(final ICamera iCamera) {
    	if(sortedWorldRenderers == null) return;
        for (int i = 0; i < sortedWorldRenderers.length; i++) {
			WorldRenderer worldRenderer = sortedWorldRenderers[i];
			worldRenderer.updateInFrustrum(iCamera);
		}
    }
    
    public final void playSound(final String string, final double double2, final double double3, final double double4, final double double5, final double double6) {
        this.mc.sndManager.playSound(string, double2, double3, double4, double5, double6);
    }
    
    public final void spawnParticle(final String string, final double double2, final double double3, final double double4, final double double5, final double double6, final double double7) {
        final double n = this.worldObj.player.posX - double2;
        final double n2 = this.worldObj.player.posY - double3;
        final double n3 = this.worldObj.player.posZ - double4;
        if (n * n + n2 * n2 + n3 * n3 > 256.0f) {
            return;
        }
        if (string == "bubble") {
            this.mc.effectRenderer.addEffect(new EntityBubbleFX(this.worldObj, double2, double3, double4, double5, double6, double7));
            return;
        }
        if (string == "smoke") {
            this.mc.effectRenderer.addEffect(new EntitySmokeFX(this.worldObj, double2, double3, double4));
            return;
        }
        if (string == "explode") {
            this.mc.effectRenderer.addEffect(new EntityExplodeFX(this.worldObj, double2, double3, double4, double5, double6, double7));
            return;
        }
        if (string == "flame") {
            this.mc.effectRenderer.addEffect(new EntityFlameFX(this.worldObj, double2, double3, double4));
            return;
        }
        if (string == "lava") {
            this.mc.effectRenderer.addEffect(new EntityLavaFX(this.worldObj, double2, double3, double4));
            return;
        }
        if (string == "splash") {
            this.mc.effectRenderer.addEffect(new EntitySplashFX(this.worldObj, double2, double3, double4));
            return;
        }
        if (string == "largesmoke") {
            this.mc.effectRenderer.addEffect(new EntitySmokeFX(this.worldObj, double2, double3, double4, 2.5f));
        }
    }
    
    public final void playMusic(final String string, final double double2, final double double3, final double double4, final double double5) {
        this.mc.sndManager.playRandomMusicIfReady(double2, double3, double4);
    }
    
    public final void obtainEntitySkin(final Entity player) {
        if (player.skinUrl != null) {
            this.renderEngine.obtainImageData(player.skinUrl, new ImageBufferDownload());
        }
    }
    
    public final void releaseEntitySkin(final Entity player) {
        if (player.skinUrl != null) {
            this.renderEngine.releaseImageData(player.skinUrl);
        }
    }
    
    public final void updateAllRenderers() {
        GL11.glNewList(this.glGenList, 4864);
        this.oobGroundRenderHeight();
        GL11.glEndList();
        GL11.glNewList(this.glGenList + 1, 4864);
        this.oobWaterRenderHeight();
        GL11.glEndList();
    }
}
