package net.minecraft.client.renderer;

import net.minecraft.game.entity.EntityLiving;
import net.minecraft.game.block.Block;
import net.minecraft.client.renderer.entity.Render;
import org.lwjgl.opengl.GL11;
import net.minecraft.game.MathHelper;
import net.minecraft.client.physics.AxisAlignedBB;
import net.minecraft.game.level.Chunk;
import net.minecraft.game.level.World;

public final class WorldRenderer {
    private World worldObj;
    private int glRenderList;
    private static Tessellator tessellator;
    public static int chunksUpdated;
    long posX, posY, posZ;
    private int sizeWidth;
    private int sizeHeight;
    private int sizeDepth;
    public boolean isInFrustrum;
    private boolean[] skipRenderPass;
    private long posXPlus;
    private long posYPlus;
    private long posZPlus;
    public boolean needsUpdate;
    private AxisAlignedBB rendererBoundingBox;
    private RenderBlocks renderBlocks;
    public boolean isVisible;
    public boolean isWaitingOnOcclusionQuery;
    public int glOcclusionQuery;
    
    public WorldRenderer(final World world, final long xCoord, final long yCoord, final long zCoord) {
        this.isInFrustrum = false;
        this.skipRenderPass = new boolean[2];
        this.isVisible = true;
        this.renderBlocks = new RenderBlocks(world);
        this.worldObj = world;
        this.posX = xCoord;
        this.posY = yCoord;
        this.posZ = zCoord;
        final int sizeWidth = 16;
        this.sizeDepth = sizeWidth;
        this.sizeHeight = sizeWidth;
        this.sizeWidth = sizeWidth;
        this.posXPlus = xCoord + this.sizeWidth / 2;
        this.posYPlus = yCoord + this.sizeHeight / 2;
        this.posZPlus = zCoord + this.sizeDepth / 2;
        MathHelper.sqrt_double((double)(this.sizeWidth * this.sizeWidth + this.sizeHeight * this.sizeHeight + this.sizeDepth * this.sizeDepth));
        this.rendererBoundingBox = new AxisAlignedBB((double)xCoord, (double)yCoord, (double)zCoord, (double)(xCoord + this.sizeWidth), (double)(yCoord + this.sizeHeight), (double)(zCoord + this.sizeDepth)).expand(2.0f, 2.0f, 2.0f);
        this.glRenderList = GL11.glGenLists(3);
        this.setDontDraw();
        GL11.glDisable(3553);
        GL11.glNewList(glRenderList + 2, 4864);
        Render.renderOffsetAABB(this.rendererBoundingBox);
        GL11.glEndList();
        GL11.glEnable(3553);
    }
    
    public final void updateRenderer() {
        if (!this.needsUpdate) {
            return;
        }
        ++WorldRenderer.chunksUpdated;
        final long posX = this.posX;
        final long posY = this.posY;
        final long posZ = this.posZ;
        final long n = this.posX + this.sizeWidth;
        final long n2 = this.posY + this.sizeHeight;
        final long n3 = this.posZ + this.sizeDepth;
        for (int i = 0; i < 2; ++i) {
            this.skipRenderPass[i] = true;
        }
        Chunk chunk = worldObj.getChunk(MathHelper.floor_double_long(posX/(double)Chunk.WIDTH),MathHelper.floor_double_long(posY/(double)Chunk.HEIGHT),MathHelper.floor_double_long(posZ/(double)Chunk.WIDTH));

        for (int i = 0; i < 2; ++i) {
            boolean b = false;
            boolean b2 = false;
            WorldRenderer.tessellator.startDrawingQuads();
//            tessellator.setTranslation(-posX, -posY, -posZ);
            GL11.glNewList(this.glRenderList + i, 4864);
            for (int y = 0; y < 16; ++y) {
                for (int z = 0; z < 16; ++z) {
                    //int n4 = (j * this.worldObj.length + k) * this.worldObj.width + posX;
                    for (int x = 0; x < 16; ++x) {
                        final int n5;
                        if ((n5 = (chunk.getBlock(x, y, z))) > 0) {
                            final Block block;
                            if ((block = Block.blocksList[n5]).getRenderBlockPass() != i) {
                                b = true;
                            }
                            else {
                                b2 |= this.renderBlocks.renderBlockByRenderType(block, posX+x, posY+y, posZ+z);
                            }
                        }
                    }
                }
            }
            tessellator.setTranslation(0, 0, 0);
            WorldRenderer.tessellator.draw();
            GL11.glEndList();
            if (b2) {
                this.skipRenderPass[i] = false;
            }
            if (!b) {
                break;
            }
        }
        needsUpdate = false;
    }
    
    public final double distanceToEntitySquared(final EntityLiving e) {
        final double n = e.posX - this.posXPlus;
        final double n2 = e.posY - this.posYPlus;
        final double n3 = e.posZ - this.posZPlus;
        return n * n + n2 * n2 + n3 * n3;
    }
    
    private void setDontDraw() {
        for (int i = 0; i < 2; ++i) {
            this.skipRenderPass[i] = true;
        }
    }
    
    public final void stopRendering() {
        this.setDontDraw();
        this.worldObj = null;
    }
    
    public final void updateInFrustrum(final ICamera e) {
//    	this.isInFrustrum = true;
        this.isInFrustrum = e.isBoundingBoxInFrustrum(this.rendererBoundingBox);
    }
    
    public final void callOcclusionQueryList() {
        GL11.glCallList(this.glRenderList + 2);
    }
    
    protected void finalize() throws Throwable {
    	GL11.glDeleteLists(glRenderList, 2);
    }
    public void render(int pass) {
    	if(skipRenderPass[pass]) return;
    	GL11.glCallList(glRenderList + pass);
    }
    
    static {
        WorldRenderer.tessellator = Tessellator.instance;
        WorldRenderer.chunksUpdated = 0;
    }
}
