package net.minecraft.client.renderer.entity;

import org.lwjgl.opengl.GL11;
import java.util.Iterator;
import net.minecraft.game.entity.other.EntityTNT;
import net.minecraft.game.entity.other.EntityItem;
import net.minecraft.game.entity.other.EntityArrow;
import net.minecraft.game.entity.other.EntityPainting;
import net.minecraft.game.entity.Entity;
import net.minecraft.client.model.ModelBiped;
import net.minecraft.game.entity.EntityLiving;
import net.minecraft.game.entity.monster.EntityGiant;
import net.minecraft.game.entity.player.EntityPlayer;
import net.minecraft.client.model.ModelZombie;
import net.minecraft.game.entity.monster.EntityZombie;
import net.minecraft.client.model.ModelSkeleton;
import net.minecraft.game.entity.monster.EntitySkeleton;
import net.minecraft.game.entity.monster.EntityCreeper;
import net.minecraft.client.model.ModelSheepFur;
import net.minecraft.client.model.ModelSheep;
import net.minecraft.game.entity.animal.EntitySheep;
import net.minecraft.client.model.ModelBase;
import net.minecraft.client.model.ModelPig;
import net.minecraft.game.entity.animal.EntityPig;
import net.minecraft.game.entity.monster.EntitySpider;
import java.util.HashMap;
import net.minecraft.game.level.World;
import net.minecraft.client.renderer.RenderEngine;
import net.minecraft.client.renderer.Vec3D;

import java.util.Map;

public final class RenderManager {
    private Map entityRenderMap;
    public static RenderManager instance;
    public RenderEngine renderEngine;
    public World worldObj;
    public double playerViewY;
    private double tickPosX;
    private double tickPosY;
    private double tickPosZ;
    
    private RenderManager() {
        (this.entityRenderMap = (Map)new HashMap()).put(EntitySpider.class, new RenderSpider());
        this.entityRenderMap.put(EntityPig.class, new RenderLiving(new ModelPig(), 0.7f));
        this.entityRenderMap.put(EntitySheep.class, new RenderSheep(new ModelSheep(), new ModelSheepFur(), 0.7f));
        this.entityRenderMap.put(EntityCreeper.class, new RenderCreeper());
        this.entityRenderMap.put(EntitySkeleton.class, new RenderLiving(new ModelSkeleton(), 0.5f));
        this.entityRenderMap.put(EntityZombie.class, new RenderLiving(new ModelZombie(), 0.5f));
        this.entityRenderMap.put(EntityPlayer.class, new RenderPlayer());
        this.entityRenderMap.put(EntityGiant.class, new RenderGiantZombie(new ModelZombie(), 0.5f, 6.0f));
        this.entityRenderMap.put(EntityLiving.class, new RenderLiving(new ModelBiped(), 0.5f));
        this.entityRenderMap.put(Entity.class, new RenderEntity());
        this.entityRenderMap.put(EntityPainting.class, new RenderPainting());
        this.entityRenderMap.put(EntityArrow.class, new RenderArrow());
        this.entityRenderMap.put(EntityItem.class, new RenderItem());
        this.entityRenderMap.put(EntityTNT.class, new RenderTNTPrimed());
        final Iterator iterator = this.entityRenderMap.values().iterator();
        while (iterator.hasNext()) {
            ((Render)iterator.next()).setRenderManager(this);
        }
    }
    
    public final Render getEntityRenderObject(final Entity entity) {
        final Class class1 = entity.getClass();
        Render render;
        if ((render = (Render)this.entityRenderMap.get(class1)) == null && class1 != Entity.class) {
            render = (Render)this.entityRenderMap.get(class1.getSuperclass());
            this.entityRenderMap.put(class1, render);
        }
        return render;
    }
    
    public final void cacheActiveRenderInfo(final World world, final RenderEngine renderEngine, final EntityPlayer entityPlayer, final double double4) {
        this.worldObj = world;
        this.renderEngine = renderEngine;
        this.playerViewY = entityPlayer.prevRotationYaw + (entityPlayer.rotationYaw - entityPlayer.prevRotationYaw) * double4;
        this.tickPosX = entityPlayer.lastTickPosX + (entityPlayer.posX - entityPlayer.lastTickPosX) * double4;
        this.tickPosY = entityPlayer.lastTickPosY + (entityPlayer.posY - entityPlayer.lastTickPosY) * double4;
        this.tickPosZ = entityPlayer.lastTickPosZ + (entityPlayer.posZ - entityPlayer.lastTickPosZ) * double4;
    }
    
    public final void renderEntity(final Entity entity, final double double2, Vec3D vec3d) {
        final double double3 = entity.lastTickPosX + (entity.posX - entity.lastTickPosX) * double2;
        final double double4 = entity.lastTickPosY + (entity.posY - entity.lastTickPosY) * double2;
        final double double5 = entity.lastTickPosZ + (entity.posZ - entity.lastTickPosZ) * double2;
        
        final double double6 = entity.prevRotationYaw + (entity.rotationYaw - entity.prevRotationYaw) * double2;
        final double lightBrightness;
        GL11.glColor3d(lightBrightness = this.worldObj.getLightBrightness((int)double3, (int)(double4 + entity.getShadowSize()), (int)double5), lightBrightness, lightBrightness);
        this.renderEntityWithPosYaw(entity, double3 - vec3d.xCoord, double4 - vec3d.yCoord, double5 - vec3d.zCoord, double6, double2);
    }
    
    public final void renderEntityWithPosYaw(final Entity entity, final double double2, final double double3, final double double4, final double double5, final double double6) {
        final Render entityRenderObject;
        if ((entityRenderObject = this.getEntityRenderObject(entity)) != null) {
            entityRenderObject.doRender(entity, double2, double3, double4, double5, double6);
            entityRenderObject.renderShadow(entity, double2, double3, double4, double6);
        }
    }
    
    public final void set(final World world) {
        this.worldObj = world;
    }
    
    public final double getDistanceToCamera(double double1, double double2, double double3) {
        double1 -= this.tickPosX;
        double2 -= this.tickPosY;
        double3 -= this.tickPosZ;
        return double1 * double1 + double2 * double2 + double3 * double3;
    }
    
    static {
        RenderManager.instance = new RenderManager();
    }
}
