package net.minecraft.game.block;

import net.minecraft.game.item.ItemBlock;
import net.minecraft.game.item.Item;
import net.minecraft.client.physics.MovingObjectPosition;
import net.minecraft.client.renderer.Vec3D;
import net.minecraft.game.entity.Entity;
import net.minecraft.game.entity.other.EntityItem;
import net.minecraft.game.item.ItemStack;
import net.minecraft.game.entity.player.InventoryPlayer;
import net.minecraft.game.entity.player.EntityPlayer;
import java.util.Random;
import net.minecraft.client.physics.AxisAlignedBB;
import net.minecraft.game.level.World;

public class Block {
    private static StepSound soundPowderFootstep;
    private static StepSound soundWoodFootstep;
    private static StepSound soundGravelFootstep;
    private static StepSound soundGrassFootstep;
    private static StepSound soundStoneFootstep;
    private static StepSound soundMetalFootstep;
    private static StepSound soundGlassFootstep;
    private static StepSound soundClothFootstep;
    private static StepSound soundSandFootstep;
    public static final Block[] blocksList;
    public static final boolean[] tickOnLoad;
    public static final boolean[] opaqueCubeLookup;
    public static final int[] lightOpacity;
    private static boolean[] canBlockGrass;
    public static final boolean[] isBlockContainer;
    public static final int[] lightValue;
    public static final Block stone;
    public static final BlockGrass grass;
    public static final Block dirt;
    public static final Block cobblestone;
    public static final Block planks;
    public static final Block sapling;
    public static final Block bedrock;
    public static final Block waterMoving;
    public static final Block waterStill;
    public static final Block lavaMoving;
    public static final Block lavaStill;
    public static final Block sand;
    public static final Block gravel;
    public static final Block oreGold;
    public static final Block oreIron;
    public static final Block oreCoal;
    public static final Block wood;
    public static final Block leaves;
    public static final Block sponge;
    public static final Block glass;
    public static final Block woolRed;
    public static final Block woolOrange;
    public static final Block woolYellow;
    public static final Block woolLime;
    public static final Block woolGreen;
    public static final Block woolAquaGreen;
    public static final Block woolCyan;
    public static final Block woolBlue;
    public static final Block woolPurple;
    public static final Block woolIndigo;
    public static final Block woolViolet;
    public static final Block woolMagenta;
    public static final Block woolPink;
    public static final Block woolDarkGray;
    public static final Block woolGray;
    public static final Block woolWhite;
    public static final BlockFlower plantYellow;
    public static final BlockFlower plantRed;
    public static final BlockFlower mushroomBrown;
    public static final BlockFlower mushroomRed;
    public static final Block blockGold;
    public static final Block blockSteel;
    public static final Block stairDouble;
    public static final Block stairSingle;
    public static final Block brick;
    public static final Block tnt;
    public static final Block bookShelf;
    public static final Block cobblestoneMossy;
    public static final Block obsidian;
    public static final Block torchWood;
    public static final BlockFire fire;
    public static final Block waterSource;
    public static final Block lavaSource;
    public static final Block chest;
    public static final Block gear;
    public static final Block oreDiamond;
    public static final Block blockDiamond;
    public static final Block workbench;
    public static final Block crops;
    public static final Block tilledField;
    public static final Block stoneOvenIdle;
    public static final Block stoneOvenActive;
    public int blockIndexInTexture;
    public final int blockID;
    private double blockHardness;
    private double blockResistance;
    public double minX;
    public double minY;
    public double minZ;
    public double maxX;
    public double maxY;
    public double maxZ;
    public StepSound stepSound;
    public double blockParticleGravity;
    public final Material blockMaterial;
    
    protected Block(final int blockID, final Material material) {
        this.stepSound = Block.soundPowderFootstep;
        this.blockParticleGravity = 1.0f;
        if (Block.blocksList[blockID] != null) {
            throw new IllegalArgumentException(new StringBuilder().append("Slot ").append(blockID).append(" is already occupied by ").append(Block.blocksList[blockID]).append(" when adding ").append(this).toString());
        }
        this.blockMaterial = material;
        Block.blocksList[blockID] = this;
        this.blockID = blockID;
        this.setBlockBounds(0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 1.0f);
        Block.opaqueCubeLookup[blockID] = this.isOpaqueCube();
        Block.lightOpacity[blockID] = (this.isOpaqueCube() ? 255 : 0);
        Block.canBlockGrass[blockID] = this.renderAsNormalBlock();
        Block.isBlockContainer[blockID] = false;
    }
    
    protected Block(final int blockID, final int blockIndexInTexture, final Material material) {
        this(blockID, material);
        this.blockIndexInTexture = blockIndexInTexture;
    }
    
    protected final Block setLightOpacity(final int integer) {
        Block.lightOpacity[this.blockID] = integer;
        return this;
    }
    
    private Block setLightValue(final double double1) {
        Block.lightValue[this.blockID] = (int)(15.0f * double1);
        return this;
    }
    
    protected final Block setResistance(final double double1) {
        this.blockResistance = double1 * 3.0f;
        return this;
    }
    
    public boolean renderAsNormalBlock() {
        return true;
    }
    
    public int getRenderType() {
        return 0;
    }
    
    protected final Block setHardness(final double double1) {
        this.blockHardness = double1;
        if (this.blockResistance < double1 * 5.0f) {
            this.blockResistance = double1 * 5.0f;
        }
        return this;
    }
    
    protected final void setTickOnLoad(final boolean boolean1) {
        Block.tickOnLoad[this.blockID] = boolean1;
    }
    
    protected final void setBlockBounds(final double minX, final double minY, final double minZ, final double maxX, final double maxY, final double maxZ) {
        this.minX = minX;
        this.minY = minY;
        this.minZ = minZ;
        this.maxX = maxX;
        this.maxY = maxY;
        this.maxZ = maxZ;
    }
    
    public double getBlockBrightness(final World world, final long m, final long l, final long renderType) {
        return world.getLightBrightness(m, l, renderType);
    }
    
    public boolean shouldSideBeRendered(final World world, final long xCoord, final long yCoord, final long zCoord, final int integer5) {
        return !world.isBlockNormalCube(xCoord, yCoord, zCoord);
    }
    
    public int getBlockTexture(final World world, final long xCoord, final long yCoord, final long zCoord, final int blockSide) {
        return this.getBlockTextureFromSideAndMetadata(blockSide, world.getBlockMetadata(xCoord, yCoord, zCoord));
    }
    
    public int getBlockTextureFromSideAndMetadata(final int integer1, final int integer2) {
        return this.getBlockTextureFromSide(integer1);
    }
    
    public int getBlockTextureFromSide(final int blockSide) {
        return this.blockIndexInTexture;
    }
    
    public final AxisAlignedBB getSelectedBoundingBoxFromPool(final long xCoord, final long yCoord, final long zCoord) {
        return new AxisAlignedBB(xCoord + this.minX, yCoord + this.minY, zCoord + this.minZ, xCoord + this.maxX, yCoord + this.maxY, zCoord + this.maxZ);
    }
    
    public AxisAlignedBB getCollisionBoundingBoxFromPool(final long xCoord, final long yCoord, final long zCoord) {
        return new AxisAlignedBB(xCoord + this.minX, yCoord + this.minY, zCoord + this.minZ, xCoord + this.maxX, yCoord + this.maxY, zCoord + this.maxZ);
    }
    
    public boolean isOpaqueCube() {
        return true;
    }
    
    public boolean isCollidable() {
        return true;
    }
    
    public void updateTick(final World world, final long xCoord, final long yCoord, final long zCoord, final Random random) {
    }
    
    public void randomDisplayTick(final World world, final long xCoord, final long yCoord, final long zCoord, final Random random) {
    }
    
    public void onBlockDestroyedByPlayer(final World world, final long l, final long yCoord, final long zCoord, final int integer5) {
    }
    
    public void onNeighborBlockChange(final World world, final long integer1, final long integer2, final long integer3, final int integer4) {
    }
    
    public int tickRate() {
        return 5;
    }
    
    public void onBlockAdded(final World world, final long integer1, final long integer2, final long integer3) {
    }
    
    public void onBlockRemoval(final World world, final long integer1, final long integer2, final long integer3) {
    }
    
    public int quantityDropped(final Random random) {
        return 1;
    }
    
    public int idDropped(final int integer, final Random random) {
        return this.blockID;
    }
    
    public final double blockStrength(EntityPlayer player) {
        if (this.blockHardness < 0.0f) {
            return 0.0f;
        }
        if (!player.canHarvestBlock(this)) {
            return 1.0f / this.blockHardness / 100.0f;
        }
        final InventoryPlayer inventory = (player = player).inventory;
        double n = 1.0f;
        if (inventory.mainInventory[inventory.currentItem] != null) {
            n = 1.0f * inventory.mainInventory[inventory.currentItem].getItem().getStrVsBlock(this);
        }
        double n2 = n;
        if (player.isInsideOfMaterial()) {
            n2 /= 5.0f;
        }
        if (!player.onGround) {
            n2 /= 5.0f;
        }
        return n2 / this.blockHardness / 30.0f;
    }
    
    public final void dropBlockAsItem(final World world, final long xCoord, final long yCoord, final long zCoord, final int integer5) {
        this.dropBlockAsItemWithChance(world, xCoord, yCoord, zCoord, integer5, 1.0f);
    }
    
    public final void dropBlockAsItemWithChance(final World world, final long xCoord, final long yCoord, final long zCoord, final int integer5, final double double6) {
        for (int quantityDropped = this.quantityDropped(world.random), i = 0; i < quantityDropped; ++i) {
            final int idDropped;
            if (world.random.nextDouble() <= double6 && (idDropped = this.idDropped(integer5, world.random)) > 0) {
                final EntityItem entity;
                (entity = new EntityItem(world, xCoord + (world.random.nextDouble() * 0.7f + 0.15f), yCoord + (world.random.nextDouble() * 0.7f + 0.15f), zCoord + (world.random.nextDouble() * 0.7f + 0.15f), new ItemStack(idDropped))).delayBeforeCanPickup = 10;
                world.entityJoinedWorld(entity);
            }
        }
    }
    
    public final double getExplosionResistance() {
        return this.blockResistance / 5.0f;
    }
    
    public MovingObjectPosition collisionRayTrace(final World world, final long integer2, final long integer3, final long integer4, Vec3D vec3D, Vec3D vec3D2) {
        vec3D = vec3D.addVector((double)(-integer2), (double)(-integer3), (double)(-integer4));
        vec3D2 = vec3D2.addVector((double)(-integer2), (double)(-integer3), (double)(-integer4));
        Vec3D intermediateWithXValue = vec3D.getIntermediateWithXValue(vec3D2, this.minX);
        Vec3D intermediateWithXValue2 = vec3D.getIntermediateWithXValue(vec3D2, this.maxX);
        Vec3D intermediateWithYValue = vec3D.getIntermediateWithYValue(vec3D2, this.minY);
        Vec3D intermediateWithYValue2 = vec3D.getIntermediateWithYValue(vec3D2, this.maxY);
        Vec3D intermediateWithZValue = vec3D.getIntermediateWithZValue(vec3D2, this.minZ);
        vec3D2 = vec3D.getIntermediateWithZValue(vec3D2, this.maxZ);
        if (!this.isVecInsideYZBounds(intermediateWithXValue)) {
            intermediateWithXValue = null;
        }
        if (!this.isVecInsideYZBounds(intermediateWithXValue2)) {
            intermediateWithXValue2 = null;
        }
        if (!this.isVecInsideXZBounds(intermediateWithYValue)) {
            intermediateWithYValue = null;
        }
        if (!this.isVecInsideXZBounds(intermediateWithYValue2)) {
            intermediateWithYValue2 = null;
        }
        if (!this.isVecInsideXYBounds(intermediateWithZValue)) {
            intermediateWithZValue = null;
        }
        if (!this.isVecInsideXYBounds(vec3D2)) {
            vec3D2 = null;
        }
        Vec3D vec3D3 = null;
        if (intermediateWithXValue != null) {
            vec3D3 = intermediateWithXValue;
        }
        if (intermediateWithXValue2 != null && (vec3D3 == null || vec3D.distance(intermediateWithXValue2) < vec3D.distance(vec3D3))) {
            vec3D3 = intermediateWithXValue2;
        }
        if (intermediateWithYValue != null && (vec3D3 == null || vec3D.distance(intermediateWithYValue) < vec3D.distance(vec3D3))) {
            vec3D3 = intermediateWithYValue;
        }
        if (intermediateWithYValue2 != null && (vec3D3 == null || vec3D.distance(intermediateWithYValue2) < vec3D.distance(vec3D3))) {
            vec3D3 = intermediateWithYValue2;
        }
        if (intermediateWithZValue != null && (vec3D3 == null || vec3D.distance(intermediateWithZValue) < vec3D.distance(vec3D3))) {
            vec3D3 = intermediateWithZValue;
        }
        if (vec3D2 != null && (vec3D3 == null || vec3D.distance(vec3D2) < vec3D.distance(vec3D3))) {
            vec3D3 = vec3D2;
        }
        if (vec3D3 == null) {
            return null;
        }
        int sideHit = -1;
        if (vec3D3 == intermediateWithXValue) {
            sideHit = 4;
        }
        if (vec3D3 == intermediateWithXValue2) {
            sideHit = 5;
        }
        if (vec3D3 == intermediateWithYValue) {
            sideHit = 0;
        }
        if (vec3D3 == intermediateWithYValue2) {
            sideHit = 1;
        }
        if (vec3D3 == intermediateWithZValue) {
            sideHit = 2;
        }
        if (vec3D3 == vec3D2) {
            sideHit = 3;
        }
        return new MovingObjectPosition(integer2, integer3, integer4, sideHit, vec3D3.addVector((double)integer2, (double)integer3, (double)integer4));
    }
    
    private boolean isVecInsideYZBounds(final Vec3D vec3D) {
        return vec3D != null && (vec3D.yCoord >= this.minY && vec3D.yCoord <= this.maxY && vec3D.zCoord >= this.minZ && vec3D.zCoord <= this.maxZ);
    }
    
    private boolean isVecInsideXZBounds(final Vec3D vec3D) {
        return vec3D != null && (vec3D.xCoord >= this.minX && vec3D.xCoord <= this.maxX && vec3D.zCoord >= this.minZ && vec3D.zCoord <= this.maxZ);
    }
    
    private boolean isVecInsideXYBounds(final Vec3D vec3D) {
        return vec3D != null && (vec3D.xCoord >= this.minX && vec3D.xCoord <= this.maxX && vec3D.yCoord >= this.minY && vec3D.yCoord <= this.maxY);
    }
    
    public void onBlockDestroyedByExplosion(final World world, final long xCoord, final long yCoord, final long zCoord) {
    }
    
    public int getRenderBlockPass() {
        return 0;
    }
    
    public boolean canPlaceBlockAt(final World world, final long xCoord, final long yCoord, final long zCoord) {
        return true;
    }
    
    public boolean blockActivated(final World world, final long xCoord, final long yCoord, final long zCoord, final EntityPlayer player) {
        return false;
    }
    
    public void onEntityWalking(final World world, final int integer2, final int integer3, final int integer4) {
    }
    
    public void onBlockPlaced(final World world, final long integer2, final long integer3, final long integer4, final int integer5) {
    }
    
    static {
        Block.soundPowderFootstep = new StepSound("stone", 1.0f, 1.0f);
        Block.soundWoodFootstep = new StepSound("wood", 1.0f, 1.0f);
        Block.soundGravelFootstep = new StepSound("gravel", 1.0f, 1.0f);
        Block.soundGrassFootstep = new StepSound("grass", 1.0f, 1.0f);
        Block.soundStoneFootstep = new StepSound("stone", 1.0f, 1.0f);
        Block.soundMetalFootstep = new StepSound("stone", 1.0f, 1.5f);
        Block.soundGlassFootstep = new StepSoundStone("stone", 1.0f, 1.0f);
        Block.soundClothFootstep = new StepSound("cloth", 1.0f, 1.0f);
        Block.soundSandFootstep = new StepSoundSand("sand", 1.0f, 1.0f);
        blocksList = new Block[256];
        tickOnLoad = new boolean[256];
        opaqueCubeLookup = new boolean[256];
        lightOpacity = new int[256];
        Block.canBlockGrass = new boolean[256];
        isBlockContainer = new boolean[256];
        lightValue = new int[256];
        final Block setResistance = new BlockStone(1, 1).setHardness(1.5f).setResistance(10.0f);
        final StepSound soundStoneFootstep = Block.soundStoneFootstep;
        final Block stone2 = setResistance;
        setResistance.stepSound = soundStoneFootstep;
        stone = stone2;
        final Block setHardness = new BlockGrass(2).setHardness(0.6f);
        final StepSound soundGrassFootstep = Block.soundGrassFootstep;
        final BlockGrass blockGrass = (BlockGrass)setHardness;
        setHardness.stepSound = soundGrassFootstep;
        grass = blockGrass;
        final Block setHardness2 = new BlockDirt(3, 2).setHardness(0.5f);
        final StepSound soundGravelFootstep = Block.soundGravelFootstep;
        final Block dirt2 = setHardness2;
        setHardness2.stepSound = soundGravelFootstep;
        dirt = dirt2;
        final Block setResistance2 = new Block(4, 16, Material.rock).setHardness(2.0f).setResistance(10.0f);
        final StepSound soundStoneFootstep2 = Block.soundStoneFootstep;
        final Block cobblestone2 = setResistance2;
        setResistance2.stepSound = soundStoneFootstep2;
        cobblestone = cobblestone2;
        final Block setResistance3 = new Block(5, 4, Material.wood).setHardness(2.0f).setResistance(5.0f);
        final StepSound soundWoodFootstep = Block.soundWoodFootstep;
        final Block planks2 = setResistance3;
        setResistance3.stepSound = soundWoodFootstep;
        planks = planks2;
        final Block setHardness3 = new BlockSapling(6, 15).setHardness(0.0f);
        final StepSound soundGrassFootstep2 = Block.soundGrassFootstep;
        final Block sapling2 = setHardness3;
        setHardness3.stepSound = soundGrassFootstep2;
        sapling = sapling2;
        final Block setResistance4 = new Block(7, 17, Material.rock).setHardness(-1.0f).setResistance(6000000.0f);
        final StepSound soundStoneFootstep3 = Block.soundStoneFootstep;
        final Block bedrock2 = setResistance4;
        setResistance4.stepSound = soundStoneFootstep3;
        bedrock = bedrock2;
        waterMoving = new BlockFluidFlowing(8, Material.water).setHardness(100.0f).setLightOpacity(3);
        waterStill = new BlockFluidStill(9, Material.water).setHardness(100.0f).setLightOpacity(3);
        lavaMoving = new BlockFluidFlowing(10, Material.lava).setHardness(0.0f).setLightValue(1.0f).setLightOpacity(255);
        lavaStill = new BlockFluidStill(11, Material.lava).setHardness(100.0f).setLightValue(1.0f).setLightOpacity(255);
        final Block setHardness4 = new BlockSand(12, 18).setHardness(0.5f);
        final StepSound soundSandFootstep = Block.soundSandFootstep;
        final Block sand2 = setHardness4;
        setHardness4.stepSound = soundSandFootstep;
        sand = sand2;
        final Block setHardness5 = new BlockGravel(13, 19).setHardness(0.6f);
        final StepSound soundGravelFootstep2 = Block.soundGravelFootstep;
        final Block gravel2 = setHardness5;
        setHardness5.stepSound = soundGravelFootstep2;
        gravel = gravel2;
        final Block setResistance5 = new BlockOre(14, 32).setHardness(3.0f).setResistance(5.0f);
        final StepSound soundStoneFootstep4 = Block.soundStoneFootstep;
        final Block oreGold2 = setResistance5;
        setResistance5.stepSound = soundStoneFootstep4;
        oreGold = oreGold2;
        final Block setResistance6 = new BlockOre(15, 33).setHardness(3.0f).setResistance(5.0f);
        final StepSound soundStoneFootstep5 = Block.soundStoneFootstep;
        final Block oreIron2 = setResistance6;
        setResistance6.stepSound = soundStoneFootstep5;
        oreIron = oreIron2;
        final Block setResistance7 = new BlockOre(16, 34).setHardness(3.0f).setResistance(5.0f);
        final StepSound soundStoneFootstep6 = Block.soundStoneFootstep;
        final Block oreCoal2 = setResistance7;
        setResistance7.stepSound = soundStoneFootstep6;
        oreCoal = oreCoal2;
        final Block setHardness6 = new BlockLog(17).setHardness(2.0f);
        final StepSound soundWoodFootstep2 = Block.soundWoodFootstep;
        final Block wood2 = setHardness6;
        setHardness6.stepSound = soundWoodFootstep2;
        wood = wood2;
        final Block setLightOpacity = new BlockLeaves(18, 52).setHardness(0.2f).setLightOpacity(1);
        final StepSound soundGrassFootstep3 = Block.soundGrassFootstep;
        final Block leaves2 = setLightOpacity;
        setLightOpacity.stepSound = soundGrassFootstep3;
        leaves = leaves2;
        final Block setHardness7 = new BlockSponge(19).setHardness(0.6f);
        final StepSound soundGrassFootstep4 = Block.soundGrassFootstep;
        final Block sponge2 = setHardness7;
        setHardness7.stepSound = soundGrassFootstep4;
        sponge = sponge2;
        final Block setHardness8 = new BlockGlass(20, 49, Material.glass, false).setHardness(0.3f);
        final StepSound soundGlassFootstep = Block.soundGlassFootstep;
        final Block glass2 = setHardness8;
        setHardness8.stepSound = soundGlassFootstep;
        glass = glass2;
        final Block setHardness9 = new Block(21, 64, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep = Block.soundClothFootstep;
        final Block woolRed2 = setHardness9;
        setHardness9.stepSound = soundClothFootstep;
        woolRed = woolRed2;
        final Block setHardness10 = new Block(22, 65, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep2 = Block.soundClothFootstep;
        final Block woolOrange2 = setHardness10;
        setHardness10.stepSound = soundClothFootstep2;
        woolOrange = woolOrange2;
        final Block setHardness11 = new Block(23, 66, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep3 = Block.soundClothFootstep;
        final Block woolYellow2 = setHardness11;
        setHardness11.stepSound = soundClothFootstep3;
        woolYellow = woolYellow2;
        final Block setHardness12 = new Block(24, 67, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep4 = Block.soundClothFootstep;
        final Block woolLime2 = setHardness12;
        setHardness12.stepSound = soundClothFootstep4;
        woolLime = woolLime2;
        final Block setHardness13 = new Block(25, 68, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep5 = Block.soundClothFootstep;
        final Block woolGreen2 = setHardness13;
        setHardness13.stepSound = soundClothFootstep5;
        woolGreen = woolGreen2;
        final Block setHardness14 = new Block(26, 69, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep6 = Block.soundClothFootstep;
        final Block woolAquaGreen2 = setHardness14;
        setHardness14.stepSound = soundClothFootstep6;
        woolAquaGreen = woolAquaGreen2;
        final Block setHardness15 = new Block(27, 70, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep7 = Block.soundClothFootstep;
        final Block woolCyan2 = setHardness15;
        setHardness15.stepSound = soundClothFootstep7;
        woolCyan = woolCyan2;
        final Block setHardness16 = new Block(28, 71, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep8 = Block.soundClothFootstep;
        final Block woolBlue2 = setHardness16;
        setHardness16.stepSound = soundClothFootstep8;
        woolBlue = woolBlue2;
        final Block setHardness17 = new Block(29, 72, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep9 = Block.soundClothFootstep;
        final Block woolPurple2 = setHardness17;
        setHardness17.stepSound = soundClothFootstep9;
        woolPurple = woolPurple2;
        final Block setHardness18 = new Block(30, 73, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep10 = Block.soundClothFootstep;
        final Block woolIndigo2 = setHardness18;
        setHardness18.stepSound = soundClothFootstep10;
        woolIndigo = woolIndigo2;
        final Block setHardness19 = new Block(31, 74, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep11 = Block.soundClothFootstep;
        final Block woolViolet2 = setHardness19;
        setHardness19.stepSound = soundClothFootstep11;
        woolViolet = woolViolet2;
        final Block setHardness20 = new Block(32, 75, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep12 = Block.soundClothFootstep;
        final Block woolMagenta2 = setHardness20;
        setHardness20.stepSound = soundClothFootstep12;
        woolMagenta = woolMagenta2;
        final Block setHardness21 = new Block(33, 76, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep13 = Block.soundClothFootstep;
        final Block woolPink2 = setHardness21;
        setHardness21.stepSound = soundClothFootstep13;
        woolPink = woolPink2;
        final Block setHardness22 = new Block(34, 77, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep14 = Block.soundClothFootstep;
        final Block woolDarkGray2 = setHardness22;
        setHardness22.stepSound = soundClothFootstep14;
        woolDarkGray = woolDarkGray2;
        final Block setHardness23 = new Block(35, 78, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep15 = Block.soundClothFootstep;
        final Block woolGray2 = setHardness23;
        setHardness23.stepSound = soundClothFootstep15;
        woolGray = woolGray2;
        final Block setHardness24 = new Block(36, 79, Material.cloth).setHardness(0.8f);
        final StepSound soundClothFootstep16 = Block.soundClothFootstep;
        final Block woolWhite2 = setHardness24;
        setHardness24.stepSound = soundClothFootstep16;
        woolWhite = woolWhite2;
        final Block setHardness25 = new BlockFlower(37, 13).setHardness(0.0f);
        final StepSound soundGrassFootstep5 = Block.soundGrassFootstep;
        final BlockFlower blockFlower = (BlockFlower)setHardness25;
        setHardness25.stepSound = soundGrassFootstep5;
        plantYellow = blockFlower;
        final Block setHardness26 = new BlockFlower(38, 12).setHardness(0.0f);
        final StepSound soundGrassFootstep6 = Block.soundGrassFootstep;
        final BlockFlower blockFlower2 = (BlockFlower)setHardness26;
        setHardness26.stepSound = soundGrassFootstep6;
        plantRed = blockFlower2;
        final Block setHardness27 = new BlockMushroom(39, 29).setHardness(0.0f);
        final StepSound soundGrassFootstep7 = Block.soundGrassFootstep;
        final Block block = setHardness27;
        setHardness27.stepSound = soundGrassFootstep7;
        mushroomBrown = (BlockFlower)block.setLightValue(0.125f);
        final Block setHardness28 = new BlockMushroom(40, 28).setHardness(0.0f);
        final StepSound soundGrassFootstep8 = Block.soundGrassFootstep;
        final BlockFlower blockFlower3 = (BlockFlower)setHardness28;
        setHardness28.stepSound = soundGrassFootstep8;
        mushroomRed = blockFlower3;
        final Block setResistance8 = new BlockOreStorage(41, 39).setHardness(3.0f).setResistance(10.0f);
        final StepSound soundMetalFootstep = Block.soundMetalFootstep;
        final Block blockGold2 = setResistance8;
        setResistance8.stepSound = soundMetalFootstep;
        blockGold = blockGold2;
        final Block setResistance9 = new BlockOreStorage(42, 38).setHardness(5.0f).setResistance(10.0f);
        final StepSound soundMetalFootstep2 = Block.soundMetalFootstep;
        final Block blockSteel2 = setResistance9;
        setResistance9.stepSound = soundMetalFootstep2;
        blockSteel = blockSteel2;
        final Block setResistance10 = new BlockStep(43, true).setHardness(2.0f).setResistance(10.0f);
        final StepSound soundStoneFootstep7 = Block.soundStoneFootstep;
        final Block stairDouble2 = setResistance10;
        setResistance10.stepSound = soundStoneFootstep7;
        stairDouble = stairDouble2;
        final Block setResistance11 = new BlockStep(44, false).setHardness(2.0f).setResistance(10.0f);
        final StepSound soundStoneFootstep8 = Block.soundStoneFootstep;
        final Block stairSingle2 = setResistance11;
        setResistance11.stepSound = soundStoneFootstep8;
        stairSingle = stairSingle2;
        final Block setResistance12 = new Block(45, 7, Material.rock).setHardness(2.0f).setResistance(10.0f);
        final StepSound soundStoneFootstep9 = Block.soundStoneFootstep;
        final Block brick2 = setResistance12;
        setResistance12.stepSound = soundStoneFootstep9;
        brick = brick2;
        final Block setHardness29 = new BlockTNT(46, 8).setHardness(0.0f);
        final StepSound soundGrassFootstep9 = Block.soundGrassFootstep;
        final Block tnt2 = setHardness29;
        setHardness29.stepSound = soundGrassFootstep9;
        tnt = tnt2;
        final Block setHardness30 = new BlockBookshelf(47, 35).setHardness(1.5f);
        final StepSound soundWoodFootstep3 = Block.soundWoodFootstep;
        final Block bookShelf2 = setHardness30;
        setHardness30.stepSound = soundWoodFootstep3;
        bookShelf = bookShelf2;
        final Block setResistance13 = new Block(48, 36, Material.rock).setHardness(2.0f).setResistance(10.0f);
        final StepSound soundStoneFootstep10 = Block.soundStoneFootstep;
        final Block cobblestoneMossy2 = setResistance13;
        setResistance13.stepSound = soundStoneFootstep10;
        cobblestoneMossy = cobblestoneMossy2;
        final Block setResistance14 = new BlockStone(49, 37).setHardness(10.0f).setResistance(10.0f);
        final StepSound soundStoneFootstep11 = Block.soundStoneFootstep;
        final Block obsidian2 = setResistance14;
        setResistance14.stepSound = soundStoneFootstep11;
        obsidian = obsidian2;
        final Block setLightValue = new BlockTorch(50, 80).setHardness(0.0f).setLightValue(0.875f);
        final StepSound soundWoodFootstep4 = Block.soundWoodFootstep;
        final Block torchWood2 = setLightValue;
        setLightValue.stepSound = soundWoodFootstep4;
        torchWood = torchWood2;
        final Block setLightValue2 = new BlockFire(51, 31).setHardness(0.0f).setLightValue(1.0f);
        final StepSound soundWoodFootstep5 = Block.soundWoodFootstep;
        final BlockFire blockFire = (BlockFire)setLightValue2;
        setLightValue2.stepSound = soundWoodFootstep5;
        fire = blockFire;
        final Block setHardness31 = new BlockFluidSource(52, Block.waterMoving.blockID).setHardness(0.0f);
        final StepSound soundWoodFootstep6 = Block.soundWoodFootstep;
        final Block waterSource2 = setHardness31;
        setHardness31.stepSound = soundWoodFootstep6;
        waterSource = waterSource2;
        final Block setHardness32 = new BlockFluidSource(53, Block.lavaMoving.blockID).setHardness(0.0f);
        final StepSound soundWoodFootstep7 = Block.soundWoodFootstep;
        final Block lavaSource2 = setHardness32;
        setHardness32.stepSound = soundWoodFootstep7;
        lavaSource = lavaSource2;
        final Block setHardness33 = new BlockChest(54).setHardness(2.5f);
        final StepSound soundWoodFootstep8 = Block.soundWoodFootstep;
        final Block chest2 = setHardness33;
        setHardness33.stepSound = soundWoodFootstep8;
        chest = chest2;
        final Block setHardness34 = new BlockGears(55, 62).setHardness(0.5f);
        final StepSound soundMetalFootstep3 = Block.soundMetalFootstep;
        final Block gear2 = setHardness34;
        setHardness34.stepSound = soundMetalFootstep3;
        gear = gear2;
        final Block setResistance15 = new BlockOre(56, 50).setHardness(3.0f).setResistance(5.0f);
        final StepSound soundStoneFootstep12 = Block.soundStoneFootstep;
        final Block oreDiamond2 = setResistance15;
        setResistance15.stepSound = soundStoneFootstep12;
        oreDiamond = oreDiamond2;
        final Block setResistance16 = new BlockOreStorage(57, 40).setHardness(5.0f).setResistance(10.0f);
        final StepSound soundMetalFootstep4 = Block.soundMetalFootstep;
        final Block blockDiamond2 = setResistance16;
        setResistance16.stepSound = soundMetalFootstep4;
        blockDiamond = blockDiamond2;
        final Block setHardness35 = new BlockWorkbench(58).setHardness(2.5f);
        final StepSound soundWoodFootstep9 = Block.soundWoodFootstep;
        final Block workbench2 = setHardness35;
        setHardness35.stepSound = soundWoodFootstep9;
        workbench = workbench2;
        final Block setHardness36 = new BlockCrops(59, 88).setHardness(0.0f);
        final StepSound soundGrassFootstep10 = Block.soundGrassFootstep;
        final Block crops2 = setHardness36;
        setHardness36.stepSound = soundGrassFootstep10;
        crops = crops2;
        final Block setHardness37 = new BlockFarmland(60).setHardness(0.6f);
        final StepSound soundGravelFootstep3 = Block.soundGravelFootstep;
        final Block tilledField2 = setHardness37;
        setHardness37.stepSound = soundGravelFootstep3;
        tilledField = tilledField2;
        final Block setHardness38 = new BlockFurnace(61, false).setHardness(3.5f);
        final StepSound soundStoneFootstep13 = Block.soundStoneFootstep;
        final Block stoneOvenIdle2 = setHardness38;
        setHardness38.stepSound = soundStoneFootstep13;
        stoneOvenIdle = stoneOvenIdle2;
        final Block setHardness39 = new BlockFurnace(62, true).setHardness(3.5f);
        final StepSound soundStoneFootstep14 = Block.soundStoneFootstep;
        final Block block2 = setHardness39;
        setHardness39.stepSound = soundStoneFootstep14;
        stoneOvenActive = block2.setLightValue(0.875f);
        for (int i = 0; i < 256; ++i) {
            if (Block.blocksList[i] != null) {
                Item.itemsList[i] = new ItemBlock(i - 256);
            }
        }
    }
}
