package net.minecraft.game.block;

import net.minecraft.game.block.tileentity.TileEntity;
import net.minecraft.client.inventory.IInventory;
import net.minecraft.client.inventory.InventoryLargeChest;
import net.minecraft.game.entity.player.EntityPlayer;
import net.minecraft.game.entity.Entity;
import net.minecraft.game.entity.other.EntityItem;
import net.minecraft.game.item.ItemStack;
import net.minecraft.game.block.tileentity.TileEntityChest;
import net.minecraft.game.level.World;
import java.util.Random;

public final class BlockChest extends BlockContainer {
    private Random random;
    
    protected BlockChest(final int blockID) {
        super(54, Material.wood);
        this.random = new Random();
        this.blockIndexInTexture = 26;
    }
    
    @Override
    public final int getBlockTexture(final World world, long xCoord, final long yCoord, final long zCoord, final int blockSide) {
        if (blockSide == 1) {
            return this.blockIndexInTexture - 1;
        }
        if (blockSide == 0) {
            return this.blockIndexInTexture - 1;
        }
        final int blockId = world.getBlockId(xCoord, yCoord, zCoord - 1);
        final int blockId2 = world.getBlockId(xCoord, yCoord, zCoord + 1);
        final int blockId3 = world.getBlockId(xCoord - 1, yCoord, zCoord);
        final int blockId4 = world.getBlockId(xCoord + 1, yCoord, zCoord);
        if (blockId == this.blockID || blockId2 == this.blockID) {
            if (blockSide == 2 || blockSide == 3) {
                return this.blockIndexInTexture;
            }
            int n = 0;
            if (blockId == this.blockID) {
                n = -1;
            }
            final int n2 = world.getBlockId(xCoord - 1, yCoord, (blockId == this.blockID) ? (zCoord - 1) : (zCoord + 1));
            final int n3 = world.getBlockId(xCoord + 1, yCoord, (blockId == this.blockID) ? (zCoord - 1) : (zCoord + 1));
            if (blockSide == 4) {
                n = -1 - n;
            }
            xCoord = 5;
            if ((Block.opaqueCubeLookup[blockId3] || Block.opaqueCubeLookup[n2]) && !Block.opaqueCubeLookup[blockId4] && !Block.opaqueCubeLookup[n3]) {
                xCoord = 5;
            }
            if ((Block.opaqueCubeLookup[blockId4] || Block.opaqueCubeLookup[n3]) && !Block.opaqueCubeLookup[blockId3] && !Block.opaqueCubeLookup[n2]) {
                xCoord = 4;
            }
            return ((blockSide == xCoord) ? (this.blockIndexInTexture + 16) : (this.blockIndexInTexture + 32)) + n;
        }
        else if (blockId3 == this.blockID || blockId4 == this.blockID) {
            if (blockSide == 4 || blockSide == 5) {
                return this.blockIndexInTexture;
            }
            int n = 0;
            if (blockId3 == this.blockID) {
                n = -1;
            }
            final int n2 = world.getBlockId((blockId3 == this.blockID) ? (xCoord - 1) : (xCoord + 1), yCoord, zCoord - 1);
            final int n3 = world.getBlockId((blockId3 == this.blockID) ? (xCoord - 1) : (xCoord + 1), yCoord, zCoord + 1);
            if (blockSide == 3) {
                n = -1 - n;
            }
            xCoord = 3;
            if ((Block.opaqueCubeLookup[blockId] || Block.opaqueCubeLookup[n2]) && !Block.opaqueCubeLookup[blockId2] && !Block.opaqueCubeLookup[n3]) {
                xCoord = 3;
            }
            if ((Block.opaqueCubeLookup[blockId2] || Block.opaqueCubeLookup[n3]) && !Block.opaqueCubeLookup[blockId] && !Block.opaqueCubeLookup[n2]) {
                xCoord = 2;
            }
            return ((blockSide == xCoord) ? (this.blockIndexInTexture + 16) : (this.blockIndexInTexture + 32)) + n;
        }
        else {
            int n = 3;
            if (Block.opaqueCubeLookup[blockId] && !Block.opaqueCubeLookup[blockId2]) {
                n = 3;
            }
            if (Block.opaqueCubeLookup[blockId2] && !Block.opaqueCubeLookup[blockId]) {
                n = 2;
            }
            if (Block.opaqueCubeLookup[blockId3] && !Block.opaqueCubeLookup[blockId4]) {
                n = 5;
            }
            if (Block.opaqueCubeLookup[blockId4] && !Block.opaqueCubeLookup[blockId3]) {
                n = 4;
            }
            if (blockSide == n) {
                return this.blockIndexInTexture + 1;
            }
            return this.blockIndexInTexture;
        }
    }
    
    @Override
    public final int getBlockTextureFromSide(final int blockSide) {
        if (blockSide == 1) {
            return this.blockIndexInTexture - 1;
        }
        if (blockSide == 0) {
            return this.blockIndexInTexture - 1;
        }
        if (blockSide == 3) {
            return this.blockIndexInTexture + 1;
        }
        return this.blockIndexInTexture;
    }
    
    @Override
    public final boolean canPlaceBlockAt(final World world, final long xCoord, final long yCoord, final long zCoord) {
        int n = 0;
        if (world.getBlockId(xCoord - 1, yCoord, zCoord) == this.blockID) {
            ++n;
        }
        if (world.getBlockId(xCoord + 1, yCoord, zCoord) == this.blockID) {
            ++n;
        }
        if (world.getBlockId(xCoord, yCoord, zCoord - 1) == this.blockID) {
            ++n;
        }
        if (world.getBlockId(xCoord, yCoord, zCoord + 1) == this.blockID) {
            ++n;
        }
        return n <= 1 && !this.isThereANeighborChest(world, xCoord - 1, yCoord, zCoord) && !this.isThereANeighborChest(world, xCoord + 1, yCoord, zCoord) && !this.isThereANeighborChest(world, xCoord, yCoord, zCoord - 1) && !this.isThereANeighborChest(world, xCoord, yCoord, zCoord + 1);
    }
    
    private boolean isThereANeighborChest(final World world, final long l, final long yCoord, final long zCoord) {
        return world.getBlockId(l, yCoord, zCoord) == this.blockID && (world.getBlockId(l - 1, yCoord, zCoord) == this.blockID || world.getBlockId(l + 1, yCoord, zCoord) == this.blockID || world.getBlockId(l, yCoord, zCoord - 1) == this.blockID || world.getBlockId(l, yCoord, zCoord + 1) == this.blockID);
    }
    
    @Override
    public final void onBlockRemoval(final World world, final long xCoord, final long yCoord, final long zCoord) {
        final TileEntityChest tileEntityChest = (TileEntityChest)world.getBlockTileEntity(xCoord, yCoord, zCoord);
        for (int i = 0; i < tileEntityChest.getSizeInventory(); ++i) {
            final ItemStack stackInSlot;
            if ((stackInSlot = tileEntityChest.getStackInSlot(i)) != null) {
                final double n = this.random.nextDouble() * 0.8f + 0.1f;
                final double n2 = this.random.nextDouble() * 0.8f + 0.1f;
                final double n3 = this.random.nextDouble() * 0.8f + 0.1f;
                while (stackInSlot.stackSize > 0) {
                    int stackSize;
                    if ((stackSize = this.random.nextInt(21) + 10) > stackInSlot.stackSize) {
                        stackSize = stackInSlot.stackSize;
                    }
                    final ItemStack itemStack = stackInSlot;
                    itemStack.stackSize -= stackSize;
                    final EntityItem entity;
                    (entity = new EntityItem(world, xCoord + n, yCoord + n2, zCoord + n3, new ItemStack(stackInSlot.itemID, stackSize, stackInSlot.itemDamage))).motionX = (double)this.random.nextGaussian() * 0.05f;
                    entity.motionY = (double)this.random.nextGaussian() * 0.05f + 0.2f;
                    entity.motionZ = (double)this.random.nextGaussian() * 0.05f;
                    world.entityJoinedWorld(entity);
                }
            }
        }
        super.onBlockRemoval(world, xCoord, yCoord, zCoord);
    }
    
    @Override
    public final boolean blockActivated(final World world, final long xCoord, final long yCoord, final long zCoord, final EntityPlayer player) {
        IInventory iInventory = (TileEntityChest)world.getBlockTileEntity(xCoord, yCoord, zCoord);
        if (world.isBlockNormalCube(xCoord, yCoord + 1, zCoord)) {
            return true;
        }
        if (world.getBlockId(xCoord - 1, yCoord, zCoord) == this.blockID && world.isBlockNormalCube(xCoord - 1, yCoord + 1, zCoord)) {
            return true;
        }
        if (world.getBlockId(xCoord + 1, yCoord, zCoord) == this.blockID && world.isBlockNormalCube(xCoord + 1, yCoord + 1, zCoord)) {
            return true;
        }
        if (world.getBlockId(xCoord, yCoord, zCoord - 1) == this.blockID && world.isBlockNormalCube(xCoord, yCoord + 1, zCoord - 1)) {
            return true;
        }
        if (world.getBlockId(xCoord, yCoord, zCoord + 1) == this.blockID && world.isBlockNormalCube(xCoord, yCoord + 1, zCoord + 1)) {
            return true;
        }
        if (world.getBlockId(xCoord - 1, yCoord, zCoord) == this.blockID) {
            iInventory = new InventoryLargeChest("Large chest", (IInventory)world.getBlockTileEntity(xCoord - 1, yCoord, zCoord), iInventory);
        }
        if (world.getBlockId(xCoord + 1, yCoord, zCoord) == this.blockID) {
            iInventory = new InventoryLargeChest("Large chest", iInventory, (IInventory)world.getBlockTileEntity(xCoord + 1, yCoord, zCoord));
        }
        if (world.getBlockId(xCoord, yCoord, zCoord - 1) == this.blockID) {
            iInventory = new InventoryLargeChest("Large chest", (IInventory)world.getBlockTileEntity(xCoord, yCoord, zCoord - 1), iInventory);
        }
        if (world.getBlockId(xCoord, yCoord, zCoord + 1) == this.blockID) {
            iInventory = new InventoryLargeChest("Large chest", iInventory, (IInventory)world.getBlockTileEntity(xCoord, yCoord, zCoord + 1));
        }
        player.displayGUIChest(iInventory);
        return true;
    }
    
    @Override
    protected final TileEntity getBlockEntity() {
        return new TileEntityChest();
    }
}
