package net.minecraft.game.block;

import net.minecraft.game.entity.Entity;
import net.minecraft.game.entity.other.EntityItem;
import net.minecraft.game.item.ItemStack;
import net.minecraft.game.item.Item;
import java.util.Random;
import net.minecraft.game.level.World;

public final class BlockCrops extends BlockFlower {
    protected BlockCrops(final int blockID, final int textureSlot) {
        super(59, 88);
        this.blockIndexInTexture = 88;
        this.setTickOnLoad(true);
        this.setBlockBounds(0.0f, 0.0f, 0.0f, 1.0f, 0.25f, 1.0f);
    }
    
    @Override
    protected final boolean canThisPlantGrowOnThisBlockID(final int blockID) {
        return blockID == Block.tilledField.blockID;
    }
    
    @Override
    public final void updateTick(final World world, final long xCoord, final long yCoord, final long zCoord, final Random random) {
        super.updateTick(world, xCoord, yCoord, zCoord, random);
        int blockMetadata;
        if (world.getBlockLightValue(xCoord, yCoord + 1, zCoord) >= 9 && (blockMetadata = world.getBlockMetadata(xCoord, yCoord, zCoord)) < 7) {
            double n = 1.0f;
            final int blockId = world.getBlockId(xCoord, yCoord, zCoord - 1);
            long i = world.getBlockId(xCoord, yCoord, zCoord + 1);
            final int blockId2 = world.getBlockId(xCoord - 1, yCoord, zCoord);
            long j = world.getBlockId(xCoord + 1, yCoord, zCoord);
            int n2 = world.getBlockId(xCoord - 1, yCoord, zCoord - 1);
            final int blockId3 = world.getBlockId(xCoord + 1, yCoord, zCoord - 1);
            final int blockId4 = world.getBlockId(xCoord + 1, yCoord, zCoord + 1);
            final int blockId5 = world.getBlockId(xCoord - 1, yCoord, zCoord + 1);
            final boolean b = blockId2 == this.blockID || j == this.blockID;
            final boolean b2 = blockId == this.blockID || i == this.blockID;
            final boolean b3 = n2 == this.blockID || blockId3 == this.blockID || blockId4 == this.blockID || blockId5 == this.blockID;
            for (i = xCoord - 1; i <= xCoord + 1; ++i) {
                for (j = zCoord - 1; j <= zCoord + 1; ++j) {
                    n2 = world.getBlockId(i, yCoord - 1, j);
                    double n3 = 0.0f;
                    if (n2 == Block.tilledField.blockID) {
                        n3 = 1.0f;
                        if (world.getBlockMetadata(i, yCoord - 1, j) > 0) {
                            n3 = 3.0f;
                        }
                    }
                    if (i != xCoord || j != zCoord) {
                        n3 /= 4.0f;
                    }
                    n += n3;
                }
            }
            if (b3 || (b && b2)) {
                n /= 2.0f;
            }
            if (random.nextInt((int)(100.0f / n)) == 0) {
                ++blockMetadata;
                world.setBlockMetadataWithNotify(xCoord, yCoord, zCoord, blockMetadata);
            }
        }
    }
    
    @Override
    public final int getBlockTextureFromSideAndMetadata(final int integer1, int integer2) {
        if (integer2 < 0) {
            integer2 = 7;
        }
        return this.blockIndexInTexture + integer2;
    }
    
    @Override
    public final int getRenderType() {
        return 6;
    }
    
    @Override
    public final void onBlockDestroyedByPlayer(final World world, final long xCoord, final long yCoord, final long zCoord, final int integer5) {
        super.onBlockDestroyedByPlayer(world, xCoord, yCoord, zCoord, integer5);
        for (int i = 0; i < 3; ++i) {
            if (world.random.nextInt(15) <= integer5) {
                final EntityItem entity;
                (entity = new EntityItem(world, xCoord + (world.random.nextDouble() * 0.7f + 0.15f), yCoord + (world.random.nextDouble() * 0.7f + 0.15f), zCoord + (world.random.nextDouble() * 0.7f + 0.15f), new ItemStack(Item.seeds))).delayBeforeCanPickup = 10;
                world.entityJoinedWorld(entity);
            }
        }
    }
    
    @Override
    public final int idDropped(final int integer, final Random random) {
        System.out.println(new StringBuilder().append("Get resource: ").append(integer).toString());
        if (integer == 7) {
            return Item.wheat.shiftedIndex;
        }
        return -1;
    }
    
    @Override
    public final int quantityDropped(final Random random) {
        return 1;
    }
}
