package net.minecraft.game.block;

import net.minecraft.client.physics.AxisAlignedBB;
import java.util.Random;
import net.minecraft.game.level.World;

public class BlockFlower extends Block {
    protected BlockFlower(final int blockID, final int textureSlot) {
        super(blockID, Material.plants);
        this.blockIndexInTexture = textureSlot;
        this.setTickOnLoad(true);
        this.setBlockBounds(0.3f, 0.0f, 0.3f, 0.7f, 0.6f, 0.7f);
    }
    
    @Override
    public final boolean canPlaceBlockAt(final World world, final long xCoord, final long yCoord, final long zCoord) {
        return this.canThisPlantGrowOnThisBlockID(world.getBlockId(xCoord, yCoord - 1, zCoord));
    }
    
    protected boolean canThisPlantGrowOnThisBlockID(final int blockID) {
        return blockID == Block.grass.blockID || blockID == Block.dirt.blockID || blockID == Block.tilledField.blockID;
    }
    
    @Override
    public final void onNeighborBlockChange(final World world, final long xCoord, final long yCoord, final long zCoord, final int blockID) {
        super.onNeighborBlockChange(world, xCoord, yCoord, zCoord, blockID);
        this.checkFlowerChange(world, xCoord, yCoord, zCoord);
    }
    
    @Override
    public void updateTick(final World world, final long xCoord, final long yCoord, final long zCoord, final Random random) {
        this.checkFlowerChange(world, xCoord, yCoord, zCoord);
    }
    
    private void checkFlowerChange(final World world, final long xCoord, final long yCoord, final long zCoord) {
        if (!this.canBlockStay(world, xCoord, yCoord, zCoord)) {
            this.dropBlockAsItem(world, xCoord, yCoord, zCoord, world.getBlockMetadata(xCoord, yCoord, zCoord));
            world.setBlockWithNotify(xCoord, yCoord, zCoord, 0);
        }
    }
    
    public boolean canBlockStay(final World world, final long xCoord, final long yCoord, final long zCoord) {
        return (world.getBlockLightValue(xCoord, yCoord, zCoord) >= 8 || (world.getBlockLightValue(xCoord, yCoord, zCoord) >= 4 && world.canBlockSeeTheSky(xCoord, yCoord, zCoord))) && this.canThisPlantGrowOnThisBlockID(world.getBlockId(xCoord, yCoord - 1, zCoord));
    }
    
    @Override
    public final AxisAlignedBB getCollisionBoundingBoxFromPool(final long xCoord, final long yCoord, final long zCoord) {
        return null;
    }
    
    @Override
    public final boolean isOpaqueCube() {
        return false;
    }
    
    @Override
    public final boolean renderAsNormalBlock() {
        return false;
    }
    
    @Override
    public int getRenderType() {
        return 1;
    }
}
