package net.minecraft.game.block;

import net.minecraft.client.physics.AxisAlignedBB;
import java.util.Random;
import net.minecraft.game.level.World;

public class BlockFluid extends Block {
    protected int stillId;
    protected int movingId;
    
    protected BlockFluid(final int blockID, final Material material) {
        super(blockID, material);
        this.blockIndexInTexture = 14;
        if (material == Material.lava) {
            this.blockIndexInTexture = 30;
        }
        Block.isBlockContainer[blockID] = true;
        this.movingId = blockID;
        this.stillId = blockID + 1;
        this.setBlockBounds(0.01f, -0.09f, 0.01f, 1.01f, 0.90999997f, 1.01f);
        this.setTickOnLoad(true);
        this.setResistance(2.0f);
    }
    
    @Override
    public final int getBlockTextureFromSide(final int blockSide) {
        if (this.blockMaterial == Material.lava) {
            return this.blockIndexInTexture;
        }
        if (blockSide == 1) {
            return this.blockIndexInTexture;
        }
        if (blockSide == 0) {
            return this.blockIndexInTexture;
        }
        return this.blockIndexInTexture + 32;
    }
    
    @Override
    public final boolean renderAsNormalBlock() {
        return false;
    }
    
    @Override
    public void onBlockAdded(final World world, final long xCoord, final long yCoord, final long zCoord) {
        world.scheduleBlockUpdate(xCoord, yCoord, zCoord, this.movingId);
    }
    
    @Override
    public void updateTick(final World world, final long xCoord, final long yCoord, final long zCoord, final Random random) {
        this.update(world, xCoord, yCoord, zCoord, 0);
    }
    
    public boolean update(final World world, final long xCoord, long yCoord, final long zCoord, int integer5) {
        integer5 = 0;
        while (this.canFlow(world, xCoord, --yCoord, zCoord)) {
            final boolean setBlockWithNotify;
            if (setBlockWithNotify = world.setBlockWithNotify(xCoord, yCoord, zCoord, this.movingId)) {
                integer5 = 1;
            }
            if (setBlockWithNotify && this.blockMaterial != Material.lava) {
                continue;
            }
            break;
        }
        ++yCoord;
        if (this.blockMaterial == Material.water || integer5 == 0) {
            integer5 = ((integer5 = ((integer5 = ((integer5 |= (this.flow(world, xCoord - 1, yCoord, zCoord) ? 1 : 0)) | (this.flow(world, xCoord + 1, yCoord, zCoord) ? 1 : 0))) | (this.flow(world, xCoord, yCoord, zCoord - 1) ? 1 : 0))) | (this.flow(world, xCoord, yCoord, zCoord + 1) ? 1 : 0));
        }
        if (this.blockMaterial == Material.lava) {
            integer5 = ((integer5 = ((integer5 = ((integer5 |= (extinguishFireLava(world, xCoord - 1, yCoord, zCoord) ? 1 : 0)) | (extinguishFireLava(world, xCoord + 1, yCoord, zCoord) ? 1 : 0))) | (extinguishFireLava(world, xCoord, yCoord, zCoord - 1) ? 1 : 0))) | (extinguishFireLava(world, xCoord, yCoord, zCoord + 1) ? 1 : 0));
        }
        if (integer5 == 0) {
            world.setTileNoUpdate(xCoord, yCoord, zCoord, this.stillId);
        }
        else {
            world.scheduleBlockUpdate(xCoord, yCoord, zCoord, this.movingId);
        }
        return integer5 != 0;
    }
    
    protected final boolean canFlow(final World g, final long xCoord, final long l, final long zCoord) {
        if (!g.getBlockMaterial(xCoord, l, zCoord).liquidSolidCheck()) {
            return false;
        }
        if (this.blockMaterial == Material.water) {
            for (long i = xCoord - 2; i <= xCoord + 2; ++i) {
                for (long j = l - 2; j <= l + 2; ++j) {
                    for (long k = zCoord - 2; k <= zCoord + 2; ++k) {
                        if (g.getBlockId(i, j, k) == Block.sponge.blockID) {
                            return false;
                        }
                    }
                }
            }
        }
        return true;
    }
    
    private static boolean extinguishFireLava(final World world, final long integer2, final long integer3, final long integer4) {
        if (Block.fire.getChanceOfNeighborsEncouragingFire(world.getBlockId(integer2, integer3, integer4))) {
            Block.fire.fireSpread(world, integer2, integer3, integer4);
            return true;
        }
        return false;
    }
    
    private boolean flow(final World world, final long integer2, final long integer3, final long integer4) {
        if (!this.canFlow(world, integer2, integer3, integer4)) {
            return false;
        }
        if (world.setBlockWithNotify(integer2, integer3, integer4, this.movingId)) {
            world.scheduleBlockUpdate(integer2, integer3, integer4, this.movingId);
        }
        return false;
    }
    
    @Override
    public final double getBlockBrightness(final World world, final long integer2, final long integer3, final long integer4) {
        if (this.blockMaterial == Material.lava) {
            return 100.0f;
        }
        return world.getLightBrightness(integer2, integer3, integer4);
    }
    
    @Override
    public boolean shouldSideBeRendered(final World world, final long xCoord, final long yCoord, final long zCoord, final int integer5) {
        final int blockId;
        return (blockId = world.getBlockId(xCoord, yCoord, zCoord)) != this.movingId && blockId != this.stillId && ((integer5 == 1 && (world.getBlockId(xCoord - 1, yCoord, zCoord) == 0 || world.getBlockId(xCoord + 1, yCoord, zCoord) == 0 || world.getBlockId(xCoord, yCoord, zCoord - 1) == 0 || world.getBlockId(xCoord, yCoord, zCoord + 1) == 0)) || super.shouldSideBeRendered(world, xCoord, yCoord, zCoord, integer5));
    }
    
    @Override
    public boolean isCollidable() {
        return false;
    }
    
    @Override
    public AxisAlignedBB getCollisionBoundingBoxFromPool(final long xCoord, final long yCoord, final long zCoord) {
        return null;
    }
    
    @Override
    public boolean isOpaqueCube() {
        return false;
    }
    
    @Override
    public void onNeighborBlockChange(final World world, final long xCoord, final long yCoord, final long zCoord, final int blockID) {
        if (blockID != 0) {
            final Material blockMaterial = Block.blocksList[blockID].blockMaterial;
            if ((this.blockMaterial == Material.water && blockMaterial == Material.lava) || (blockMaterial == Material.water && this.blockMaterial == Material.lava)) {
                world.setBlockWithNotify(xCoord, yCoord, zCoord, Block.stone.blockID);
            }
        }
        world.scheduleBlockUpdate(xCoord, yCoord, zCoord, this.blockID);
    }
    
    @Override
    public int tickRate() {
        if (this.blockMaterial == Material.lava) {
            return 25;
        }
        return 5;
    }
    
    @Override
    public int quantityDropped(final Random random) {
        return 0;
    }
    
    @Override
    public int getRenderBlockPass() {
        if (this.blockMaterial == Material.water) {
            return 1;
        }
        return 0;
    }
    
    @Override
    public final void randomDisplayTick(final World world, final long xCoord, final long yCoord, final long zCoord, final Random random) {
        if (random.nextInt(128) == -1 && world.getBlockMaterial(xCoord, yCoord + 1, zCoord).getIsSolid()) {
            if (this.blockMaterial == Material.lava) {
                world.playSoundAtPlayer(xCoord + 0.5f, yCoord + 0.5f, zCoord + 0.5f, "liquid.lava", random.nextDouble() * 0.25f + 0.75f, random.nextDouble() * 0.5f + 0.3f);
            }
            if (this.blockMaterial == Material.water) {
                world.playSoundAtPlayer(xCoord + 0.5f, yCoord + 0.5f, zCoord + 0.5f, "liquid.water", random.nextDouble() * 0.25f + 0.75f, random.nextDouble() + 0.5f);
            }
        }
        if (this.blockMaterial == Material.lava && world.getBlockMaterial(xCoord, yCoord + 1, zCoord) == Material.air && !world.isBlockNormalCube(xCoord, yCoord + 1, zCoord) && random.nextInt(100) == 0) {
            world.spawnParticle("lava", xCoord + random.nextDouble(), yCoord + this.maxY, zCoord + random.nextDouble(), 0.0f, 0.0f, 0.0f);
        }
        if (this.blockMaterial == Material.water) {
            if (liquidAirCheck(world, xCoord + 1, yCoord, zCoord)) {
                for (int i = 0; i < 4; ++i) {
                    world.spawnParticle("splash", xCoord + 1 + 0.125f, (double)yCoord, zCoord + random.nextDouble(), 0.0f, 0.0f, 0.0f);
                }
            }
            if (liquidAirCheck(world, xCoord - 1, yCoord, zCoord)) {
                for (int i = 0; i < 4; ++i) {
                    world.spawnParticle("splash", xCoord - 0.125f, (double)yCoord, zCoord + random.nextDouble(), 0.0f, 0.0f, 0.0f);
                }
            }
            if (liquidAirCheck(world, xCoord, yCoord, zCoord + 1)) {
                for (int i = 0; i < 4; ++i) {
                    world.spawnParticle("splash", xCoord + random.nextDouble(), (double)yCoord, zCoord + 1 + 0.125f, 0.0f, 0.0f, 0.0f);
                }
            }
            if (liquidAirCheck(world, xCoord, yCoord, zCoord - 1)) {
                for (int i = 0; i < 4; ++i) {
                    world.spawnParticle("splash", xCoord + random.nextDouble(), (double)yCoord, zCoord - 0.125f, 0.0f, 0.0f, 0.0f);
                }
            }
        }
    }
    
    private static boolean liquidAirCheck(final World world, final long l, final long yCoord, final long zCoord) {
        final Material blockMaterial = world.getBlockMaterial(l, yCoord, zCoord);
        final Material blockMaterial2 = world.getBlockMaterial(l, yCoord - 1, zCoord);
        return !blockMaterial.getIsSolid() && !blockMaterial.getIsLiquid() && (blockMaterial2.getIsSolid() || blockMaterial2.getIsLiquid());
    }
}
