package net.minecraft.game.block;

import java.util.Random;
import net.minecraft.game.level.World;

public final class BlockFluidSource extends Block {
    private int fluid;
    
    protected BlockFluidSource(final int blockID, final int fluidType) {
        super(blockID, Block.blocksList[fluidType].blockIndexInTexture, Material.water);
        this.fluid = fluidType;
        this.setTickOnLoad(true);
    }
    
    @Override
    public final void onBlockAdded(final World world, final long xCoord, final long yCoord, final long zCoord) {
        super.onBlockAdded(world, xCoord, yCoord, zCoord);
        if (world.getBlockId(xCoord - 1, yCoord, zCoord) == 0) {
            world.setBlockWithNotify(xCoord - 1, yCoord, zCoord, this.fluid);
        }
        if (world.getBlockId(xCoord + 1, yCoord, zCoord) == 0) {
            world.setBlockWithNotify(xCoord + 1, yCoord, zCoord, this.fluid);
        }
        if (world.getBlockId(xCoord, yCoord, zCoord - 1) == 0) {
            world.setBlockWithNotify(xCoord, yCoord, zCoord - 1, this.fluid);
        }
        if (world.getBlockId(xCoord, yCoord, zCoord + 1) == 0) {
            world.setBlockWithNotify(xCoord, yCoord, zCoord + 1, this.fluid);
        }
    }
    
    @Override
    public final void updateTick(final World world, final long xCoord, final long yCoord, final long zCoord, final Random random) {
        super.updateTick(world, xCoord, yCoord, zCoord, random);
        if (world.getBlockId(xCoord - 1, yCoord, zCoord) == 0) {
            world.setBlockWithNotify(xCoord - 1, yCoord, zCoord, this.fluid);
        }
        if (world.getBlockId(xCoord + 1, yCoord, zCoord) == 0) {
            world.setBlockWithNotify(xCoord + 1, yCoord, zCoord, this.fluid);
        }
        if (world.getBlockId(xCoord, yCoord, zCoord - 1) == 0) {
            world.setBlockWithNotify(xCoord, yCoord, zCoord - 1, this.fluid);
        }
        if (world.getBlockId(xCoord, yCoord, zCoord + 1) == 0) {
            world.setBlockWithNotify(xCoord, yCoord, zCoord + 1, this.fluid);
        }
    }
}
