package net.minecraft.game.block;

import net.minecraft.client.physics.MovingObjectPosition;
import net.minecraft.client.renderer.Vec3D;
import java.util.Random;
import net.minecraft.game.level.World;
import net.minecraft.client.physics.AxisAlignedBB;

public final class BlockTorch extends Block {
    protected BlockTorch(final int blockID, final int textureSlot) {
        super(50, 80, Material.circuits);
        this.setTickOnLoad(true);
    }
    
    @Override
    public final AxisAlignedBB getCollisionBoundingBoxFromPool(final long xCoord, final long yCoord, final long zCoord) {
        return null;
    }
    
    @Override
    public final boolean isOpaqueCube() {
        return false;
    }
    
    @Override
    public final boolean renderAsNormalBlock() {
        return false;
    }
    
    @Override
    public final int getRenderType() {
        return 2;
    }
    
    @Override
    public final boolean canPlaceBlockAt(final World world, final long xCoord, final long yCoord, final long zCoord) {
        return world.isBlockNormalCube(xCoord - 1, yCoord, zCoord) || world.isBlockNormalCube(xCoord + 1, yCoord, zCoord) || world.isBlockNormalCube(xCoord, yCoord, zCoord - 1) || world.isBlockNormalCube(xCoord, yCoord, zCoord + 1) || world.isBlockNormalCube(xCoord, yCoord - 1, zCoord);
    }
    
    @Override
    public final void onBlockPlaced(final World world, final long integer2, final long integer3, final long integer4, final int integer5) {
        int blockMetadata = world.getBlockMetadata(integer2, integer3, integer4);
        if (integer5 == 1 && world.isBlockNormalCube(integer2, integer3 - 1, integer4)) {
            blockMetadata = 5;
        }
        if (integer5 == 2 && world.isBlockNormalCube(integer2, integer3, integer4 + 1)) {
            blockMetadata = 4;
        }
        if (integer5 == 3 && world.isBlockNormalCube(integer2, integer3, integer4 - 1)) {
            blockMetadata = 3;
        }
        if (integer5 == 4 && world.isBlockNormalCube(integer2 + 1, integer3, integer4)) {
            blockMetadata = 2;
        }
        if (integer5 == 5 && world.isBlockNormalCube(integer2 - 1, integer3, integer4)) {
            blockMetadata = 1;
        }
        world.setBlockMetadataWithNotify(integer2, integer3, integer4, blockMetadata);
    }
    
    @Override
    public final void updateTick(final World world, final long xCoord, final long yCoord, final long zCoord, final Random random) {
        super.updateTick(world, xCoord, yCoord, zCoord, random);
        if (world.getBlockMetadata(xCoord, yCoord, zCoord) == 0) {
            this.onBlockAdded(world, xCoord, yCoord, zCoord);
        }
    }
    
    @Override
    public final void onBlockAdded(final World world, final long xCoord, final long yCoord, final long zCoord) {
        if (world.isBlockNormalCube(xCoord - 1, yCoord, zCoord)) {
            world.setBlockMetadataWithNotify(xCoord, yCoord, zCoord, 1);
        }
        else if (world.isBlockNormalCube(xCoord + 1, yCoord, zCoord)) {
            world.setBlockMetadataWithNotify(xCoord, yCoord, zCoord, 2);
        }
        else if (world.isBlockNormalCube(xCoord, yCoord, zCoord - 1)) {
            world.setBlockMetadataWithNotify(xCoord, yCoord, zCoord, 3);
        }
        else if (world.isBlockNormalCube(xCoord, yCoord, zCoord + 1)) {
            world.setBlockMetadataWithNotify(xCoord, yCoord, zCoord, 4);
        }
        else if (world.isBlockNormalCube(xCoord, yCoord - 1, zCoord)) {
            world.setBlockMetadataWithNotify(xCoord, yCoord, zCoord, 5);
        }
        this.dropTorchIfCantStay(world, xCoord, yCoord, zCoord);
    }
    
    @Override
    public final void onNeighborBlockChange(final World world, long xCoord, long yCoord, long zCoord, int blockID) {
        if (this.dropTorchIfCantStay(world, xCoord, yCoord, zCoord)) {
            blockID = world.getBlockMetadata(xCoord, yCoord, zCoord);
            boolean b = false;
            if (!world.isBlockNormalCube(xCoord - 1, yCoord, zCoord) && blockID == 1) {
                b = true;
            }
            if (!world.isBlockNormalCube(xCoord + 1, yCoord, zCoord) && blockID == 2) {
                b = true;
            }
            if (!world.isBlockNormalCube(xCoord, yCoord, zCoord - 1) && blockID == 3) {
                b = true;
            }
            if (!world.isBlockNormalCube(xCoord, yCoord, zCoord + 1) && blockID == 4) {
                b = true;
            }
            if (!world.isBlockNormalCube(xCoord, yCoord - 1, zCoord) && blockID == 5) {
                b = true;
            }
            if (b) {
                this.dropBlockAsItem(world, xCoord, yCoord, zCoord, world.getBlockMetadata(xCoord, yCoord, zCoord));
                world.setBlockWithNotify(xCoord, yCoord, zCoord, 0);
            }
        }
    }
    
    private boolean dropTorchIfCantStay(final World g, final long xCoord, final long yCoord, final long zCoord) {
        if (!this.canPlaceBlockAt(g, xCoord, yCoord, zCoord)) {
            this.dropBlockAsItem(g, xCoord, yCoord, zCoord, g.getBlockMetadata(xCoord, yCoord, zCoord));
            g.setBlockWithNotify(xCoord, yCoord, zCoord, 0);
            return false;
        }
        return true;
    }
    
    @Override
    public final MovingObjectPosition collisionRayTrace(final World world, final long integer2, final long integer3, final long integer4, final Vec3D vec3D, final Vec3D vec3D2) {
        final byte blockMetadata;
        if ((blockMetadata = world.getBlockMetadata(integer2, integer3, integer4)) == 1) {
            this.setBlockBounds(0.0f, 0.2f, 0.35f, 0.3f, 0.8f, 0.65f);
        }
        else if (blockMetadata == 2) {
            this.setBlockBounds(0.7f, 0.2f, 0.35f, 1.0f, 0.8f, 0.65f);
        }
        else if (blockMetadata == 3) {
            this.setBlockBounds(0.35f, 0.2f, 0.0f, 0.65f, 0.8f, 0.3f);
        }
        else if (blockMetadata == 4) {
            this.setBlockBounds(0.35f, 0.2f, 0.7f, 0.65f, 0.8f, 1.0f);
        }
        else {
            this.setBlockBounds(0.4f, 0.0f, 0.4f, 0.6f, 0.6f, 0.6f);
        }
        return super.collisionRayTrace(world, integer2, integer3, integer4, vec3D, vec3D2);
    }
    
    @Override
    public final void randomDisplayTick(final World world, final long xCoord, final long yCoord, final long zCoord, final Random random) {
        final byte blockMetadata = world.getBlockMetadata(xCoord, yCoord, zCoord);
        final double n = xCoord + 0.5f;
        final double n2 = yCoord + 0.7f;
        final double n3 = zCoord + 0.5f;
        if (blockMetadata == 1) {
            world.spawnParticle("smoke", n - 0.27f, n2 + 0.22f, n3, 0.0f, 0.0f, 0.0f);
            world.spawnParticle("flame", n - 0.27f, n2 + 0.22f, n3, 0.0f, 0.0f, 0.0f);
            return;
        }
        if (blockMetadata == 2) {
            world.spawnParticle("smoke", n + 0.27f, n2 + 0.22f, n3, 0.0f, 0.0f, 0.0f);
            world.spawnParticle("flame", n + 0.27f, n2 + 0.22f, n3, 0.0f, 0.0f, 0.0f);
            return;
        }
        if (blockMetadata == 3) {
            world.spawnParticle("smoke", n, n2 + 0.22f, n3 - 0.27f, 0.0f, 0.0f, 0.0f);
            world.spawnParticle("flame", n, n2 + 0.22f, n3 - 0.27f, 0.0f, 0.0f, 0.0f);
            return;
        }
        if (blockMetadata == 4) {
            world.spawnParticle("smoke", n, n2 + 0.22f, n3 + 0.27f, 0.0f, 0.0f, 0.0f);
            world.spawnParticle("flame", n, n2 + 0.22f, n3 + 0.27f, 0.0f, 0.0f, 0.0f);
            return;
        }
        world.spawnParticle("smoke", n, n2, n3, 0.0f, 0.0f, 0.0f);
        world.spawnParticle("flame", n, n2, n3, 0.0f, 0.0f, 0.0f);
    }
}
