package net.minecraft.game.block.tileentity;

import com.nbt.NBTBase;
import com.nbt.NBTTagList;
import com.nbt.NBTTagCompound;
import net.minecraft.game.item.ItemStack;
import net.minecraft.client.inventory.IInventory;

public final class TileEntityChest extends TileEntity implements IInventory {
    private ItemStack[] chestContents;
    
    public TileEntityChest() {
        this.chestContents = new ItemStack[36];
    }
    
    public final int getSizeInventory() {
        return 27;
    }
    
    public final ItemStack getStackInSlot(final int integer) {
        return this.chestContents[integer];
    }
    
    public final ItemStack decrStackSize(final int integer1, final int integer2) {
        if (this.chestContents[integer1] == null) {
            return null;
        }
        if (this.chestContents[integer1].stackSize <= integer2) {
            final ItemStack itemStack = this.chestContents[integer1];
            this.chestContents[integer1] = null;
            return itemStack;
        }
        final ItemStack splitStack = this.chestContents[integer1].splitStack(integer2);
        if (this.chestContents[integer1].stackSize == 0) {
            this.chestContents[integer1] = null;
        }
        return splitStack;
    }
    
    public final void setInventorySlotContents(final int integer, final ItemStack itemStack) {
        this.chestContents[integer] = itemStack;
        if (itemStack != null && itemStack.stackSize > 64) {
            itemStack.stackSize = 64;
        }
    }
    
    public final String getInvName() {
        return "Chest";
    }
    
    @Override
    public final void readFromNBT(final NBTTagCompound nbtTagCompound) {
        final NBTTagList tagList = nbtTagCompound.getTagList("Items");
        this.chestContents = new ItemStack[27];
        for (int i = 0; i < tagList.tagCount(); ++i) {
            final NBTTagCompound nbtTagCompound2;
            final int n;
            if ((n = ((nbtTagCompound2 = (NBTTagCompound)tagList.tagAt(i)).getByte("Slot") & 0xFF)) >= 0 && n < this.chestContents.length) {
                this.chestContents[n] = new ItemStack(nbtTagCompound2);
            }
        }
    }
    
    @Override
    public final void writeToNBT(final NBTTagCompound nbtTagCompound) {
        nbtTagCompound.setString("id", "Chest");
        final NBTTagList j = new NBTTagList();
        for (int i = 0; i < this.chestContents.length; ++i) {
            if (this.chestContents[i] != null) {
                final NBTTagCompound nbtTagCompound2;
                (nbtTagCompound2 = new NBTTagCompound()).setByte("Slot", (byte)i);
                this.chestContents[i].writeToNBT(nbtTagCompound2);
                j.setTag(nbtTagCompound2);
            }
        }
        nbtTagCompound.setTag("Items", (NBTBase)j);
    }
    
    public final int getInventoryStackLimit() {
        return 64;
    }
}
