package net.minecraft.game.block.tileentity;

import net.minecraft.game.block.Material;
import net.minecraft.game.item.Item;
import net.minecraft.game.level.World;
import net.minecraft.game.block.Block;
import com.nbt.NBTBase;
import com.nbt.NBTTagList;
import com.nbt.NBTTagCompound;
import net.minecraft.game.item.ItemStack;
import net.minecraft.client.inventory.IInventory;

public final class TileEntityFurnace extends TileEntity implements IInventory {
    private ItemStack[] furnaceItemStacks;
    private int furnaceBurnTime;
    private int currentItemBurnTime;
    private int furnaceCookTime;
    
    public TileEntityFurnace() {
        this.furnaceItemStacks = new ItemStack[3];
        this.furnaceBurnTime = 0;
        this.currentItemBurnTime = 0;
        this.furnaceCookTime = 0;
    }
    
    public final int getSizeInventory() {
        return this.furnaceItemStacks.length;
    }
    
    public final ItemStack getStackInSlot(final int integer) {
        return this.furnaceItemStacks[integer];
    }
    
    public final ItemStack decrStackSize(final int integer1, final int integer2) {
        if (this.furnaceItemStacks[integer1] == null) {
            return null;
        }
        if (this.furnaceItemStacks[integer1].stackSize <= integer2) {
            final ItemStack itemStack = this.furnaceItemStacks[integer1];
            this.furnaceItemStacks[integer1] = null;
            return itemStack;
        }
        final ItemStack splitStack = this.furnaceItemStacks[integer1].splitStack(integer2);
        if (this.furnaceItemStacks[integer1].stackSize == 0) {
            this.furnaceItemStacks[integer1] = null;
        }
        return splitStack;
    }
    
    public final void setInventorySlotContents(final int integer, final ItemStack itemStack) {
        this.furnaceItemStacks[integer] = itemStack;
        if (itemStack != null && itemStack.stackSize > 64) {
            itemStack.stackSize = 64;
        }
    }
    
    public final String getInvName() {
        return "Chest";
    }
    
    @Override
    public final void readFromNBT(final NBTTagCompound nbtTagCompound) {
        super.readFromNBT(nbtTagCompound);
        final NBTTagList tagList = nbtTagCompound.getTagList("Items");
        this.furnaceItemStacks = new ItemStack[this.furnaceItemStacks.length];
        for (int i = 0; i < tagList.tagCount(); ++i) {
            final NBTTagCompound nbtTagCompound2;
            final byte byte1;
            if ((byte1 = (nbtTagCompound2 = (NBTTagCompound)tagList.tagAt(i)).getByte("Slot")) >= 0 && byte1 < this.furnaceItemStacks.length) {
                this.furnaceItemStacks[byte1] = new ItemStack(nbtTagCompound2);
            }
        }
        this.furnaceBurnTime = nbtTagCompound.getShort("BurnTime");
        this.furnaceCookTime = nbtTagCompound.getShort("CookTime");
        this.currentItemBurnTime = getItemBurnTime(this.furnaceItemStacks[1]);
        System.out.println(new StringBuilder().append("Lit: ").append(this.furnaceBurnTime).append("/").append(this.currentItemBurnTime).toString());
    }
    
    @Override
    public final void writeToNBT(final NBTTagCompound nbtTagCompound) {
        super.writeToNBT(nbtTagCompound);
        nbtTagCompound.setShort("BurnTime", (short)this.furnaceBurnTime);
        nbtTagCompound.setShort("CookTime", (short)this.furnaceCookTime);
        nbtTagCompound.setString("id", "Furnace");
        final NBTTagList j = new NBTTagList();
        for (int i = 0; i < this.furnaceItemStacks.length; ++i) {
            if (this.furnaceItemStacks[i] != null) {
                final NBTTagCompound nbtTagCompound2;
                (nbtTagCompound2 = new NBTTagCompound()).setByte("Slot", (byte)i);
                this.furnaceItemStacks[i].writeToNBT(nbtTagCompound2);
                j.setTag(nbtTagCompound2);
            }
        }
        nbtTagCompound.setTag("Items", (NBTBase)j);
    }
    
    public final int getInventoryStackLimit() {
        return 64;
    }
    
    public final int getCookProgressScaled(final int integer) {
        return this.furnaceCookTime * 24 / 200;
    }
    
    public final int getBurnTimeRemainingScaled(final int integer) {
        return this.furnaceBurnTime * 12 / this.currentItemBurnTime;
    }
    
    public final boolean isBurning() {
        return this.furnaceBurnTime > 0;
    }
    
    @Override
    public final void updateEntity() {
        final boolean b = this.furnaceBurnTime > 0;
        if (this.furnaceBurnTime > 0) {
            --this.furnaceBurnTime;
        }
        if (this.furnaceBurnTime == 0 && this.canSmelt()) {
            final int itemBurnTime = getItemBurnTime(this.furnaceItemStacks[1]);
            this.furnaceBurnTime = itemBurnTime;
            this.currentItemBurnTime = itemBurnTime;
            if (this.furnaceBurnTime > 0 && this.furnaceItemStacks[1] != null) {
                final ItemStack itemStack = this.furnaceItemStacks[1];
                --itemStack.stackSize;
                if (this.furnaceItemStacks[1].stackSize == 0) {
                    this.furnaceItemStacks[1] = null;
                }
            }
        }
        if (this.isBurning() && this.canSmelt()) {
            ++this.furnaceCookTime;
            if (this.furnaceCookTime == 200) {
                this.furnaceCookTime = 0;
                if (this.canSmelt()) {
                    final int smeltItem = smeltItem(this.furnaceItemStacks[0].getItem().shiftedIndex);
                    if (this.furnaceItemStacks[2] == null) {
                        this.furnaceItemStacks[2] = new ItemStack(smeltItem, 1);
                    }
                    else if (this.furnaceItemStacks[2].itemID == smeltItem) {
                        final ItemStack itemStack2 = this.furnaceItemStacks[2];
                        ++itemStack2.stackSize;
                    }
                    final ItemStack itemStack3 = this.furnaceItemStacks[0];
                    --itemStack3.stackSize;
                    if (this.furnaceItemStacks[0].stackSize <= 0) {
                        this.furnaceItemStacks[0] = null;
                    }
                }
            }
        }
        else {
            this.furnaceCookTime = 0;
        }
        if (b != this.furnaceBurnTime > 0) {
            final boolean b2 = this.furnaceBurnTime > 0;
            final World worldObj = this.worldObj;
            final long xCoord = this.xCoord;
            final long yCoord = this.yCoord;
            final long zCoord = this.zCoord;
            final long n = yCoord;
            final long n2 = xCoord;
            final World world = worldObj;
            final boolean b3 = b2;
            final byte blockMetadata = world.getBlockMetadata(n2, n, zCoord);
            final TileEntity blockTileEntity = world.getBlockTileEntity(n2, n, zCoord);
            if (b3) {
                world.setBlockWithNotify(n2, n, zCoord, Block.stoneOvenActive.blockID);
            }
            else {
                world.setBlockWithNotify(n2, n, zCoord, Block.stoneOvenIdle.blockID);
            }
            world.setBlockMetadataWithNotify(n2, n, zCoord, blockMetadata);
            world.setBlockTileEntity(n2, n, zCoord, blockTileEntity);
        }
    }
    
    private boolean canSmelt() {
        final int smeltItem;
        return this.furnaceItemStacks[0] != null && (smeltItem = smeltItem(this.furnaceItemStacks[0].getItem().shiftedIndex)) >= 0 && (this.furnaceItemStacks[2] == null || (this.furnaceItemStacks[2].itemID == smeltItem && (this.furnaceItemStacks[2].stackSize < 64 || this.furnaceItemStacks[2].stackSize < Item.itemsList[smeltItem].getItemStackLimit())));
    }
    
    private static int smeltItem(final int integer) {
        if (integer == Block.oreIron.blockID) {
            return Item.ingotIron.shiftedIndex;
        }
        if (integer == Block.oreGold.blockID) {
            return Item.ingotGold.shiftedIndex;
        }
        if (integer == Block.oreDiamond.blockID) {
            return Item.diamond.shiftedIndex;
        }
        if (integer == Block.sand.blockID) {
            return Block.glass.blockID;
        }
        if (integer == Item.porkRaw.shiftedIndex) {
            return Item.porkCooked.shiftedIndex;
        }
        if (integer == Block.cobblestone.blockID) {
            return Block.stone.blockID;
        }
        return -1;
    }
    
    private static int getItemBurnTime(final ItemStack itemStack) {
        if (itemStack == null) {
            return 0;
        }
        final int shiftedIndex;
        if ((shiftedIndex = itemStack.getItem().shiftedIndex) < 256 && Block.blocksList[shiftedIndex].blockMaterial == Material.wood) {
            return 300;
        }
        if (shiftedIndex == Item.stick.shiftedIndex) {
            return 100;
        }
        if (shiftedIndex == Item.coal.shiftedIndex) {
            return 1600;
        }
        return 0;
    }
}
