package net.minecraft.game.entity;

import net.minecraft.client.renderer.Vec3D;
import net.minecraft.game.MathHelper;
import net.minecraft.game.level.World;

public class EntityCreature extends EntityLiving {
    private PathEntity pathToEntity;
    protected Entity playerToAttack;
    protected boolean hasAttacked;
    
    public EntityCreature(final World world) {
        super(world);
        this.hasAttacked = false;
    }
    
    @Override
    protected void updatePlayerActionState() {
        this.hasAttacked = false;
        if (this.playerToAttack == null) {
            this.playerToAttack = this.findPlayerToAttack();
            if (this.playerToAttack != null) {
                this.pathToEntity = this.worldObj.pathFinder.createEntityPathTo(this, this.playerToAttack, 16.0f);
            }
        }
        else if (!this.playerToAttack.isEntityAlive()) {
            this.playerToAttack = null;
        }
        else {
            final Entity playerToAttack;
            final double n = (playerToAttack = this.playerToAttack).posX - super.posX;
            final double posZ = playerToAttack.posY - super.posY;
            final double n2 = playerToAttack.posZ - super.posZ;
            final double sqrt_double = MathHelper.sqrt_double(n * n + posZ * posZ + n2 * n2);
            if (this.worldObj.rayTraceBlocks(new Vec3D(this.posX, this.posY + this.getEyeHeight(), this.posZ), new Vec3D(this.playerToAttack.posX, this.playerToAttack.posY + this.playerToAttack.getEyeHeight(), this.playerToAttack.posZ)) == null) {
                this.attackEntity(this.playerToAttack, sqrt_double);
            }
        }
        if (this.hasAttacked) {
            this.moveStrafing = 0.0f;
            this.moveForward = 0.0f;
            this.isJumping = false;
            return;
        }
        if (this.playerToAttack != null && (this.pathToEntity == null || this.rand.nextInt(20) == 0)) {
            this.pathToEntity = this.worldObj.pathFinder.createEntityPathTo(this, this.playerToAttack, 16.0f);
        }
        else if (this.pathToEntity == null || this.rand.nextInt(100) == 0) {
            int integer2 = -1;
            int integer3 = -1;
            int integer4 = -1;
            double n3 = -99999.0f;
            for (int i = 0; i < 200; ++i) {
                final int integer5 = (int)(this.posX + this.rand.nextInt(21) - 10.0f);
                final int integer6 = (int)(this.posY + this.rand.nextInt(9) - 4.0f);
                final int integer7 = (int)(this.posZ + this.rand.nextInt(21) - 10.0f);
                final double blockPathWeight;
                if ((blockPathWeight = this.getBlockPathWeight(integer5, integer6, integer7)) > n3) {
                    n3 = blockPathWeight;
                    integer2 = integer5;
                    integer3 = integer6;
                    integer4 = integer7;
                }
            }
            if (integer2 > 0) {
                this.pathToEntity = this.worldObj.pathFinder.createEntityPathTo(this, integer2, integer3, integer4, 16.0f);
            }
        }
        final boolean handleWaterMovement = this.handleWaterMovement();
        final boolean handleLavaMovement = this.handleLavaMovement();
        if (this.pathToEntity == null || this.rand.nextInt(100) == 0) {
            super.updatePlayerActionState();
            this.pathToEntity = null;
            return;
        }
        Vec3D vec3D = this.pathToEntity.getPosition(this);
        double n3 = this.width * 2.0f;
        while (vec3D != null) {
            final Vec3D vec3D2 = vec3D;
            final double posX = this.posX;
            final double posY = this.posY;
            final double posZ = this.posZ;
            double n = posY;
            double n2 = posX;
            final Vec3D vec3D3 = vec3D2;
            n2 -= vec3D3.xCoord;
            n -= vec3D3.yCoord;
            final double n4 = posZ - vec3D3.zCoord;
            if (n2 * n2 + n * n + n4 * n4 >= n3 * n3 || vec3D.yCoord > this.posY) {
                break;
            }
            this.pathToEntity.incrementPathIndex();
            if (this.pathToEntity.isFinished()) {
                vec3D = null;
                this.pathToEntity = null;
            }
            else {
                vec3D = this.pathToEntity.getPosition(this);
            }
        }
        this.isJumping = false;
        if (vec3D != null) {
            final double n4 = vec3D.xCoord - this.posX;
            final double n2 = vec3D.zCoord - this.posZ;
            final double n = vec3D.yCoord - this.posY;
            this.rotationYaw = (double)(Math.atan2((double)n2, (double)n4) * 180.0 / 3.1415927410125732) - 90.0f;
            this.moveForward = this.moveSpeed;
            if (n > 0.0f) {
                this.isJumping = true;
            }
        }
        if (this.rand.nextDouble() < 0.8f && (handleWaterMovement || handleLavaMovement)) {
            this.isJumping = true;
        }
    }
    
    protected void attackEntity(final Entity entity, final double double2) {
    }
    
    protected double getBlockPathWeight(final int integer1, final int integer2, final int integer3) {
        return 0.0f;
    }
    
    protected Entity findPlayerToAttack() {
        return null;
    }
    
    @Override
    public boolean getCanSpawnHere(final double double1, final double double2, final double double3) {
        return super.getCanSpawnHere(double1, double2, double3) && this.getBlockPathWeight((int)double1, (int)double2, (int)double3) >= 0.0f;
    }
}
