package net.minecraft.game.entity;

import java.util.List;
import com.nbt.NBTTagCompound;
import net.minecraft.game.block.StepSound;
import net.minecraft.game.block.Block;
import net.minecraft.game.MathHelper;
import net.minecraft.game.level.World;

public class EntityLiving extends Entity {
    public int heartsHalvesLife;
    public double renderYawOffset;
    public double prevRenderYawOffset;
    private double rotationYawHead;
    private double prevRotationYawHead;
    protected String texture;
    private int scoreValue;
    public int health;
    public int prevHealth;
    private int livingSoundTime;
    public int hurtTime;
    public int maxHurtTime;
    public double attackedAtYaw;
    public int deathTime;
    public int attackTime;
    public double prevCameraPitch;
    public double cameraPitch;
    public double newPosZ;
    public double newRotationYaw;
    public double newRotationPitch;
    protected int entityAge;
    protected double moveStrafing;
    protected double moveForward;
    private double randomYawVelocity;
    protected boolean isJumping;
    private double defaultPitch;
    protected double moveSpeed;
    
    public EntityLiving(final World world) {
        super(world);
        this.heartsHalvesLife = 20;
        this.renderYawOffset = 0.0f;
        this.prevRenderYawOffset = 0.0f;
        this.texture = "/char.png";
        this.scoreValue = 0;
        this.attackedAtYaw = 0.0f;
        this.deathTime = 0;
        this.attackTime = 0;
        Math.random();
        this.entityAge = 0;
        this.isJumping = false;
        this.defaultPitch = 0.0f;
        this.moveSpeed = 0.7f;
        this.health = 10;
        this.preventEntitySpawning = true;
        Math.random();
        this.setPosition(this.posX, this.posY, this.posZ);
        Math.random();
        this.rotationYaw = (double)(Math.random() * 3.1415927410125732 * 2.0);
        this.stepHeight = 0.5f;
    }
    
    @Override
    public final String addToPlayerScore() {
        return this.texture;
    }
    
    @Override
    public final boolean canBeCollidedWith() {
        return !this.isDead;
    }
    
    @Override
    public final boolean canBePushed() {
        return !this.isDead;
    }
    
    @Override
    protected double getEyeHeight() {
        return this.height * 0.85f;
    }
    
    @Override
    public void onEntityUpdate() {
        super.onEntityUpdate();
        if (this.rand.nextInt(1000) < this.livingSoundTime++) {
            this.livingSoundTime = -80;
            final String livingSound;
            if ((livingSound = this.getLivingSound()) != null) {
                this.worldObj.playSoundAtEntity(this, livingSound, 1.0f, (this.rand.nextDouble() - this.rand.nextDouble()) * 0.2f + 1.0f);
            }
        }
        if (this.isInsideOfMaterial()) {
            --this.air;
            if (this.air == -20) {
                this.air = 0;
                for (int i = 0; i < 8; ++i) {
                    final double n = this.rand.nextDouble() - this.rand.nextDouble();
                    final double sqrt_double = this.rand.nextDouble() - this.rand.nextDouble();
                    final double renderYawOffset = this.rand.nextDouble() - this.rand.nextDouble();
                    this.worldObj.spawnParticle("bubble", this.posX + n, this.posY + sqrt_double, this.posZ + renderYawOffset, this.motionX, this.motionY, this.motionZ);
                }
                this.attackEntityFrom(null, 2);
            }
            this.fire = 0;
        }
        else {
            this.air = this.maxAir;
        }
        this.prevCameraPitch = this.cameraPitch;
        if (this.attackTime > 0) {
            --this.attackTime;
        }
        if (this.hurtTime > 0) {
            --this.hurtTime;
        }
        if (this.heartsLife > 0) {
            --this.heartsLife;
        }
        if (this.health <= 0) {
            ++this.deathTime;
            if (this.deathTime > 20) {
                this.setEntityDead();
            }
        }
        this.prevRenderYawOffset = this.renderYawOffset;
        this.prevRotationYaw = this.rotationYaw;
        this.prevRotationPitch = this.rotationPitch;
        this.onLivingUpdate();
        double n2 = this.posX - this.prevPosX;
        final double n = this.posZ - this.prevPosZ;
        final double sqrt_double = MathHelper.sqrt_double(n2 * n2 + n * n);
        double renderYawOffset = this.renderYawOffset;
        double n3 = 0.0f;
        double n4 = 0.0f;
        if (sqrt_double > 0.05f) {
            n4 = 1.0f;
            n3 = sqrt_double * 3.0f;
            renderYawOffset = (double)Math.atan2((double)n, (double)n2) * 180.0f / 3.1415927f - 90.0f;
        }
        if (!this.onGround) {
            n4 = 0.0f;
        }
        this.rotationYawHead += (n4 - this.rotationYawHead) * 0.3f;
        for (n2 = renderYawOffset - this.renderYawOffset; n2 < -180.0f; n2 += 360.0f) {}
        while (n2 >= 180.0f) {
            n2 -= 360.0f;
        }
        this.renderYawOffset += n2 * 0.1f;
        for (n2 = this.rotationYaw - this.renderYawOffset; n2 < -180.0f; n2 += 360.0f) {}
        while (n2 >= 180.0f) {
            n2 -= 360.0f;
        }
        final boolean b = n2 < -90.0f || n2 >= 90.0f;
        if (n2 < -75.0f) {
            n2 = -75.0f;
        }
        if (n2 >= 75.0f) {
            n2 = 75.0f;
        }
        this.renderYawOffset = this.rotationYaw - n2;
        this.renderYawOffset += n2 * 0.1f;
        if (b) {
            n3 = -n3;
        }
        while (this.rotationYaw - this.prevRotationYaw < -180.0f) {
            this.prevRotationYaw -= 360.0f;
        }
        while (this.rotationYaw - this.prevRotationYaw >= 180.0f) {
            this.prevRotationYaw += 360.0f;
        }
        while (this.renderYawOffset - this.prevRenderYawOffset < -180.0f) {
            this.prevRenderYawOffset -= 360.0f;
        }
        while (this.renderYawOffset - this.prevRenderYawOffset >= 180.0f) {
            this.prevRenderYawOffset += 360.0f;
        }
        while (this.rotationPitch - this.prevRotationPitch < -180.0f) {
            this.prevRotationPitch -= 360.0f;
        }
        while (this.rotationPitch - this.prevRotationPitch >= 180.0f) {
            this.prevRotationPitch += 360.0f;
        }
        this.prevRotationYawHead += n3;
    }
    
    @Override
    protected final void setSize(final double width, final double height) {
        super.setSize(width, height);
    }
    
    public final void heal(final int integer) {
        if (this.health <= 0) {
            return;
        }
        this.health += integer;
        if (this.health > 20) {
            this.health = 20;
        }
        this.heartsLife = this.heartsHalvesLife / 2;
    }
    
    @Override
    public boolean attackEntityFrom(final Entity entity, final int integer) {
        if (!this.worldObj.survivalWorld) {
            return false;
        }
        this.entityAge = 0;
        if (this.health <= 0) {
            return false;
        }
        this.newRotationYaw = 1.5f;
        if (this.heartsLife > this.heartsHalvesLife / 2.0f) {
            if (this.prevHealth - integer >= this.health) {
                return false;
            }
            this.health = this.prevHealth - integer;
        }
        else {
            this.prevHealth = this.health;
            this.heartsLife = this.heartsHalvesLife;
            this.health -= integer;
            final int n = 10;
            this.maxHurtTime = n;
            this.hurtTime = n;
        }
        this.attackedAtYaw = 0.0f;
        if (entity != null) {
            final double n2 = entity.posX - this.posX;
            double n3 = entity.posZ - this.posZ;
            this.attackedAtYaw = (double)(Math.atan2((double)n3, (double)n2) * 180.0 / 3.1415927410125732) - this.rotationYaw;
            final double n4 = n2;
            final double n5 = n3;
            n3 = n4;
            final double sqrt_double = MathHelper.sqrt_double(n3 * n3 + n5 * n5);
            this.motionX /= 2.0f;
            this.motionY /= 2.0f;
            this.motionZ /= 2.0f;
            this.motionX -= n3 / sqrt_double * 0.4f;
            this.motionY += 0.4f;
            this.motionZ -= n5 / sqrt_double * 0.4f;
            if (this.motionY > 0.4f) {
                this.motionY = 0.4f;
            }
        }
        else {
            this.attackedAtYaw = (double)((int)(Math.random() * 2.0) * 180);
        }
        if (this.health <= 0) {
            this.worldObj.playSoundAtEntity(this, this.getDeathSound(), 1.0f, (this.rand.nextDouble() - this.rand.nextDouble()) * 0.2f + 1.0f);
            this.onDeath(entity);
        }
        else {
            this.worldObj.playSoundAtEntity(this, this.getHurtSound(), 1.0f, (this.rand.nextDouble() - this.rand.nextDouble()) * 0.2f + 1.0f);
        }
        return true;
    }
    
    protected String getLivingSound() {
        return null;
    }
    
    protected String getHurtSound() {
        return "random.hurt";
    }
    
    protected String getDeathSound() {
        return "random.hurt";
    }
    
    public void onDeath(final Entity entity) {
        final int scoreValue;
        if ((scoreValue = this.scoreValue()) > 0) {
            for (int nextInt = this.rand.nextInt(3), i = 0; i < nextInt; ++i) {
                this.dropItemWithOffset(scoreValue, 1);
            }
        }
    }
    
    protected int scoreValue() {
        return 0;
    }
    
    @Override
    protected final void fall(final double double1) {
        int blockId;
        if ((blockId = (int)Math.ceil((double)(double1 - 3.0f))) > 0) {
            this.attackEntityFrom(null, blockId);
            if ((blockId = this.worldObj.getBlockId((int)this.posX, (int)(this.posY - 0.2f - this.yOffset), (int)this.posZ)) > 0) {
                final StepSound stepSound = Block.blocksList[blockId].stepSound;
                this.worldObj.playSoundAtEntity(this, stepSound.stepSoundDir2(), stepSound.soundVolume * 0.5f, stepSound.soundPitch * 0.75f);
            }
        }
    }
    
    @Override
    protected void writeEntityToNBT(final NBTTagCompound nbtTagCompound) {
        nbtTagCompound.setShort("Health", (short)this.health);
        nbtTagCompound.setShort("HurtTime", (short)this.hurtTime);
        nbtTagCompound.setShort("DeathTime", (short)this.deathTime);
        nbtTagCompound.setShort("AttackTime", (short)this.attackTime);
    }
    
    @Override
    protected void readEntityFromNBT(final NBTTagCompound nbtTagCompound) {
        this.health = nbtTagCompound.getShort("Health");
        if (!nbtTagCompound.hasKey("Health")) {
            this.health = 10;
        }
        this.hurtTime = nbtTagCompound.getShort("HurtTime");
        this.deathTime = nbtTagCompound.getShort("DeathTime");
        this.attackTime = nbtTagCompound.getShort("AttackTime");
    }
    
    @Override
    protected String getEntityString() {
        return "Mob";
    }
    
    @Override
    public final boolean isEntityAlive() {
        return !this.isDead && this.health > 0;
    }
    
    public void onLivingUpdate() {
        ++this.entityAge;
        final Entity player;
        if (this.entityAge > 600 && this.rand.nextInt(800) == 0 && (player = this.worldObj.getPlayer()) != null) {
            double double1 = player.posX - this.posX;
            final double moveForward = player.posY - this.posY;
            final double n = player.posZ - this.posZ;
            if ((double1 = double1 * double1 + moveForward * moveForward + n * n) < 1024.0f) {
                this.entityAge = 0;
            }
            else {
                this.setEntityDead();
            }
        }
        if (this.health <= 0) {
            this.isJumping = false;
            this.moveStrafing = 0.0f;
            this.moveForward = 0.0f;
            this.randomYawVelocity = 0.0f;
        }
        else {
            this.updatePlayerActionState();
        }
        final boolean handleWaterMovement = this.handleWaterMovement();
        final boolean handleLavaMovement = this.handleLavaMovement();
        if (this.isJumping) {
            if (handleWaterMovement) {
                this.motionY += 0.04f;
            }
            else if (handleLavaMovement) {
                this.motionY += 0.04f;
            }
            else if (this.onGround) {
                this.motionY = 0.42f;
            }
        }
        this.moveStrafing *= 0.98f;
        this.moveForward *= 0.98f;
        this.randomYawVelocity *= 0.9f;
        final double moveStrafing = this.moveStrafing;
        final double moveForward = this.moveForward;
        double double1 = moveStrafing;
        if (this.handleWaterMovement()) {
            final double n2 = this.posY;
            this.moveFlying(double1, moveForward, 0.02f);
            this.moveEntity(this.motionX, this.motionY, this.motionZ);
            this.motionX *= 0.8f;
            this.motionY *= 0.8f;
            this.motionZ *= 0.8f;
            this.motionY -= (double)0.02;
            if (this.isCollidedHorizontally && this.isOffsetPositionInLiquid(this.motionX, this.motionY + 0.6f - this.posY + n2, this.motionZ)) {
                this.motionY = 0.3f;
            }
        }
        else if (this.handleLavaMovement()) {
            final double n2 = this.posY;
            this.moveFlying(double1, moveForward, 0.02f);
            this.moveEntity(this.motionX, this.motionY, this.motionZ);
            this.motionX *= 0.5f;
            this.motionY *= 0.5f;
            this.motionZ *= 0.5f;
            this.motionY -= (double)0.02;
            if (this.isCollidedHorizontally && this.isOffsetPositionInLiquid(this.motionX, this.motionY + 0.6f - this.posY + n2, this.motionZ)) {
                this.motionY = 0.3f;
            }
        }
        else {
            this.moveFlying(double1, moveForward, this.onGround ? 0.1f : 0.02f);
            this.moveEntity(this.motionX, this.motionY, this.motionZ);
            this.motionX *= 0.91f;
            this.motionY *= 0.98f;
            this.motionZ *= 0.91f;
            this.motionY -= (double)0.08;
            if (this.onGround) {
                this.motionX *= 0.6f;
                this.motionZ *= 0.6f;
            }
        }
        this.newPosZ = this.newRotationYaw;
        final double n2 = this.posX - this.prevPosX;
        double1 = this.posZ - this.prevPosZ;
        if ((double1 = MathHelper.sqrt_double(n2 * n2 + double1 * double1) * 4.0f) > 1.0f) {
            double1 = 1.0f;
        }
        this.newRotationYaw += (double1 - this.newRotationYaw) * 0.4f;
        this.newRotationPitch += this.newRotationYaw;
        final List entitiesWithinAABBExcludingEntity;
        if ((entitiesWithinAABBExcludingEntity = this.worldObj.getEntitiesWithinAABBExcludingEntity(this, this.boundingBox.expand(0.2f, 0.0f, 0.2f))) != null && entitiesWithinAABBExcludingEntity.size() > 0) {
            for (int i = 0; i < entitiesWithinAABBExcludingEntity.size(); ++i) {
                final Entity entity;
                if ((entity = (Entity)entitiesWithinAABBExcludingEntity.get(i)).canBePushed()) {
                    entity.applyEntityCollision(this);
                }
            }
        }
    }
    
    protected void updatePlayerActionState() {
        if (this.rand.nextDouble() < 0.07f) {
            this.moveStrafing = (this.rand.nextDouble() - 0.5f) * this.moveSpeed;
            this.moveForward = this.rand.nextDouble() * this.moveSpeed;
        }
        this.isJumping = (this.rand.nextDouble() < 0.01f);
        if (this.rand.nextDouble() < 0.04f) {
            this.randomYawVelocity = (this.rand.nextDouble() - 0.5f) * 60.0f;
        }
        this.rotationYaw += this.randomYawVelocity;
        this.rotationPitch = 0.0f;
        final boolean handleWaterMovement = this.handleWaterMovement();
        final boolean handleLavaMovement = this.handleLavaMovement();
        if (handleWaterMovement || handleLavaMovement) {
            this.isJumping = (this.rand.nextDouble() < 0.8f);
        }
    }
    
    public boolean getCanSpawnHere(final double double1, final double double2, final double double3) {
        this.setPosition(double1, double2 + this.height / 2.0f, double3);
        return this.worldObj.checkIfAABBIsClear1(this.boundingBox) && this.worldObj.getCollidingBoundingBoxes(this.boundingBox).size() == 0 && !this.worldObj.getIsAnyLiquid(this.boundingBox);
    }
}
