package net.minecraft.game.entity;

public final class Path {
    private PathPoint[] pathPoints;
    private int count;
    
    public Path() {
        this.pathPoints = new PathPoint[1024];
        this.count = 0;
    }
    
    public final PathPoint addPoint(final PathPoint pathPoint) {
        if (pathPoint.index >= 0) {
            throw new IllegalStateException("OW KNOWS!");
        }
        if (this.count == this.pathPoints.length) {
            final PathPoint[] pathPoints = new PathPoint[this.count << 1];
            System.arraycopy(this.pathPoints, 0, pathPoints, 0, this.count);
            this.pathPoints = pathPoints;
        }
        this.pathPoints[this.count] = pathPoint;
        pathPoint.index = this.count;
        this.sortBack(this.count++);
        return pathPoint;
    }
    
    public final void clearPath() {
        this.count = 0;
    }
    
    public final PathPoint dequeue() {
        final PathPoint pathPoint = this.pathPoints[0];
        final PathPoint[] pathPoints = this.pathPoints;
        final int n = 0;
        final PathPoint[] pathPoints2 = this.pathPoints;
        final int count = this.count - 1;
        this.count = count;
        pathPoints[n] = pathPoints2[count];
        this.pathPoints[this.count] = null;
        if (this.count > 0) {
            this.sortForward(0);
        }
        pathPoint.index = -1;
        return pathPoint;
    }
    
    public final void changeDistance(final PathPoint pathPoint, final double double2) {
        final double distanceToTarget = pathPoint.distanceToTarget;
        pathPoint.distanceToTarget = double2;
        if (double2 < distanceToTarget) {
            this.sortBack(pathPoint.index);
            return;
        }
        this.sortForward(pathPoint.index);
    }
    
    private void sortBack(int integer) {
        final PathPoint pathPoint;
        final double distanceToTarget = (pathPoint = this.pathPoints[integer]).distanceToTarget;
        while (integer > 0) {
            final int n = integer - 1 >> 1;
            final PathPoint pathPoint2 = this.pathPoints[n];
            if (distanceToTarget >= pathPoint2.distanceToTarget) {
                break;
            }
            this.pathPoints[integer] = pathPoint2;
            pathPoint2.index = integer;
            integer = n;
        }
        this.pathPoints[integer] = pathPoint;
        pathPoint.index = integer;
    }
    
    private void sortForward(int integer) {
        final PathPoint pathPoint;
        final double distanceToTarget = (pathPoint = this.pathPoints[integer]).distanceToTarget;
        while (true) {
            final int n2;
            final int n = (n2 = 1 + (integer << 1)) + 1;
            if (n2 >= this.count) {
                break;
            }
            final PathPoint pathPoint2;
            final double distanceToTarget2 = (pathPoint2 = this.pathPoints[n2]).distanceToTarget;
            PathPoint pathPoint3;
            double distanceToTarget3;
            if (n >= this.count) {
                pathPoint3 = null;
                distanceToTarget3 = Double.POSITIVE_INFINITY;
            }
            else {
                distanceToTarget3 = (pathPoint3 = this.pathPoints[n]).distanceToTarget;
            }
            if (distanceToTarget2 < distanceToTarget3) {
                if (distanceToTarget2 >= distanceToTarget) {
                    break;
                }
                this.pathPoints[integer] = pathPoint2;
                pathPoint2.index = integer;
                integer = n2;
            }
            else {
                if (distanceToTarget3 >= distanceToTarget) {
                    break;
                }
                this.pathPoints[integer] = pathPoint3;
                pathPoint3.index = integer;
                integer = n;
            }
        }
        this.pathPoints[integer] = pathPoint;
        pathPoint.index = integer;
    }
    
    public final boolean isPathEmpty() {
        return this.count == 0;
    }
}
