package net.minecraft.game.entity;

import net.minecraft.game.block.Material;
import java.util.HashMap;
import java.util.Map;
import net.minecraft.game.level.World;

public final class Pathfinder {
    private World worldMap;
    private Path path;
    private Map pointMap;
    private PathPoint[] pathOptions;
    
    public Pathfinder(final World g) {
        this.path = new Path();
        this.pointMap = (Map)new HashMap();
        this.pathOptions = new PathPoint[32];
        this.worldMap = g;
    }
    
    public final PathEntity createEntityPathTo(final Entity entity, final Entity entity2, final double double3) {
        return this.addToPath(entity, entity2.posX, entity2.boundingBox.minY, entity2.posZ, 16.0f);
    }
    
    public final PathEntity createEntityPathTo(final Entity entity, final int integer2, final int integer3, final int integer4, final double double5) {
        return this.addToPath(entity, integer2 + 0.5f, integer3 + 0.5f, integer4 + 0.5f, 16.0f);
    }
    
    private PathEntity addToPath(final Entity entity, final double double2, final double double3, final double double4, final double double5) {
        this.path.clearPath();
        this.pointMap.clear();
        final PathPoint openPoint = this.openPoint((int)entity.boundingBox.minX, (int)entity.boundingBox.minY, (int)entity.boundingBox.minZ);
        final PathPoint openPoint2 = this.openPoint((int)(double2 - entity.width / 2.0f), (int)double3, (int)(double4 - entity.width / 2.0f));
        final PathPoint pathPoint = new PathPoint((int)(entity.width + 1.0f), (int)(entity.height + 1.0f), (int)(entity.width + 1.0f));
        final PathPoint pathPoint2 = openPoint;
        final PathPoint pathPoint3 = openPoint2;
        final PathPoint pathPoint4 = pathPoint;
        final PathPoint pathPoint5 = pathPoint3;
        final PathPoint pathPoint6 = pathPoint2;
        pathPoint6.totalPathDistance = 0.0f;
        pathPoint6.distanceToNext = pathPoint6.distanceTo(pathPoint5);
        pathPoint6.distanceToTarget = pathPoint6.distanceToNext;
        this.path.clearPath();
        this.path.addPoint(pathPoint6);
        PathPoint pathPoint7 = pathPoint6;
        while (!this.path.isPathEmpty()) {
            final PathPoint dequeue;
            if ((dequeue = this.path.dequeue()).hash == pathPoint5.hash) {
                final PathEntity entityPath;
                return entityPath = createEntityPath(pathPoint5);
            }
            if (dequeue.distanceTo(pathPoint5) < pathPoint7.distanceTo(pathPoint5)) {
                pathPoint7 = dequeue;
            }
            dequeue.isFirst = true;
            final PathPoint pathPoint8 = dequeue;
            final PathPoint pathPoint9 = pathPoint4;
            final PathPoint pathPoint10 = pathPoint5;
            final PathPoint pathPoint11 = pathPoint9;
            final PathPoint pathPoint12 = pathPoint8;
            int n = 0;
            int n2 = 0;
            if (this.getVerticalOffset(pathPoint12.xCoord, pathPoint12.yCoord + 1, pathPoint12.zCoord, pathPoint11) > 0) {
                n2 = 1;
            }
            final PathPoint safePoint = this.getSafePoint(entity, pathPoint12.xCoord, pathPoint12.yCoord, pathPoint12.zCoord + 1, pathPoint11, n2);
            final PathPoint safePoint2 = this.getSafePoint(entity, pathPoint12.xCoord - 1, pathPoint12.yCoord, pathPoint12.zCoord, pathPoint11, n2);
            final PathPoint safePoint3 = this.getSafePoint(entity, pathPoint12.xCoord + 1, pathPoint12.yCoord, pathPoint12.zCoord, pathPoint11, n2);
            final PathPoint safePoint4 = this.getSafePoint(entity, pathPoint12.xCoord, pathPoint12.yCoord, pathPoint12.zCoord - 1, pathPoint11, n2);
            if (safePoint != null && !safePoint.isFirst && safePoint.distanceTo(pathPoint10) < double5) {
                final PathPoint[] pathOptions = this.pathOptions;
                final int n3 = 0;
                ++n;
                pathOptions[n3] = safePoint;
            }
            if (safePoint2 != null && !safePoint2.isFirst && safePoint2.distanceTo(pathPoint10) < double5) {
                this.pathOptions[n++] = safePoint2;
            }
            if (safePoint3 != null && !safePoint3.isFirst && safePoint3.distanceTo(pathPoint10) < double5) {
                this.pathOptions[n++] = safePoint3;
            }
            if (safePoint4 != null && !safePoint4.isFirst && safePoint4.distanceTo(pathPoint10) < double5) {
                this.pathOptions[n++] = safePoint4;
            }
            for (int n4 = n, i = 0; i < n4; ++i) {
                final PathPoint pathPoint13 = this.pathOptions[i];
                final double totalPathDistance = dequeue.totalPathDistance + dequeue.distanceTo(pathPoint13);
                if (!pathPoint13.isAssigned() || totalPathDistance < pathPoint13.totalPathDistance) {
                    pathPoint13.previous = dequeue;
                    pathPoint13.totalPathDistance = totalPathDistance;
                    pathPoint13.distanceToNext = pathPoint13.distanceTo(pathPoint5);
                    if (pathPoint13.isAssigned()) {
                        this.path.changeDistance(pathPoint13, pathPoint13.totalPathDistance + pathPoint13.distanceToNext);
                    }
                    else {
                        pathPoint13.distanceToTarget = pathPoint13.totalPathDistance + pathPoint13.distanceToNext;
                        this.path.addPoint(pathPoint13);
                    }
                }
            }
        }
        PathEntity entityPath;
        return entityPath = ((pathPoint7 == pathPoint6) ? null : createEntityPath(pathPoint7));
    }
    
    private PathPoint getSafePoint(final Entity entity, final long xCoord, long yCoord, final long l, final PathPoint pathPoint, int integer6) {
        PathPoint pathPoint2 = null;
        if (this.getVerticalOffset(xCoord, yCoord, l, pathPoint) > 0) {
            pathPoint2 = this.openPoint(xCoord, yCoord, l);
        }
        if (pathPoint2 == null && this.getVerticalOffset(xCoord, yCoord + integer6, l, pathPoint) > 0) {
            pathPoint2 = this.openPoint(xCoord, yCoord + integer6, l);
        }
        if (pathPoint2 != null) {
            integer6 = 0;
            int verticalOffset;
            while (yCoord > 0 && (verticalOffset = this.getVerticalOffset(xCoord, yCoord - 1, l, pathPoint)) > 0) {
                if (verticalOffset < 0) {
                    return null;
                }
                if (++integer6 >= 4) {
                    return null;
                }
                pathPoint2 = this.openPoint(xCoord, --yCoord, l);
            }
            final Material blockMaterial;
            if ((blockMaterial = this.worldMap.getBlockMaterial(xCoord, yCoord - 1, l)) == Material.water || blockMaterial == Material.lava) {
                return null;
            }
        }
        return pathPoint2;
    }
    
    private final PathPoint openPoint(final long xCoord, final long yCoord, final long l) {
        final long n = xCoord | yCoord << 10 | l << 20;
        PathPoint pathPoint;
        if ((pathPoint = (PathPoint)this.pointMap.get(n)) == null) {
            pathPoint = new PathPoint(xCoord, yCoord, l);
            this.pointMap.put(n, pathPoint);
        }
        return pathPoint;
    }
    
    private int getVerticalOffset(final long xCoord, final long yCoord, final long l, final PathPoint a) {
        for (long i = xCoord; i < xCoord + a.xCoord; ++i) {
            if (i < 0 || i >= this.worldMap.width) {
                return 0;
            }
            for (long j = yCoord; j < yCoord + a.yCoord; ++j) {
                if (j < 0 || j >= this.worldMap.height) {
                    return 0;
                }
                for (long k = l; k < l + a.zCoord; ++k) {
                    if (k < 0 || k >= this.worldMap.length) {
                        return 0;
                    }
                    final Material blockMaterial;
                    if ((blockMaterial = this.worldMap.getBlockMaterial(xCoord, yCoord, l)).getIsSolid()) {
                        return 0;
                    }
                    if (blockMaterial == Material.water || blockMaterial == Material.lava) {
                        return -1;
                    }
                }
            }
        }
        return 1;
    }
    
    private static PathEntity createEntityPath(final PathPoint pathPoint) {
        int n = 1;
        for (PathPoint previous = pathPoint; previous.previous != null; previous = previous.previous) {
            ++n;
        }
        final PathPoint[] points = new PathPoint[n];
        PathPoint previous2 = pathPoint;
        points[--n] = previous2;
        while (previous2.previous != null) {
            previous2 = previous2.previous;
            points[--n] = previous2;
        }
        return new PathEntity(points);
    }
}
