package net.minecraft.game.entity.monster;

import net.minecraft.game.item.Item;
import com.nbt.NBTTagCompound;
import net.minecraft.game.MathHelper;
import net.minecraft.game.entity.Entity;
import net.minecraft.game.level.World;

public class EntitySpider extends EntityMonster {
    public EntitySpider(final World world) {
        super(world);
        this.texture = "/mob/spider.png";
        this.setSize(1.4f, 0.9f);
        this.moveSpeed = 0.8f;
    }
    
    @Override
    protected final Entity findPlayerToAttack() {
        if (this.getEntityBrightness(1.0f) < 0.5f && this.worldObj.player.getDistanceSqToEntity(this) < 256.0f) {
            return this.worldObj.player;
        }
        return null;
    }
    
    @Override
    protected final void attackEntity(final Entity entity, double double2) {
        double n;
        if ((n = this.getEntityBrightness(1.0f)) > 0.5f && this.rand.nextInt(100) == 0) {
            this.playerToAttack = null;
            return;
        }
        if (double2 > 2.0f && double2 < 6.0f && this.rand.nextInt(10) == 0) {
            if (this.onGround) {
                double2 = entity.posX - this.posX;
                final double n2 = entity.posZ - this.posZ;
                n = MathHelper.sqrt_double(double2 * double2 + n2 * n2);
                this.motionX = double2 / n * 0.5f * 0.8f + this.motionX * 0.2f;
                this.motionZ = n2 / n * 0.5f * 0.8f + this.motionZ * 0.2f;
                this.motionY = 0.4f;
            }
        }
        else {
            super.attackEntity(entity, double2);
        }
    }
    
    @Override
    protected final void writeEntityToNBT(final NBTTagCompound nbtTagCompound) {
        super.writeEntityToNBT(nbtTagCompound);
    }
    
    @Override
    protected final void readEntityFromNBT(final NBTTagCompound nbtTagCompound) {
        super.readEntityFromNBT(nbtTagCompound);
    }
    
    @Override
    protected final String getEntityString() {
        return "Spider";
    }
    
    @Override
    protected final int scoreValue() {
        return Item.silk.shiftedIndex;
    }
}
