package net.minecraft.game.entity.other;

import net.minecraft.game.item.ItemStack;
import net.minecraft.game.item.Item;
import net.minecraft.game.entity.player.EntityPlayer;
import com.nbt.NBTTagCompound;
import java.util.List;
import net.minecraft.client.physics.MovingObjectPosition;
import net.minecraft.client.renderer.Vec3D;
import net.minecraft.game.MathHelper;
import net.minecraft.game.level.World;
import net.minecraft.game.entity.EntityLiving;
import net.minecraft.game.entity.Entity;

public class EntityArrow extends Entity {
    private long xTile;
    private long yTile;
    private long zTile;
    private int inTile;
    private boolean inGround;
    public int arrowShake;
    private EntityLiving owner;
    private int ticksInGround;
    private int ticksInAir;
    
    public EntityArrow(final World world, final EntityLiving entityLiving) {
        super(world);
        this.xTile = -1;
        this.yTile = -1;
        this.zTile = -1;
        this.inTile = 0;
        this.inGround = false;
        this.arrowShake = 0;
        this.ticksInAir = 0;
        this.owner = entityLiving;
        this.setSize(0.5f, 0.5f);
        this.setPositionAndRotation(entityLiving.posX, entityLiving.posY, entityLiving.posZ, entityLiving.rotationYaw, entityLiving.rotationPitch);
        this.posX -= MathHelper.cos(this.rotationYaw / 180.0f * 3.1415927f) * 0.16f;
        this.posY -= 0.1f;
        this.posZ -= MathHelper.sin(this.rotationYaw / 180.0f * 3.1415927f) * 0.16f;
        this.setPosition(this.posX, this.posY, this.posZ);
        this.yOffset = 0.0f;
        this.motionX = -MathHelper.sin(this.rotationYaw / 180.0f * 3.1415927f) * MathHelper.cos(this.rotationPitch / 180.0f * 3.1415927f);
        this.motionZ = MathHelper.cos(this.rotationYaw / 180.0f * 3.1415927f) * MathHelper.cos(this.rotationPitch / 180.0f * 3.1415927f);
        this.motionY = -MathHelper.sin(this.rotationPitch / 180.0f * 3.1415927f);
        this.setArrowHeading(this.motionX, this.motionY, this.motionZ, 1.5f, 1.0f);
    }
    
    public final void setArrowHeading(double double1, double double2, double double3, double double4, final double double5) {
        final double sqrt_double = MathHelper.sqrt_double(double1 * double1 + double2 * double2 + double3 * double3);
        double1 /= sqrt_double;
        double2 /= sqrt_double;
        double3 /= sqrt_double;
        double1 += (double)(this.rand.nextGaussian() * 0.007499999832361937 * double5);
        double2 += (double)(this.rand.nextGaussian() * 0.007499999832361937 * double5);
        double3 += (double)(this.rand.nextGaussian() * 0.007499999832361937 * double5);
        double1 *= double4;
        double2 *= double4;
        double3 *= double4;
        this.motionX = double1;
        this.motionY = double2;
        this.motionZ = double3;
        double4 = MathHelper.sqrt_double(double1 * double1 + double3 * double3);
        final double n = (double)(Math.atan2((double)double1, (double)double3) * 180.0 / 3.1415927410125732);
        this.rotationYaw = n;
        this.prevRotationYaw = n;
        final double n2 = (double)(Math.atan2((double)double2, (double)double4) * 180.0 / 3.1415927410125732);
        this.rotationPitch = n2;
        this.prevRotationPitch = n2;
        this.ticksInGround = 0;
    }
    
    @Override
    public final void onEntityUpdate() {
        super.onEntityUpdate();
        if (this.arrowShake > 0) {
            --this.arrowShake;
        }
        if (this.inGround) {
            if (this.worldObj.getBlockId(this.xTile, this.yTile, this.zTile) == this.inTile) {
                ++this.ticksInGround;
                if (this.ticksInGround == 1200) {
                    this.setEntityDead();
                }
                return;
            }
            this.inGround = false;
            this.motionX *= this.rand.nextDouble() * 0.2f;
            this.motionY *= this.rand.nextDouble() * 0.2f;
            this.motionZ *= this.rand.nextDouble() * 0.2f;
            this.ticksInGround = 0;
            this.ticksInAir = 0;
        }
        else {
            ++this.ticksInAir;
        }
        MovingObjectPosition rayTraceBlocks = this.worldObj.rayTraceBlocks(new Vec3D(this.posX, this.posY, this.posZ), new Vec3D(this.posX + this.motionX, this.posY + this.motionY, this.posZ + this.motionZ));
        final Vec3D vec3D = new Vec3D(this.posX, this.posY, this.posZ);
        Vec3D vec3D2 = new Vec3D(this.posX + this.motionX, this.posY + this.motionY, this.posZ + this.motionZ);
        if (rayTraceBlocks != null) {
            vec3D2 = new Vec3D(rayTraceBlocks.hitVec.xCoord, rayTraceBlocks.hitVec.yCoord, rayTraceBlocks.hitVec.zCoord);
        }
        Entity entityHit = null;
        final List entitiesWithinAABBExcludingEntity = this.worldObj.blockMap.getEntitiesWithinAABBExcludingEntity(this, this.boundingBox.addCoord(this.motionX, this.motionY, this.motionZ).expand(1.0f, 1.0f, 1.0f));
        double n = 0.0f;
        for (int i = 0; i < entitiesWithinAABBExcludingEntity.size(); ++i) {
            final Entity entity;
            final MovingObjectPosition calculateIntercept;
            final double distance;
            if ((entity = (Entity)entitiesWithinAABBExcludingEntity.get(i)).canBeCollidedWith() && (entity != this.owner || this.ticksInAir >= 5) && (calculateIntercept = entity.boundingBox.expand(0.3f, 0.3f, 0.3f).calculateIntercept(vec3D, vec3D2)) != null && ((distance = vec3D.distance(calculateIntercept.hitVec)) < n || n == 0.0f)) {
                entityHit = entity;
                n = distance;
            }
        }
        if (entityHit != null) {
            rayTraceBlocks = new MovingObjectPosition(entityHit);
        }
        if (rayTraceBlocks != null) {
            if (rayTraceBlocks.entityHit != null) {
                if (rayTraceBlocks.entityHit.attackEntityFrom(this, 4)) {
                    this.worldObj.playSoundAtEntity((Entity)this, "random.drr", 1.0f, 1.2f / (this.rand.nextDouble() * 0.2f + 0.9f));
                    this.setEntityDead();
                }
                else {
                    this.motionX *= -0.1f;
                    this.motionY *= -0.1f;
                    this.motionZ *= -0.1f;
                    this.rotationYaw += 180.0f;
                    this.prevRotationYaw += 180.0f;
                    this.ticksInAir = 0;
                }
            }
            else {
                this.xTile = rayTraceBlocks.blockX;
                this.yTile = rayTraceBlocks.blockY;
                this.zTile = rayTraceBlocks.blockZ;
                this.inTile = this.worldObj.getBlockId(this.xTile, this.yTile, this.zTile);
                this.motionX = rayTraceBlocks.hitVec.xCoord - this.posX;
                this.motionY = rayTraceBlocks.hitVec.yCoord - this.posY;
                this.motionZ = rayTraceBlocks.hitVec.zCoord - this.posZ;
                final double n2 = MathHelper.sqrt_double(this.motionX * this.motionX + this.motionY * this.motionY + this.motionZ * this.motionZ);
                this.posX -= this.motionX / n2 * 0.05f;
                this.posY -= this.motionY / n2 * 0.05f;
                this.posZ -= this.motionZ / n2 * 0.05f;
                this.worldObj.playSoundAtEntity((Entity)this, "random.drr", 1.0f, 1.2f / (this.rand.nextDouble() * 0.2f + 0.9f));
                this.inGround = true;
                this.arrowShake = 7;
            }
        }
        this.posX += this.motionX;
        this.posY += this.motionY;
        this.posZ += this.motionZ;
        final double n2 = MathHelper.sqrt_double(this.motionX * this.motionX + this.motionZ * this.motionZ);
        this.rotationYaw = (double)(Math.atan2((double)this.motionX, (double)this.motionZ) * 180.0 / 3.1415927410125732);
        this.rotationPitch = (double)(Math.atan2((double)this.motionY, (double)n2) * 180.0 / 3.1415927410125732);
        while (this.rotationPitch - this.prevRotationPitch < -180.0f) {
            this.prevRotationPitch -= 360.0f;
        }
        while (this.rotationPitch - this.prevRotationPitch >= 180.0f) {
            this.prevRotationPitch += 360.0f;
        }
        while (this.rotationYaw - this.prevRotationYaw < -180.0f) {
            this.prevRotationYaw -= 360.0f;
        }
        while (this.rotationYaw - this.prevRotationYaw >= 180.0f) {
            this.prevRotationYaw += 360.0f;
        }
        this.rotationPitch = this.prevRotationPitch + (this.rotationPitch - this.prevRotationPitch) * 0.2f;
        this.rotationYaw = this.prevRotationYaw + (this.rotationYaw - this.prevRotationYaw) * 0.2f;
        double n3 = 0.99f;
        if (this.handleWaterMovement()) {
            for (int j = 0; j < 4; ++j) {
                this.worldObj.spawnParticle("bubble", this.posX - this.motionX * 0.25f, this.posY - this.motionY * 0.25f, this.posZ - this.motionZ * 0.25f, this.motionX, this.motionY, this.motionZ);
            }
            n3 = 0.8f;
        }
        this.motionX *= n3;
        this.motionY *= n3;
        this.motionZ *= n3;
        this.motionY -= 0.03f;
        this.setPosition(this.posX, this.posY, this.posZ);
    }
    
    @Override
    protected final void writeEntityToNBT(final NBTTagCompound nbtTagCompound) {
        nbtTagCompound.setShort("xTile", (short)this.xTile);
        nbtTagCompound.setShort("yTile", (short)this.yTile);
        nbtTagCompound.setShort("zTile", (short)this.zTile);
        nbtTagCompound.setByte("inTile", (byte)this.inTile);
        nbtTagCompound.setByte("shake", (byte)this.arrowShake);
        nbtTagCompound.setByte("inGround", (byte)(byte)(this.inGround ? 1 : 0));
    }
    
    @Override
    protected final void readEntityFromNBT(final NBTTagCompound nbtTagCompound) {
        this.xTile = nbtTagCompound.getShort("xTile");
        this.yTile = nbtTagCompound.getShort("yTile");
        this.zTile = nbtTagCompound.getShort("zTile");
        this.inTile = (nbtTagCompound.getByte("inTile") & 0xFF);
        this.arrowShake = (nbtTagCompound.getByte("shake") & 0xFF);
        this.inGround = (nbtTagCompound.getByte("inGround") == 1);
    }
    
    @Override
    protected final String getEntityString() {
        return "Arrow";
    }
    
    @Override
    public final void onCollideWithPlayer(final EntityPlayer player) {
        if (this.inGround && this.owner == player && this.arrowShake <= 0 && player.inventory.storePartialItemStack(new ItemStack(Item.arrow.shiftedIndex, 1))) {
            this.worldObj.playSoundAtEntity((Entity)this, "random.pop", 0.2f, ((this.rand.nextDouble() - this.rand.nextDouble()) * 0.7f + 1.0f) * 2.0f);
            player.onItemPickup(this);
            this.setEntityDead();
        }
    }
    
    @Override
    public final double getShadowSize() {
        return 0.0f;
    }
}
