package net.minecraft.game.entity.other;

import com.nbt.NBTTagCompound;
import net.minecraft.game.MathHelper;
import net.minecraft.game.level.World;
import net.minecraft.game.entity.Entity;

public class EntityTNT extends Entity {
    public int fuse;
    
    public EntityTNT(final World world, final double prevPosX, final double prevPosY, final double prevPosZ) {
        super(world);
        this.fuse = 0;
        this.preventEntitySpawning = true;
        this.setSize(0.98f, 0.98f);
        this.yOffset = this.height / 2.0f;
        this.setPosition(prevPosX, prevPosY, prevPosZ);
        final double n = (double)(Math.random() * 3.1415927410125732 * 2.0);
        this.motionX = -MathHelper.sin(n * 3.1415927f / 180.0f) * 0.02f;
        this.motionY = 0.2f;
        this.motionZ = -MathHelper.cos(n * 3.1415927f / 180.0f) * 0.02f;
        this.entityWalks = false;
        this.fuse = 80;
        this.prevPosX = prevPosX;
        this.prevPosY = prevPosY;
        this.prevPosZ = prevPosZ;
    }
    
    @Override
    public final boolean canBeCollidedWith() {
        return !this.isDead;
    }
    
    @Override
    public final void onEntityUpdate() {
        this.prevPosX = this.posX;
        this.prevPosY = this.posY;
        this.prevPosZ = this.posZ;
        this.motionY -= 0.04f;
        this.moveEntity(this.motionX, this.motionY, this.motionZ);
        this.motionX *= 0.98f;
        this.motionY *= 0.98f;
        this.motionZ *= 0.98f;
        if (this.onGround) {
            this.motionX *= 0.7f;
            this.motionZ *= 0.7f;
            this.motionY *= -0.5f;
        }
        if (this.fuse-- <= 0) {
            this.setEntityDead();
            this.worldObj.createExplosion(null, this.posX, this.posY, this.posZ, 4.0f);
            return;
        }
        this.worldObj.spawnParticle("smoke", this.posX, this.posY + 0.5f, this.posZ, 0.0f, 0.0f, 0.0f);
    }
    
    @Override
    protected final void writeEntityToNBT(final NBTTagCompound nbtTagCompound) {
        nbtTagCompound.setByte("Fuse", (byte)this.fuse);
    }
    
    @Override
    protected final void readEntityFromNBT(final NBTTagCompound nbtTagCompound) {
        this.fuse = nbtTagCompound.getByte("Fuse");
    }
    
    @Override
    protected final String getEntityString() {
        return "PrimedTnt";
    }
    
    @Override
    public final double getShadowSize() {
        return 0.0f;
    }
}
