package net.minecraft.game.level;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Arrays;
import java.util.Iterator;

import net.minecraft.game.block.Block;

public class Chunk {
	public static final int WIDTH=16;
	public static final int HEIGHT=16;
	public long timeSinceLoaded = System.currentTimeMillis();
	private byte[][][] data;
	private byte[] metadata;
	public Chunk(byte[][][] data) {
		this.data = data;
		this.metadata = new byte[(WIDTH * WIDTH * HEIGHT) >> 1];
	}
	public Chunk(int xPos,int zPos) {
		this.data = new byte[WIDTH][HEIGHT][WIDTH];
		this.metadata = new byte[(WIDTH * WIDTH * HEIGHT) >> 1];
		for(int x=0;x<WIDTH;x++)
			for(int y=0;y<HEIGHT;y++)
				for(int z=0;z<WIDTH;z++)
					this.data[x][y][z] = (byte) (genBlock(y));
	}
	
	private int genBlock(int y) {
		// TODO Auto-generated method stub
		switch(y) {
		case 0:
//			return Block.bedrock.blockID;
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
			return Block.stone.blockID;
		case 6:
		case 7:
		case 8:
			return Block.dirt.blockID;
		case 9:
			return Block.grass.blockID;
		}
		return 0;
	}
	
	public int getBlock(int x,int y,int z) {
		if(x<0||y<0||z<0||x>WIDTH||y>HEIGHT||z>WIDTH) 
			return 0;
		timeSinceLoaded = System.currentTimeMillis();
		return this.data[x][y][z]&0xFF;
	}
	public boolean setBlock(int x,int y,int z,int id) {
		if(x<0||y<0||z<0||x>WIDTH||y>HEIGHT||z>WIDTH) 
			return false;
		timeSinceLoaded = System.currentTimeMillis();
		this.data[x][y][z] = (byte)id;
		return true;
	}
	
	public int getBlockData(int x, int y, int z) {
		int pos = (x << 8 | z << 4 | y) >> 1;
		if(((x << 8 | z << 4 | y) & 0x01) == 0x01) {
			return (this.metadata[pos] & 0xF0) >> 4;
		} else {
			return (this.metadata[pos] & 0x0F);
		}
	}
	public void setBlockData(int x, int y, int z, int data) {
		int pos = (x << 8 | z << 4 | y) >> 1;
		if(((x << 8 | z << 4 | y) & 0x01) == 0x10) {
			this.metadata[pos] &= 0x0F;
			this.metadata[pos] |= (data & 0x0F) << 4;
		} else {
			this.metadata[pos] &= 0xF0;
			this.metadata[pos] |= (data & 0x0F);
		}
	}
	
	public void writeTo(OutputStream out) throws IOException {
		for (byte[][] bs : data)
			for (byte[] bs2 : bs)
				out.write(bs2);
	}
}
