package net.minecraft.game.level;

import java.util.Iterator;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.zip.GZIPOutputStream;
import com.nbt.NBTBase;
import net.minecraft.game.block.Block;
import java.io.OutputStream;
import net.minecraft.game.entity.other.EntityPainting;
import net.minecraft.game.entity.other.EntityItem;
import net.minecraft.game.entity.monster.EntityGiant;
import net.minecraft.game.entity.monster.EntityZombie;
import net.minecraft.game.entity.monster.EntitySpider;
import net.minecraft.game.entity.monster.EntitySkeleton;
import net.minecraft.game.entity.monster.EntityCreeper;
import net.minecraft.game.entity.animal.EntitySheep;
import net.minecraft.game.entity.animal.EntityPig;
import net.minecraft.game.block.tileentity.TileEntity;
import net.minecraft.game.entity.Entity;
import com.nbt.NBTTagList;
import net.minecraft.game.block.tileentity.TileEntityFurnace;
import net.minecraft.game.block.tileentity.TileEntityChest;
import com.nbt.NBTTagCompound;
import com.nbt.NBTTagShort;
import net.minecraft.client.gui.GuiLevelLoading;
import java.io.InputStream;
import net.minecraft.client.gui.GuiLoading;

public abstract class LevelLoader {
    private GuiLoading guiLoading;
    
    public LevelLoader(final GuiLoading guiLoading) {
        this.guiLoading = guiLoading;
    }
    
    public final World load(InputStream var_1_29) throws IOException {
        if (this.guiLoading != null) {
            this.guiLoading.setTitle("Loading level");
        }
        if (this.guiLoading != null) {
            this.guiLoading.setText("Reading..");
        }
		NBTTagCompound var13 = GuiLevelLoading.writeLevelTags(var_1_29);
        final NBTTagCompound compoundTag = var13.getCompoundTag("About");
        final NBTTagCompound compoundTag2 = var13.getCompoundTag("Map");
        final NBTTagCompound compoundTag3 = var13.getCompoundTag("Environment");
        final NBTTagList tagList = var13.getTagList("Entities");
        int n = compoundTag2.getShort("Width");
        int short1 = compoundTag2.getShort("Length");
        final short short2 = compoundTag2.getShort("Height");
        final World world = new World();
        if (this.guiLoading != null) {
            this.guiLoading.setText("Preparing level..");
        }
        final NBTTagList tagList2 = compoundTag2.getTagList("Spawn");
        world.xSpawn = ((NBTTagShort)tagList2.tagAt(0)).shortValue;
        world.ySpawn = ((NBTTagShort)tagList2.tagAt(1)).shortValue;
        world.zSpawn = ((NBTTagShort)tagList2.tagAt(2)).shortValue;
        world.authorName = compoundTag.getString("Author");
        world.name = compoundTag.getString("Name");
        world.createTime = compoundTag.getLong("CreatedOn");
        world.cloudColor = compoundTag3.getInteger("CloudColor");
        world.skyColor = compoundTag3.getInteger("SkyColor");
        world.fogColor = compoundTag3.getInteger("FogColor");
        world.skyBrightness = compoundTag3.getByte("SkyBrightness");
        if (world.skyBrightness < 0) {
            world.skyBrightness = 0;
        }
        if (world.skyBrightness > 16) {
            world.skyBrightness = world.skyBrightness * 15 / 100;
        }
        world.cloudHeight = compoundTag3.getShort("CloudHeight");
        world.getGroundLevel = compoundTag3.getShort("SurroundingGroundHeight");
        world.getWaterLevel = compoundTag3.getShort("SurroundingWaterHeight");
        world.defaultFluid = compoundTag3.getByte("SurroundingWaterType");
        world.timeOfDay = compoundTag3.getShort("TimeOfDay");
        world.skylightSubtracted = world.getSkyBrightness();
        world.generate(n, short2, short1, compoundTag2.getByteArray("Blocks"), compoundTag2.getByteArray("Data"));
        if (this.guiLoading != null) {
            this.guiLoading.setText("Preparing entities..");
        }
        for (int i = 0; i < tagList.tagCount(); ++i) {
            try {
                final NBTTagCompound nbtTagCompound;
                final String string = (nbtTagCompound = (NBTTagCompound)tagList.tagAt(i)).getString("id");
                final Entity loadEntity;
                if ((loadEntity = this.loadEntity(world, string)) != null) {
                    loadEntity.readFromNBT(nbtTagCompound);
                    world.entityJoinedWorld(loadEntity);
                }
                else {
                    System.out.println("Skipping unknown entity id \"" + string + "\"");
                }
            }
            catch (Exception ex) {
                System.out.println("Error reading entity");
                ex.printStackTrace();
            }
        }
        final NBTTagList tagList3 = var13.getTagList("TileEntities");
        for (int j = 0; j < tagList3.tagCount(); ++j) {
            try {
                final NBTTagCompound nbtTagCompound2;
                n = (nbtTagCompound2 = (NBTTagCompound)tagList3.tagAt(j)).getInteger("Pos");
                final String string2;
                final String s;
                Object o;
                final TileEntity tileEntity = (TileEntity)((s = (string2 = nbtTagCompound2.getString("id"))).equals("Chest") ? (o = new TileEntityChest()) : (s.equals("Furnace") ? (o = new TileEntityFurnace()) : (o = null)));
                final TileEntity tileEntity2 = (TileEntity)o;
                if (tileEntity != null) {
                    final int integer1 = n % 1024;
                    short1 = (n >> 10) % 1024;
                    n = (n >> 20) % 1024;
                    tileEntity2.readFromNBT(nbtTagCompound2);
                    world.setBlockTileEntity(integer1, short1, n, tileEntity2);
                }
                else {
                    System.out.println("Skipping unknown tile entity id \"" + string2 + "\"");
                }
            }
            catch (Exception ex2) {
                System.out.println("Error reading tileentity");
                ex2.printStackTrace();
            }
        }
        return world;
    }
    
    protected Entity loadEntity(final World world, final String playerName) {
        if (playerName.equals("Pig")) {
            return new EntityPig(world);
        }
        if (playerName.equals("Sheep")) {
            return new EntitySheep(world);
        }
        if (playerName.equals("Creeper")) {
            return new EntityCreeper(world);
        }
        if (playerName.equals("Skeleton")) {
            return new EntitySkeleton(world);
        }
        if (playerName.equals("Spider")) {
            return new EntitySpider(world);
        }
        if (playerName.equals("Zombie")) {
            return new EntityZombie(world);
        }
        if (playerName.equals("Giant")) {
            return new EntityGiant(world);
        }
        if (playerName.equals("Item")) {
            return new EntityItem(world);
        }
        if (playerName.equals("Painting")) {
            return new EntityPainting(world);
        }
        return null;
    }
    
    public final void save(final World world, OutputStream outputStream) throws IOException {
        
    }
}
