package net.minecraft.game.level.generator;

import java.util.Random;

import net.minecraft.game.block.Block;
import net.minecraft.game.level.Chunk;
import net.minecraft.game.level.generator.noise.NoiseGeneratorCombined;
import net.minecraft.game.level.generator.noise.NoiseGeneratorOctaves;
import net.minecraft.game.level.generator.noise.OctaveNoiseInfdev0227;

public class ChunkGen2 {
	private Random rand;
	private long seed;
	private OctaveNoiseInfdev0227 noise1;
	private OctaveNoiseInfdev0227 noise2;
	private OctaveNoiseInfdev0227 noise3;
	private OctaveNoiseInfdev0227 noise4;
	private OctaveNoiseInfdev0227 noise5;
	private OctaveNoiseInfdev0227 noise6;
	public static final int HEIGHT = 128;
	
	public ChunkGen2() {
		reset(System.nanoTime());
	}
	
	public void reset(long seed) {
		this.seed = seed;
		rand = new Random(seed);
		noise1 = new OctaveNoiseInfdev0227(rand, 16);
		noise2 = new OctaveNoiseInfdev0227(rand, 16);
		noise3 = new OctaveNoiseInfdev0227(rand, 8);
		noise4 = new OctaveNoiseInfdev0227(rand, 4);
		noise5 = new OctaveNoiseInfdev0227(rand, 4);
		noise6 = new OctaveNoiseInfdev0227(rand, 5);
	}
	
	public long getSeed() {
		return seed;
	}

	public void setSeed(long seed) {
		reset(seed);
	}
	
	public Chunk[] provideChunk(long chunkX,long chunkZ) {
		//Random bedrockRandom = new Random((chunkX<<32)^chunkZ);
		long posX = chunkX*16,
			 posZ = chunkZ*16;
		Chunk[] chunks = new Chunk[HEIGHT / Chunk.HEIGHT];
		byte[][][][] data = new byte[chunks.length][Chunk.WIDTH][Chunk.HEIGHT][Chunk.WIDTH];
		for(long x=posX;x<posX+Chunk.WIDTH;x++)
			for(long z=posZ;z<posZ+Chunk.WIDTH;z++) {
				long var1 = x / 1024;
				long var2 = z / 1024;
				float var3 = (float)(noise1.compute(x * 32D, 0.0D, z * 32D) - noise2.compute(x * 64D, 0.0D, z * 64D)) / 512F / 4F;
				float var4 = (float)noise5.compute(x / 4F, z / 4F);
				float var5 = (float)noise6.compute(x / 8F, z / 8F) / 8F;
				var4 = (float) (var4 > 0F ? noise3.compute(x * .25714284D * 2, z * .25714284D * 2) * var5 / 4F : noise4.compute(x * .25714284D, z * .25714284D)) * var5;
				int height = (int)(var3+64+var4);
				if(noise5.compute(x, z)<0F) {
					height = height / 2 << 1;
					if(noise5.compute(x / 5, z / 5) < 0F)
						height++;
				}
				
				for (int i = 0; i < chunks.length; i++) {
					for(int j=0;j<Chunk.HEIGHT;j++) {
						int y=i*Chunk.HEIGHT+j;
						int block = 0;
						if((x==0||z==0)&&y<=height+2) 
							block = Block.obsidian.blockID;
						else if (y==height+1 && height >= 64 && Math.random() < 0.02) 
							block = 37;
						else if (y == height && y >= 64)
							block = Block.grass.blockID;
						else if(y<=height-2)
							block = Block.stone.blockID;
						else if(y<=height) 
							block = Block.dirt.blockID;
						else if(y<=64)
							block = Block.waterStill.blockID;
						rand.setSeed(var1+var2*13871);
						long pyramidX = (var1<<10) + HEIGHT + rand.nextInt(512);
						long pyramidZ = (var2<<10) + HEIGHT + rand.nextInt(512);
						long pyramidHeight1 = x - pyramidX;
						long pyramidHeight2 = z - pyramidZ;
						if(pyramidHeight1 < 0) {
							pyramidHeight1 *= -1;
						}
						if(pyramidHeight2 < 0) {
							pyramidHeight2 *= -1;
						}
						if(pyramidHeight2 > pyramidHeight1) {
							pyramidHeight1 = pyramidHeight2;
						}
						if((pyramidHeight1 = 127 - pyramidHeight1) == 255) {
							pyramidHeight1 = 1;
						}
						if(pyramidHeight1 < height) {
							pyramidHeight1 = height;
						}
						if(y <= pyramidHeight1 && (block==0||block==Block.waterStill.blockID)) {
							block = Block.brick.blockID;
						}
						data[i][(int) (x - posX)][j][(int) (z - posZ)] = (byte) block;
					}
				}
			}
		for(int i=0;i<chunks.length;i++) {
			chunks[i] = new Chunk(data[i]);
		}
		
		return chunks;
	}
}
