package net.minecraft.game.level.generator;

import java.util.ArrayList;
import net.minecraft.game.block.BlockFlower;
import net.minecraft.game.level.MobSpawner;
import net.minecraft.game.MathHelper;
import net.minecraft.game.block.Block;
import net.minecraft.game.level.generator.noise.NoiseGeneratorCombined;
import net.minecraft.game.level.generator.noise.NoiseGeneratorOctaves;
import net.minecraft.game.level.World;
import java.util.Random;
import net.minecraft.client.gui.GuiLoading;

public final class LevelGenerator {
    private GuiLoading guiLoading;
    private int width;
    private int depth;
    private int height;
    private Random rand;
    private byte[] blocksByteArray;
    private int waterLevel;
    private int groundLevel;
    public boolean islandGen;
    public boolean doubleingGen;
    public boolean flatGen;
    public int levelType;
    private int phaseBar;
    private int phases;
    private double phaseBareLength;
    private int[] blocksIntArray;
    
    public LevelGenerator(final GuiLoading guiLoading) {
        this.rand = new Random();
        this.islandGen = false;
        this.doubleingGen = false;
        this.flatGen = false;
        this.phaseBareLength = 0.0f;
        this.blocksIntArray = new int[1048576];
        this.guiLoading = guiLoading;
    }
    
    public final World generate(final String authorName, int width, final int depth, final int height) {
        int i = 1;
        if (this.doubleingGen) {
            i = (height - 64) / 48 + 1;
        }
        this.phases = 13 + i * 4;
        this.guiLoading.setTitle("Generating level");
        final World world;
        (world = new World()).getWaterLevel = this.waterLevel;
        world.getGroundLevel = this.groundLevel;
        this.width = width;
        this.depth = depth;
        this.height = height;
        this.blocksByteArray = new byte[width * depth * height];
        for (int j = 0; j < i; ++j) {
            this.waterLevel = height - 32 - j * 48;
            this.groundLevel = this.waterLevel - 2;
            int[] array;
            if (this.flatGen) {
                array = new int[width * depth];
                for (int k = 0; k < array.length; ++k) {
                    array[k] = 0;
                }
                this.loadingBar();
                this.loadingBar();
            }
            else {
                this.guiLoading.setText("Raising..");
                this.loadingBar();
                final NoiseGeneratorCombined noiseGeneratorCombined = new NoiseGeneratorCombined(new NoiseGeneratorOctaves(this.rand, 8), new NoiseGeneratorOctaves(this.rand, 8));
                final NoiseGeneratorCombined noiseGeneratorCombined2 = new NoiseGeneratorCombined(new NoiseGeneratorOctaves(this.rand, 8), new NoiseGeneratorOctaves(this.rand, 8));
                final NoiseGeneratorOctaves noiseGeneratorOctaves = new NoiseGeneratorOctaves(this.rand, 6);
                final NoiseGeneratorOctaves noiseGeneratorOctaves2 = new NoiseGeneratorOctaves(this.rand, 2);
                final int[] array2 = new int[this.width * this.depth];
                for (int l = 0; l < this.width; ++l) {
                    final double abs = Math.abs((l / (this.width - 1.0) - 0.5) * 2.0);
                    this.setNextPhase(l * 100.0f / (this.width - 1));
                    for (int n = 0; n < this.depth; ++n) {
                        final double abs2 = Math.abs((n / (this.depth - 1.0) - 0.5) * 2.0);
                        final double n2 = noiseGeneratorCombined.NoiseGenerator(l * 1.3f, n * 1.3f) / 6.0 - 4.0;
                        double n3 = noiseGeneratorCombined2.NoiseGenerator(l * 1.3f, n * 1.3f) / 5.0 + 10.0 - 4.0;
                        final double n4;
                        if ((n4 = noiseGeneratorOctaves.NoiseGenerator(l, n) / 8.0) > 0.0) {
                            n3 = n2;
                        }
                        double n5 = Math.max(n2, n3) / 2.0;
                        if (this.islandGen) {
                            double n6 = Math.sqrt(abs * abs + abs2 * abs2) * 1.2000000476837158;
                            if ((n6 = Math.max(n6 = Math.min(n6, noiseGeneratorOctaves2.NoiseGenerator(l * 0.05f, n * 0.05f) / 4.0 + 1.0), Math.max(abs, abs2))) > 1.0) {
                                n6 = 1.0;
                            }
                            if (n6 < 0.0) {
                                n6 = 0.0;
                            }
                            n6 *= n6;
                            if ((n5 = n5 * (1.0 - n6) - n6 * 10.0 + 5.0) < 0.0) {
                                n5 -= n5 * n5 * 0.20000000298023224;
                            }
                        }
                        else if (n5 < 0.0) {
                            n5 *= 0.8;
                        }
                        array2[l + n * this.width] = (int)n5;
                    }
                }
                array = array2;
                this.guiLoading.setText("Eroding..");
                this.loadingBar();
                final int[] array3 = array;
                final NoiseGeneratorCombined noiseGeneratorCombined3 = new NoiseGeneratorCombined(new NoiseGeneratorOctaves(this.rand, 8), new NoiseGeneratorOctaves(this.rand, 8));
                final NoiseGeneratorCombined noiseGeneratorCombined4 = new NoiseGeneratorCombined(new NoiseGeneratorOctaves(this.rand, 8), new NoiseGeneratorOctaves(this.rand, 8));
                for (int n7 = 0; n7 < this.width; ++n7) {
                    this.setNextPhase(n7 * 100.0f / (this.width - 1));
                    for (int height2 = 0; height2 < this.depth; ++height2) {
                        final double n8 = noiseGeneratorCombined3.NoiseGenerator(n7 << 1, height2 << 1) / 8.0;
                        final int n9 = (noiseGeneratorCombined4.NoiseGenerator(n7 << 1, height2 << 1) > 0.0) ? 1 : 0;
                        if (n8 > 2.0) {
                            int n10 = (((n10 = array3[n7 + height2 * this.width]) - n9) / 2 << 1) + n9;
                            array3[n7 + height2 * this.width] = n10;
                        }
                    }
                }
            }
            this.guiLoading.setText("Soiling..");
            this.loadingBar();
            final int[] array4 = array;
            int n11 = this.width;
            int n12 = this.depth;
            int n7 = this.height;
            final NoiseGeneratorOctaves noiseGeneratorOctaves3 = new NoiseGeneratorOctaves(this.rand, 8);
            final NoiseGeneratorOctaves noiseGeneratorOctaves4 = new NoiseGeneratorOctaves(this.rand, 8);
            for (int n13 = 0; n13 < n11; ++n13) {
                final double abs3 = Math.abs((n13 / (n11 - 1.0) - 0.5) * 2.0);
                this.setNextPhase(n13 * 100.0f / (n11 - 1));
                for (int n14 = 0; n14 < n12; ++n14) {
                    double max = (max = Math.max(abs3, Math.abs((n14 / (n12 - 1.0) - 0.5) * 2.0))) * max * max;
                    final int n16;
                    final int n15 = (n16 = array4[n13 + n14 * n11] + this.waterLevel) + ((int)(noiseGeneratorOctaves3.NoiseGenerator(n13, n14) / 24.0) - 4);
                    array4[n13 + n14 * n11] = Math.max(n16, n15);
                    if (array4[n13 + n14 * n11] > n7 - 2) {
                        array4[n13 + n14 * n11] = n7 - 2;
                    }
                    if (array4[n13 + n14 * n11] <= 0) {
                        array4[n13 + n14 * n11] = 1;
                    }
                    final double n4;
                    int height3;
                    if ((height3 = (int)((height3 = (int)(Math.sqrt(Math.abs(n4 = noiseGeneratorOctaves4.NoiseGenerator(n13 * 2.3, n14 * 2.3) / 24.0)) * Math.signum(n4) * 20.0) + this.waterLevel) * (1.0 - max) + max * this.height)) > this.waterLevel) {
                        height3 = this.height;
                    }
                    for (int n17 = 0; n17 < n7; ++n17) {
                        final int n18 = (n17 * this.depth + n14) * this.width + n13;
                        int n19 = 0;
                        if (n17 <= n16) {
                            n19 = Block.dirt.blockID;
                        }
                        if (n17 <= n15) {
                            n19 = Block.stone.blockID;
                        }
                        if (this.doubleingGen && n17 < height3) {
                            n19 = 0;
                        }
                        if (this.blocksByteArray[n18] == 0) {
                            this.blocksByteArray[n18] = (byte)n19;
                        }
                    }
                }
            }
            this.guiLoading.setText("Growing..");
            this.loadingBar();
            final int[] array5 = array;
            n11 = this.width;
            n12 = this.depth;
            final NoiseGeneratorOctaves noiseGeneratorOctaves5 = new NoiseGeneratorOctaves(this.rand, 8);
            final NoiseGeneratorOctaves noiseGeneratorOctaves6 = new NoiseGeneratorOctaves(this.rand, 8);
            int n20 = this.waterLevel - 1;
            if (this.levelType == 2) {
                n20 += 2;
            }
            for (int n13 = 0; n13 < n11; ++n13) {
                this.setNextPhase(n13 * 100.0f / (n11 - 1));
                for (int n21 = 0; n21 < n12; ++n21) {
                    boolean b = noiseGeneratorOctaves5.NoiseGenerator(n13, n21) > 8.0;
                    if (this.islandGen) {
                        b = (noiseGeneratorOctaves5.NoiseGenerator(n13, n21) > -8.0);
                    }
                    if (this.levelType == 2) {
                        b = (noiseGeneratorOctaves5.NoiseGenerator(n13, n21) > -32.0);
                    }
                    final boolean b2 = noiseGeneratorOctaves6.NoiseGenerator(n13, n21) > 12.0;
                    if (this.levelType == 1 || this.levelType == 3) {
                        b = (noiseGeneratorOctaves5.NoiseGenerator(n13, n21) > -8.0);
                    }
                    final int n;
                    final int n22 = ((n = array5[n13 + n21 * n11]) * this.depth + n21) * this.width + n13;
                    final int n23;
                    if (((n23 = (this.blocksByteArray[((n + 1) * this.depth + n21) * this.width + n13] & 0xFF)) == Block.waterMoving.blockID || n23 == Block.waterStill.blockID || n23 == 0) && n <= this.waterLevel - 1 && b2) {
                        this.blocksByteArray[n22] = (byte)Block.gravel.blockID;
                    }
                    if (n23 == 0) {
                        int n24 = -1;
                        if (n <= n20 && b) {
                            n24 = Block.sand.blockID;
                            if (this.levelType == 1) {
                                n24 = Block.grass.blockID;
                            }
                        }
                        if (this.blocksByteArray[n22] != 0 && n24 > 0) {
                            this.blocksByteArray[n22] = (byte)n24;
                        }
                    }
                }
            }
        }
        this.guiLoading.setText("Carving..");
        this.loadingBar();
        int n12 = this.width;
        int n7 = this.depth;
        int height2 = this.height;
        int n20 = n12 * n7 * height2 / 256 / 64 << 1;
        for (int n13 = 0; n13 < n20; ++n13) {
            this.setNextPhase(n13 * 100.0f / (n20 - 1));
            double n25 = this.rand.nextDouble() * n12;
            double n26 = this.rand.nextDouble() * height2;
            double n27 = this.rand.nextDouble() * n7;
            final int n = (int)((this.rand.nextDouble() + this.rand.nextDouble()) * 200.0f);
            double n28 = this.rand.nextDouble() * 3.1415927f * 2.0f;
            double n29 = 0.0f;
            double double1 = this.rand.nextDouble() * 3.1415927f * 2.0f;
            double n30 = 0.0f;
            final double n31 = this.rand.nextDouble() * this.rand.nextDouble();
            for (int n15 = 0; n15 < n; ++n15) {
                n25 += MathHelper.sin(n28) * MathHelper.cos(double1);
                n27 += MathHelper.cos(n28) * MathHelper.cos(double1);
                n26 += MathHelper.sin(double1);
                n28 += n29 * 0.2f;
                n29 = (n29 *= 0.9f) + (this.rand.nextDouble() - this.rand.nextDouble());
                double1 = (double1 += n30 * 0.5f) * 0.5f;
                n30 = (n30 *= 0.75f) + (this.rand.nextDouble() - this.rand.nextDouble());
                if (this.rand.nextDouble() >= 0.25f) {
                    final double n32 = n25 + (this.rand.nextDouble() * 4.0f - 2.0f) * 0.2f;
                    final double n33 = n26 + (this.rand.nextDouble() * 4.0f - 2.0f) * 0.2f;
                    final double n34 = n27 + (this.rand.nextDouble() * 4.0f - 2.0f) * 0.2f;
                    int j;
                    double n35;
                    int n36;
                    int n37;
                    double n38;
                    double n39;
                    double n40;
                    for (n35 = MathHelper.sin(n15 * 3.1415927f / n) * (1.2f + ((this.height - n33) / this.height * 3.5f + 1.0f) * n31), i = (int)(n32 - n35); i <= (int)(n32 + n35); ++i) {
                        for (n36 = (int)(n33 - n35); n36 <= (int)(n33 + n35); ++n36) {
                            for (n37 = (int)(n34 - n35); n37 <= (int)(n34 + n35); ++n37) {
                                n38 = i - n32;
                                n39 = n36 - n33;
                                n40 = n37 - n34;
                                if ((n38 = n38 * n38 + n39 * n39 * 2.0f + n40 * n40) < n35 * n35 && i > 0 && n36 > 0 && n37 > 0 && i < this.width - 1 && n36 < this.height - 1 && n37 < this.depth - 1) {
                                    j = (n36 * this.depth + n37) * this.width + i;
                                    if (this.blocksByteArray[j] == Block.stone.blockID) {
                                        this.blocksByteArray[j] = 0;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        int j = this.populateOre(Block.oreCoal.blockID, 1000, 10, (height << 2) / 5);
        final int populateOre = this.populateOre(Block.oreIron.blockID, 800, 8, height * 3 / 5);
        int k = this.populateOre(Block.oreGold.blockID, 500, 6, (height << 1) / 5);
        i = this.populateOre(Block.oreDiamond.blockID, 800, 2, height / 5);
        System.out.println(new StringBuilder().append("Coal: ").append(j).append(", Iron: ").append(populateOre).append(", Gold: ").append(k).append(", Diamond: ").append(i).toString());
        this.guiLoading.setText("Melting..");
        this.loadingBar();
        this.lavaGen();
        world.cloudHeight = height + 2;
        if (this.doubleingGen) {
            this.groundLevel = -128;
            this.waterLevel = this.groundLevel + 1;
            world.cloudHeight = -16;
        }
        else if (!this.islandGen) {
            this.groundLevel = this.waterLevel + 1;
            this.waterLevel = this.groundLevel - 16;
        }
        else {
            this.groundLevel = this.waterLevel - 9;
        }
        this.guiLoading.setText("Watering..");
        this.loadingBar();
        this.liquidThemeSpawner();
        if (!this.doubleingGen) {
            i = Block.waterStill.blockID;
            if (this.levelType == 1) {
                i = Block.lavaStill.blockID;
            }
            for (int j1 = 0; j1 < width; ++j1) {
                this.flood(j1, this.waterLevel - 1, 0, 0, i);
                this.flood(j1, this.waterLevel - 1, depth - 1, 0, i);
            }
            for (int j2 = 0; j2 < depth; ++j2) {
                this.flood(width - 1, this.waterLevel - 1, j2, 0, i);
                this.flood(0, this.waterLevel - 1, j2, 0, i);
            }
        }
        if (this.levelType == 0) {
            world.skyColor = 10079487;
            world.fogColor = 16777215;
            world.cloudColor = 16777215;
        }
        if (this.levelType == 1) {
            world.cloudColor = 2164736;
            world.fogColor = 1049600;
            world.skyColor = 1049600;
            final World world2 = world;
            final World world3 = world;
            final int n41 = 7;
            world3.skyBrightness = n41;
            world2.skylightSubtracted = n41;
            world.defaultFluid = Block.lavaMoving.blockID;
            if (this.doubleingGen) {
                world.cloudHeight = height + 2;
                this.waterLevel = -16;
            }
        }
        if (this.levelType == 2) {
            world.skyColor = 13033215;
            world.fogColor = 13033215;
            world.cloudColor = 15658751;
            final World world4 = world;
            final World world5 = world;
            final int n42 = 15;
            world5.skyBrightness = n42;
            world4.skylightSubtracted = n42;
            world.skyBrightness = 16;
            world.cloudHeight = height + 64;
        }
        if (this.levelType == 3) {
            world.skyColor = 7699847;
            world.fogColor = 5069403;
            world.cloudColor = 5069403;
            final World world6 = world;
            final World world7 = world;
            final int n43 = 12;
            world7.skyBrightness = n43;
            world6.skylightSubtracted = n43;
        }
        world.getWaterLevel = this.waterLevel;
        world.getGroundLevel = this.groundLevel;
        this.guiLoading.setText("Assembling..");
        this.loadingBar();
        this.setNextPhase(0.0f);
        world.generate(width, height, depth, this.blocksByteArray, null);
        this.guiLoading.setText("Building..");
        this.loadingBar();
        this.setNextPhase(0.0f);
        world.findSpawn();
        generateHouse(world);
        this.guiLoading.setText("Planting..");
        this.loadingBar();
        if (this.levelType != 1) {
            this.growGrassOnDirt(world);
        }
        this.loadingBar();
        this.treeGenerator(world);
        if (this.levelType == 3) {
            for (i = 0; i < 50; ++i) {
                this.treeGenerator(world);
            }
        }
        i = 100;
        if (this.levelType == 2) {
            i = 1000;
        }
        this.loadingBar();
        this.populateFlowersAndMushrooms(world, Block.plantYellow, i);
        this.loadingBar();
        this.populateFlowersAndMushrooms(world, Block.plantRed, i);
        this.loadingBar();
        this.populateFlowersAndMushrooms(world, Block.mushroomBrown, 50);
        this.loadingBar();
        this.populateFlowersAndMushrooms(world, Block.mushroomRed, 50);
        this.guiLoading.setText("Lighting..");
        this.loadingBar();
        for (j = 0; j < 10000; ++j) {
            this.setNextPhase((double)(j * 100 / 10000));
            world.tickEntities();
        }
        this.guiLoading.setText("Spawning..");
        this.loadingBar();
        final MobSpawner mobSpawner = new MobSpawner(world);
        for (width = 0; width < 1000; ++width) {
            this.setNextPhase(width * 100.0f / 999.0f);
            mobSpawner.spawn();
        }
        world.createTime = System.currentTimeMillis();
        world.authorName = authorName;
        world.name = "A Nice World";
        if (this.phaseBar != this.phases) {
            throw new IllegalStateException(new StringBuilder().append("Wrong number of phases! Wanted ").append(this.phases).append(", got ").append(this.phaseBar).toString());
        }
        return world;
    }
    
    private static void generateHouse(final World world) {
        final int xSpawn = world.xSpawn;
        final int ySpawn = world.ySpawn;
        final int zSpawn = world.zSpawn;
        for (int i = xSpawn - 3; i <= xSpawn + 3; ++i) {
            for (int j = ySpawn - 2; j <= ySpawn + 2; ++j) {
                for (int k = zSpawn - 3; k <= zSpawn + 3; ++k) {
                    int integer4 = (j < ySpawn - 1) ? Block.obsidian.blockID : 0;
                    if (i == xSpawn - 3 || k == zSpawn - 3 || i == xSpawn + 3 || k == zSpawn + 3 || j == ySpawn - 2 || j == ySpawn + 2) {
                        integer4 = Block.stone.blockID;
                        if (j >= ySpawn - 1) {
                            integer4 = Block.planks.blockID;
                        }
                    }
                    if (k == zSpawn - 3 && i == xSpawn && j >= ySpawn - 1 && j <= ySpawn) {
                        integer4 = 0;
                    }
                    world.setBlockWithNotify(i, j, k, integer4);
                }
            }
        }
        world.setBlockWithNotify(xSpawn - 3 + 1, ySpawn, zSpawn, Block.torchWood.blockID);
        world.setBlockWithNotify(xSpawn + 3 - 1, ySpawn, zSpawn, Block.torchWood.blockID);
    }
    
    private void growGrassOnDirt(final World world) {
        for (int i = 0; i < this.width; ++i) {
            this.setNextPhase(i * 100.0f / (this.width - 1));
            for (int j = 0; j < this.height; ++j) {
                for (int k = 0; k < this.depth; ++k) {
                    if (world.getBlockId(i, j, k) == Block.dirt.blockID && world.getBlockLightValue(i, j + 1, k) >= 4 && !world.getBlockMaterial(i, j + 1, k).getCanBlockGrass()) {
                        world.setTileNoNeighborChange(i, j, k, Block.grass.blockID);
                    }
                }
            }
        }
    }
    
    private void treeGenerator(final World world) {
        for (int n = this.width * this.depth * this.height / 80000, i = 0; i < n; ++i) {
            if (i % 100 == 0) {
                this.setNextPhase(i * 100.0f / (n - 1));
            }
            final int nextInt = this.rand.nextInt(this.width);
            final int nextInt2 = this.rand.nextInt(this.height);
            final int nextInt3 = this.rand.nextInt(this.depth);
            for (int j = 0; j < 25; ++j) {
                int integer1 = nextInt;
                int integer2 = nextInt2;
                int integer3 = nextInt3;
                for (int k = 0; k < 20; ++k) {
                    integer1 += this.rand.nextInt(12) - this.rand.nextInt(12);
                    integer2 += this.rand.nextInt(3) - this.rand.nextInt(6);
                    integer3 += this.rand.nextInt(12) - this.rand.nextInt(12);
                    if (integer1 >= 0 && integer2 >= 0 && integer3 >= 0 && integer1 < this.width && integer2 < this.height && integer3 < this.depth) {
                        world.growTrees(integer1, integer2, integer3);
                    }
                }
            }
        }
    }
    
    private void populateFlowersAndMushrooms(final World world, final BlockFlower flowers, int integer) {
        integer = (int)(this.width * (long)this.depth * this.height * integer / 1600000L);
        for (int i = 0; i < integer; ++i) {
            if (i % 100 == 0) {
                this.setNextPhase(i * 100.0f / (integer - 1));
            }
            final int nextInt = this.rand.nextInt(this.width);
            final int nextInt2 = this.rand.nextInt(this.height);
            final int nextInt3 = this.rand.nextInt(this.depth);
            for (int j = 0; j < 10; ++j) {
                int integer2 = nextInt;
                int integer3 = nextInt2;
                int integer4 = nextInt3;
                for (int k = 0; k < 10; ++k) {
                    integer2 += this.rand.nextInt(4) - this.rand.nextInt(4);
                    integer3 += this.rand.nextInt(2) - this.rand.nextInt(2);
                    integer4 += this.rand.nextInt(4) - this.rand.nextInt(4);
                    if (integer2 >= 0 && integer4 >= 0 && integer3 > 0 && integer2 < this.width && integer4 < this.depth && integer3 < this.height && world.getBlockId(integer2, integer3, integer4) == 0 && flowers.canBlockStay(world, integer2, integer3, integer4)) {
                        world.setBlockWithNotify(integer2, integer3, integer4, flowers.blockID);
                    }
                }
            }
        }
    }
    
    private int populateOre(int integer1, int integer2, final int integer3, final int integer4) {
        int n = 0;
        integer1 = (byte)integer1;
        final int width = this.width;
        final int depth = this.depth;
        final int height = this.height;
        integer2 = width * depth * height / 256 / 64 * integer2 / 100;
        for (int i = 0; i < integer2; ++i) {
            this.setNextPhase(i * 100.0f / (integer2 - 1));
            double n2 = this.rand.nextDouble() * width;
            double n3 = this.rand.nextDouble() * height;
            double n4 = this.rand.nextDouble() * depth;
            if (n3 <= integer4) {
                final int n5 = (int)((this.rand.nextDouble() + this.rand.nextDouble()) * 75.0f * integer3 / 100.0f);
                double n6 = this.rand.nextDouble() * 3.1415927f * 2.0f;
                double n7 = 0.0f;
                double double1 = this.rand.nextDouble() * 3.1415927f * 2.0f;
                double n8 = 0.0f;
                for (int j = 0; j < n5; ++j) {
                    n2 += MathHelper.sin(n6) * MathHelper.cos(double1);
                    n4 += MathHelper.cos(n6) * MathHelper.cos(double1);
                    n3 += MathHelper.sin(double1);
                    n6 += n7 * 0.2f;
                    n7 = (n7 *= 0.9f) + (this.rand.nextDouble() - this.rand.nextDouble());
                    double1 = (double1 += n8 * 0.5f) * 0.5f;
                    n8 = (n8 *= 0.9f) + (this.rand.nextDouble() - this.rand.nextDouble());
                    final double n9 = MathHelper.sin(j * 3.1415927f / n5) * integer3 / 100.0f + 1.0f;
                    for (int k = (int)(n2 - n9); k <= (int)(n2 + n9); ++k) {
                        for (int l = (int)(n3 - n9); l <= (int)(n3 + n9); ++l) {
                            for (int n10 = (int)(n4 - n9); n10 <= (int)(n4 + n9); ++n10) {
                                double n11 = k - n2;
                                final double n12 = l - n3;
                                final double n13 = n10 - n4;
                                if ((n11 = n11 * n11 + n12 * n12 * 2.0f + n13 * n13) < n9 * n9 && k > 0 && l > 0 && n10 > 0 && k < this.width - 1 && l < this.height - 1 && n10 < this.depth - 1) {
                                    final int n14 = (l * this.depth + n10) * this.width + k;
                                    if (this.blocksByteArray[n14] == Block.stone.blockID) {
                                        this.blocksByteArray[n14] = (byte)integer1;
                                        ++n;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return n;
    }
    
    private void liquidThemeSpawner() {
        int integer5 = Block.waterStill.blockID;
        if (this.levelType == 1) {
            integer5 = Block.lavaStill.blockID;
        }
        for (int n = this.width * this.depth * this.height / 1000, i = 0; i < n; ++i) {
            if (i % 100 == 0) {
                this.setNextPhase(i * 100.0f / (n - 1));
            }
            final int nextInt = this.rand.nextInt(this.width);
            final int nextInt2 = this.rand.nextInt(this.height);
            final int nextInt3 = this.rand.nextInt(this.depth);
            if (this.blocksByteArray[(nextInt2 * this.depth + nextInt3) * this.width + nextInt] == 0) {
                final long flood;
                if ((flood = this.flood(nextInt, nextInt2, nextInt3, 0, 255)) > 0L && flood < 640L) {
                    this.flood(nextInt, nextInt2, nextInt3, 255, integer5);
                }
                else {
                    this.flood(nextInt, nextInt2, nextInt3, 255, 0);
                }
            }
        }
        this.setNextPhase(100.0f);
    }
    
    private void loadingBar() {
        ++this.phaseBar;
        this.setNextPhase(this.phaseBareLength = 0.0f);
    }
    
    private void setNextPhase(final double double1) {
        if (double1 < 0.0f) {
            throw new IllegalStateException("Failed to set next phase!");
        }
        this.guiLoading.setProgress((int)((this.phaseBar - 1 + double1 / 100.0f) * 100.0f / this.phases));
    }
    
    private void lavaGen() {
        final int n = this.width * this.depth * this.height / 2000;
        final int groundLevel = this.groundLevel;
        for (int i = 0; i < n; ++i) {
            if (i % 100 == 0) {
                this.setNextPhase(i * 100.0f / (n - 1));
            }
            final int nextInt = this.rand.nextInt(this.width);
            final int min = Math.min(Math.min(this.rand.nextInt(groundLevel), this.rand.nextInt(groundLevel)), Math.min(this.rand.nextInt(groundLevel), this.rand.nextInt(groundLevel)));
            final int nextInt2 = this.rand.nextInt(this.depth);
            if (this.blocksByteArray[(min * this.depth + nextInt2) * this.width + nextInt] == 0) {
                final long flood;
                if ((flood = this.flood(nextInt, min, nextInt2, 0, 255)) > 0L && flood < 640L) {
                    this.flood(nextInt, min, nextInt2, 255, Block.lavaStill.blockID);
                }
                else {
                    this.flood(nextInt, min, nextInt2, 255, 0);
                }
            }
        }
        this.setNextPhase(100.0f);
    }
    
    private long flood(int integer1, int integer2, int integer3, int integer4, final int integer5) {
        final byte b = (byte)integer5;
        integer4 = (byte)integer4;
        final ArrayList list = new ArrayList();
        int i = 0;
        int n = 1;
        int n2 = 1;
        while (1 << n < this.width) {
            ++n;
        }
        while (1 << n2 < this.depth) {
            ++n2;
        }
        final int n3 = this.depth - 1;
        final int n4 = this.width - 1;
        final int[] blocksIntArray = this.blocksIntArray;
        final int n5 = 0;
        ++i;
        blocksIntArray[n5] = ((integer2 << n2) + integer3 << n) + integer1;
        long n6 = 0L;
        integer1 = this.width * this.depth;
        while (i > 0) {
            integer2 = this.blocksIntArray[--i];
            if (i == 0 && list.size() > 0) {
                this.blocksIntArray = (int[])list.remove(list.size() - 1);
                i = this.blocksIntArray.length;
            }
            integer3 = (integer2 >> n & n3);
            final int n7 = integer2 >> n + n2;
            int n8;
            int j;
            for (j = (n8 = (integer2 & n4)); j > 0 && this.blocksByteArray[integer2 - 1] == integer4; --j, --integer2) {}
            while (n8 < this.width && this.blocksByteArray[integer2 + n8 - j] == integer4) {
                ++n8;
            }
            int n9 = integer2 >> n & n3;
            int n10 = integer2 >> n + n2;
            if (integer5 == 255 && (j == 0 || n8 == this.width - 1 || n7 == 0 || n7 == this.height - 1 || integer3 == 0 || integer3 == this.depth - 1)) {
                return -1L;
            }
            if (n9 != integer3 || n10 != n7) {
                System.out.println("Diagonal flood!?");
            }
            n9 = 0;
            n10 = 0;
            int n11 = 0;
            n6 += n8 - j;
            for (j = j; j < n8; ++j) {
                this.blocksByteArray[integer2] = b;
                if (integer3 > 0) {
                    final byte b2;
                    if ((b2 = (byte)((this.blocksByteArray[integer2 - this.width] == integer4) ? 1 : 0)) != 0 && n9 == 0) {
                        if (i == this.blocksIntArray.length) {
                            list.add(this.blocksIntArray);
                            this.blocksIntArray = new int[1048576];
                            i = 0;
                        }
                        this.blocksIntArray[i++] = integer2 - this.width;
                    }
                    n9 = b2;
                }
                if (integer3 < this.depth - 1) {
                    final byte b2;
                    if ((b2 = (byte)((this.blocksByteArray[integer2 + this.width] == integer4) ? 1 : 0)) != 0 && n10 == 0) {
                        if (i == this.blocksIntArray.length) {
                            list.add(this.blocksIntArray);
                            this.blocksIntArray = new int[1048576];
                            i = 0;
                        }
                        this.blocksIntArray[i++] = integer2 + this.width;
                    }
                    n10 = b2;
                }
                if (n7 > 0) {
                    byte b2 = this.blocksByteArray[integer2 - integer1];
                    if ((b == Block.lavaMoving.blockID || b == Block.lavaStill.blockID) && (b2 == Block.waterMoving.blockID || b2 == Block.waterStill.blockID)) {
                        this.blocksByteArray[integer2 - integer1] = (byte)Block.stone.blockID;
                    }
                    if ((b2 = (byte)((b2 == integer4) ? 1 : 0)) != 0 && n11 == 0) {
                        if (i == this.blocksIntArray.length) {
                            list.add(this.blocksIntArray);
                            this.blocksIntArray = new int[1048576];
                            i = 0;
                        }
                        this.blocksIntArray[i++] = integer2 - integer1;
                    }
                    n11 = b2;
                }
                ++integer2;
            }
        }
        return n6;
    }
}
