package net.minecraft.game.level.generator.noise;

import java.util.*;

import net.minecraft.game.MathHelper;

public final class PerlinNoiseInfdev0227 {
	private int[] noise;
	private double offsetX;
	private double offsetY;
	private double offsetZ;

	public PerlinNoiseInfdev0227() {
		this(new Random());
	}

	public PerlinNoiseInfdev0227(Random random) {
		this.noise = new int[512];
		this.offsetX = random.nextDouble() * 256.0;
		this.offsetY = random.nextDouble() * 256.0;
		this.offsetZ = random.nextDouble() * 256.0;
		for (int i = 0; i < 256; ++i) {
			this.noise[i] = i;
		}
		for (int j = 0; j < 256; ++j) {
			final int n = random.nextInt(256 - j) + j;
			final int n2 = this.noise[j];
			this.noise[j] = this.noise[n];
			this.noise[n] = n2;
			this.noise[j + 256] = this.noise[j];
		}
	}

	private double noise(double x, double y, double z) {
		final double n4 = x + this.offsetX;
		final double n5 = y + this.offsetY;
		final double n6 = z + this.offsetZ;
		final int n7 = MathHelper.floor_double(n4) & 0xFF;
		final int n8 = MathHelper.floor_double(n5) & 0xFF;
		final int n9 = MathHelper.floor_double(n6) & 0xFF;
		final double n10 = n4 - MathHelper.floor_double(n4);
		final double n11 = n5 - MathHelper.floor_double(n5);
		final double n12 = n6 - MathHelper.floor_double(n6);
		final double a = fade(n10);
		final double a2 = fade(n11);
		final double a3 = fade(n12);
		final int n13 = this.noise[n7] + n8;
		final int n14 = this.noise[n13] + n9;
		final int n15 = this.noise[n13 + 1] + n9;
		final int n16 = this.noise[n7 + 1] + n8;
		final int n17 = this.noise[n16] + n9;
		final int n18 = this.noise[n16 + 1] + n9;
		return lerp(a3,
				lerp(a2, lerp(a, grad(this.noise[n14], n10, n11, n12), grad(this.noise[n17], n10 - 1.0, n11, n12)),
						lerp(a, grad(this.noise[n15], n10, n11 - 1.0, n12),
								grad(this.noise[n18], n10 - 1.0, n11 - 1.0, n12))),
				lerp(a2, lerp(a, grad(this.noise[n14 + 1], n10, n11, n12 - 1.0),
						grad(this.noise[n17 + 1], n10 - 1.0, n11, n12 - 1.0)),
						lerp(a, grad(this.noise[n15 + 1], n10, n11 - 1.0, n12 - 1.0),
								grad(this.noise[n18 + 1], n10 - 1.0, n11 - 1.0, n12 - 1.0))));
	}

	private static double fade(final double t) {
		return t * t * t * (t * (t * 6.0 - 15.0) + 10.0);
	}

	private static double lerp(final double t, final double a, final double b) {
		return a + t * (b - a);
	}

	private static double grad(int hash, final double x, final double y, final double z) {
		final double n5 = ((hash &= 0xF) < 8) ? x : y;
		final double n6 = (hash < 4) ? y : ((hash == 12 || hash == 14) ? x : z);
		return (((hash & 0x1) == 0x0) ? n5 : (-n5)) + (((hash & 0x2) == 0x0) ? n6 : (-n6));
	}

	public double compute(final double x, final double y) {
		return this.noise(x, y, 0.0);
	}

	public double compute(final double x, final double y, final double z) {
		return this.noise(x, y, z);
	}
}
