package net.minecraft.game.recipe;

import java.util.Map;
import java.util.HashMap;
import java.util.Comparator;
import java.util.Collections;
import net.minecraft.game.block.Block;
import net.minecraft.game.item.ItemStack;
import net.minecraft.game.item.Item;
import java.util.ArrayList;
import java.util.List;

public final class CraftingManager {
    private static final CraftingManager instance;
    private List recipes;
    
    public static final CraftingManager getInstance() {
        return CraftingManager.instance;
    }
    
    private CraftingManager() {
        this.recipes = (List)new ArrayList();
        new RecipesTools().addRecipes(this);
        new RecipesSwords().addRecipe(this);
        new RecipesOreBlocks().addRecipe(this);
        new RecipeBlank();
        this.addRecipe(new ItemStack(Item.bowlSoup), "Y", "X", "#", 'X', Block.mushroomBrown, 'Y', Block.mushroomRed, '#', Item.bowlEmpty);
        this.addRecipe(new ItemStack(Item.bowlSoup), "Y", "X", "#", 'X', Block.mushroomRed, 'Y', Block.mushroomBrown, '#', Item.bowlEmpty);
        new RecipeBlank2();
        this.addRecipe(new ItemStack(Block.chest), "###", "# #", "###", '#', Block.planks);
        this.addRecipe(new ItemStack(Block.stoneOvenIdle), "###", "# #", "###", '#', Block.cobblestone);
        this.addRecipe(new ItemStack(Block.workbench), "##", "##", '#', Block.planks);
        new RecipesArmor().addRecipe(this);
        this.addRecipe(new ItemStack(Block.woolGray, 1), "###", "###", "###", '#', Item.silk);
        this.addRecipe(new ItemStack(Block.tnt, 1), "X#X", "#X#", "X#X", 'X', Item.gunpowder, '#', Block.sand);
        this.addRecipe(new ItemStack(Block.stairSingle, 3), "###", '#', Block.cobblestone);
        this.addRecipe(new ItemStack(Block.planks, 4), "#", '#', Block.wood);
        this.addRecipe(new ItemStack(Item.stick, 4), "#", "#", '#', Block.planks);
        this.addRecipe(new ItemStack(Block.torchWood, 4), "X", "#", 'X', Item.coal, '#', Item.stick);
        this.addRecipe(new ItemStack(Item.bowlEmpty, 4), "# #", " # ", '#', Block.planks);
        this.addRecipe(new ItemStack(Item.flintAndSteel, 1), "A ", " B", 'A', Item.ingotIron, 'B', Item.flint);
        this.addRecipe(new ItemStack(Item.bread, 1), "###", '#', Item.wheat);
        this.addRecipe(new ItemStack(Item.painting, 1), "###", "#X#", "###", '#', Block.planks, 'X', Block.woolGray);
        Collections.sort(this.recipes, (Comparator)new RecipeSorter(this));
        System.out.println(new StringBuilder().append(this.recipes.size()).append(" recipes").toString());
    }
    
    final void addRecipe(final ItemStack itemStack, final Object... arr) {
        String s = "";
        int i = 0;
        int width = 0;
        int height = 0;
        if (arr[0] instanceof String[]) {
            final int n = 0;
            ++i;
            final String[] array = (String[])arr[n];
            for (int j = 0; j < array.length; ++j) {
                final String s2 = array[j];
                ++height;
                width = s2.length();
                s += s2;
            }
        }
        else {
            while (arr[i] instanceof String) {
                final String s3 = (String)arr[i++];
                ++height;
                width = s3.length();
                s += s3;
            }
        }
        final HashMap hashMap = new HashMap();
        while (i < arr.length) {
            final Character c = (Character)arr[i];
            int k = 0;
            if (arr[i + 1] instanceof Item) {
                k = ((Item)arr[i + 1]).shiftedIndex;
            }
            else if (arr[i + 1] instanceof Block) {
                k = ((Block)arr[i + 1]).blockID;
            }
            ((Map)hashMap).put(c, k);
            i += 2;
        }
        final int[] ingredientMap = new int[width * height];
        for (int k = 0; k < width * height; ++k) {
            final char char1 = s.charAt(k);
            if (((Map)hashMap).containsKey(char1)) {
                ingredientMap[k] = (int)((Map)hashMap).get(char1);
            }
            else {
                ingredientMap[k] = -1;
            }
        }
        this.recipes.add(new ShapelessRecipes(width, height, ingredientMap, itemStack));
    }
    
    public final ItemStack findMatchingRecipe(final int[] arr) {
        for (int i = 0; i < this.recipes.size(); ++i) {
            final ShapelessRecipes shapelessRecipes;
            if ((shapelessRecipes = (ShapelessRecipes)this.recipes.get(i)).matchRecipe(arr)) {
                return shapelessRecipes.createResult();
            }
        }
        return null;
    }
    
    static {
        instance = new CraftingManager();
    }
}
